 /****************************************************************************
 * This file is part of Bunny Maze Application                                *
 *                                                                           *
 * Copyright (c) 2011, Eyelead Software. All rights reserved.                *
 * PROPRIETARY AND CONFIDENTIAL                                              *
 *                                                                           *
 * This file is subject to the terms and conditions of the                   *
 * Commercial Software License Agreement by and between                      *
 * Qualcomm Incorporated and Eyelead Software S.A                            *
 * as made and entered on September 30th, 2011                               *
 *****************************************************************************/

#hive 1
#using stdlib gui analytics
#using chartboost

String APPNAME = "bunnymaze";

//main window
Canvas WIN;

//main HUD
Layout HUD;

//loading icons
Widget w_snapdragon,w_eyelead,w_eyelead_logo,w_eyelead_hive3d,w_loadring,w_bunnylogo,w_bunnybg;
Timer time;
int loadperc=0;
int is_ios=0;
int is_kindle=0;

void setupFont(Widget _w) {
	_w.setParam(PARAM_FONTIMAGE,"LINEAR:textures/ui/font.image");
	_w.setparam(PARAM_FONTSIZE,"40,40");
	_w.setparam(PARAM_FONTSPACING,"-0.42");
	_w.setparam(PARAM_TEXTALIGN,"center");
	_w.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);	
	_w.setSize(16,16);
}

void urlencode(string &_s) {
	_s.replace(" ","_");
	_s.replace("$","_");
	_s.replace("&","_");
	_s.replace("+","_");
	_s.replace(",","_");
	_s.replace("/","_");
	_s.replace(":","_");
	_s.replace(";","_");
	_s.replace("=","_");
	_s.replace("?","_");
	_s.replace("@","_");
	_s.replace("<","_");
	_s.replace(">","_");
	_s.replace("#","_");
	_s.replace("%","_");
	_s.replace("(","_");
	_s.replace(")","_");
	_s.replace("{","_");
	_s.replace("}","_");
	_s.replace("|","_");
	_s.replace("^","_");	
	_s.replace("~","_");
	_s.replace("[","_");
	_s.replace("]","_");
	_s.replace("`","_");
}

//initial arguments
void init(String _args)
{
	//set script tag
	setTag("ui.init");

	//sysSetGlobalTimeMultiplier(2.0);
	//sysaudiopitchshift(0.5);

	is_ios=sysGetPlatform().startsWithNoCase("mobile.ios");
	if (!is_ios) {
		is_kindle=sysGetPlatform().containsNoCase("kindle") && sysGetPlatform().containsNoCase("fire");
	}

	//start analytics
	analytics_start("UA-26291053-2");
	
	if (is_ios) {
		//chartboost_start("4f993712f77659fe5b00002e","73f838f598a39923a1c233a227280a8209800a28");
		analytics_trackview("/platform/ios");
	}
	else {
		//chartboost_start("4f97eedbf876598a1900000f","469ac90351dff7cea4c8bfa264e8f20a29e7d2a8");
		analytics_trackview("/platform/android");
	}

	if (debug()) {
		appPropertySet(APPNAME,"showfps",string(_args.containsNoCase("+fps")));
		appPropertySet(APPNAME,"unlocklevels",string(_args.containsNoCase("+unlocklevels")));
		appPropertySet(APPNAME,"unlocksets",string(_args.containsNoCase("+unlocksets")));
		appPropertySet(APPNAME,"billing_ui",string(_args.containsNoCase("+billing")));
	}
	else {
		appPropertySet(APPNAME,"showfps","0");
		appPropertySet(APPNAME,"unlocklevels","0");
		appPropertySet(APPNAME,"unlocksets","0");
		appPropertySet(APPNAME,"billing_ui","0");
	}

	string glrenderer=sysGetGDIInfo("renderer");
	string glvendor=sysGetGDIInfo("vendor");
	glrenderer.sanitizeArg();
	glvendor.sanitizeArg();
	if (glrenderer.startsWithNoCase("adreno")) {
		lists l_adrenostr=lists();
		glrenderer.split(" ",l_adrenostr);
		if (l_adrenostr.num()>1) {
			string adrenover=l_adrenostr.get(l_adrenostr.num()-1);
			adrenover.sanitizeArg();
			if (adrenover.isNumber()) {
				int adrnum=int(adrenover);
				debugLog("Detected Adreno GPU: "+string(adrnum));
				if (adrnum<205) sysSetRenderHQ(0);
			}
		}
	}	
	
	if (int(_args.containsNoCase("+newgame"))) appPropertySet(APPNAME,"state","");
	appPropertyFlush();
	
	//stop ticking
	enableTick(0);
	
	//open window
	WIN=Canvas3D(float3(0,0,0),"color|particle|audio");
	WIN.setPos("0%","0%");
	WIN.setSize("100%","100%");
	WIN.show();	
	
	//remove multiply particles on low quality
	if (sysisrenderhq()==0) {
		WIN.setRenderParam("particle","pass.particle_mul",0.0);
	}

	//attach a HUD to the window
	HUD=Layout(WIN);
	HUD.setBaseSize(800,480,"autofit");
	HUD.show();
	
	//attach a black background to the window
	Widget bgicon=Widget(HUD,UI_PANEL);
	bgicon.enableGhost(1);
	bgicon.setSize("100%","100%");
	bgicon.setParam(PARAM_COLOR,"0,0,0,255");
	
	int is_snapdragon=(sysGetPlatform().startsWithNoCase("mobile.") && glvendor.containsNoCase("qualcomm"));	
	if (sysGetPlatform().startsWithNoCase("desktop.")) is_snapdragon=1;
	appPropertySet(APPNAME,"snapdragon",string(is_snapdragon));	
	analytics_trackview("/device/"+glvendor+"/"+glrenderer);
	
	w_bunnybg=Widget(HUD,UI_ICON);
	w_bunnybg.setSize("100%","100%");
	w_bunnybg.setParam(PARAM_IMAGE,"LINEAR:textures/ui/bluebg.image");
	w_bunnybg.hide();
	
	w_bunnylogo=Widget(HUD,UI_ICON);
	w_bunnylogo.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);	
	w_bunnylogo.setSize(460,230);
	w_bunnylogo.setPos(400,240);
	w_bunnylogo.hide();
	
	w_loadring=Widget(HUD,UI_LOADRING);
	w_loadring.setPivot(PIVOTX_RIGHT,PIVOTY_BOTTOM);
	w_loadring.setSize(32,32);
	w_loadring.setPos(790,470);
	w_loadring.setParam(PARAM_ROTATION_SPEED,"-3");
	w_loadring.setParam(PARAM_FRAMERATE,"16");
	w_loadring.setParam(PARAM_LOADRING_SEGMENTS,"8");
	w_loadring.setParam(PARAM_IMAGE,"LINEAR:textures/ui/loadingprogressring.image");
	w_loadring.hide();
	
	string laststate=appPropertyGet(APPNAME,"state","");
	if (laststate.startsWithNoCase("LOAD:")==0) 
		w_bunnylogo.setParam(PARAM_IMAGE,"LINEAR:textures/ui/logo-"+APPNAME+".image");
	if (laststate.length())
	{
		//goto to last step
		fCall("cbResumeLoaded");
	}
	else {
		w_snapdragon=Widget(HUD,UI_ICON);
		//pick the right resolution
		if (is_snapdragon && glrenderer.containsNoCase("adreno")) {
			w_snapdragon.setParam(PARAM_IMAGE,"LINEAR:textures/ui/dragon_eyes_splash.image");
		}
		else {
			w_snapdragon.setParam(PARAM_IMAGE,"LINEAR:textures/ui/dragon_orig_opt.image");
		}
		w_snapdragon.setSize("100%","100%");
		w_snapdragon.hide();

		w_eyelead=Widget(HUD,UI_PANEL);
		w_eyelead.setParam(PARAM_COLOR,"0,0,0,255");
		w_eyelead.setSize("100%","100%");
		w_eyelead.hide();
		
		w_eyelead_logo=Widget(HUD,UI_ICON);
		w_eyelead_logo.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
		w_eyelead_logo.setParam(PARAM_IMAGE,"LINEAR:textures/ui/ui_loading_eyelead.image");
		w_eyelead_logo.setSize(256,512);
		w_eyelead_logo.setPos(400,240);
		w_eyelead_logo.hide();
		
		w_eyelead_hive3d=Widget(HUD,UI_ICON);
		w_eyelead_hive3d.setPivot(PIVOTX_RIGHT,PIVOTY_BOTTOM);
		w_eyelead_hive3d.setParam(PARAM_IMAGE,"LINEAR:textures/ui/ui_loading_hive3d.image");
		w_eyelead_hive3d.setSize(256,128);
		w_eyelead_hive3d.setPos(790,480);
		w_eyelead_hive3d.hide();
		
		w_eyelead_hive3d.onLoad("cbSnapFadeIn");
	}
	
	//add event listener to capture events that change background color
	addListener(ON_EVENT,"HIDE","e_hide");
}
void cbSnapFadeIn() {
	w_snapdragon.fadeIn(300,"");
	addListener(ON_TIMEOUT_MSEC,"1000","cbSnapFadeOut");	
}
void cbSnapFadeOut() {
	w_snapdragon.fadeOut(150,"");
	w_eyelead.fadeIn(300,"cbEyeDelay");
	w_eyelead_logo.fadeIn(300,"");
	w_eyelead_hive3d.fadeIn(300,"");
}
void cbEyeDelay() {
	HUD.delChild(w_snapdragon);
	w_snapdragon=0;
	addListener(ON_TIMEOUT_MSEC,"1000","cbEyeFadeOut");	
}
void cbEyeFadeOut() {
	w_eyelead.fadeOut(150,"");
	w_eyelead_logo.fadeOut(150,"");
	w_eyelead_hive3d.fadeOut(150,"cbLoadFadeInBunny");
}
void cbLoadFadeInBunny() {
	w_bunnybg.fadeIn(300,"");
	w_bunnylogo.fadeIn(300,"cbLoadFadeIn");
	w_loadring.fadeTo(300,0.75,"");
	addListener(ON_TIMEOUT_MSEC,"100","cbRing");
}
void cbLoadFadeIn() {
	HUD.delChild(w_eyelead);
	HUD.delChild(w_eyelead_logo);
	HUD.delChild(w_eyelead_hive3d);
	w_eyelead=0;
	w_eyelead_logo=0;
	w_eyelead_hive3d=0;
	clearScene("/");
	loadScene("levelsets/generic/generic.hvfs","/","cbEyeSceneLoaded");
}
void cbEyeSceneLoaded() {
	if (["ui.home"->isLoaded() as int]) {
		sendEvent("HOME","sc.control");
		addTickPath("/sector");
	}
	else {
		debugLog("Waiting for ui.home");
		addListener(ON_TIMEOUT_MSEC,"100","cbEyeSceneLoaded");
	}
}
void cbResumeLoaded() {
	w_loadring.fadeTo(500,0.75,"");
	w_bunnybg.fadeIn(500,"");
	w_bunnylogo.fadeIn(500,"cbLoadMainScene");
	addListener(ON_TIMEOUT_MSEC,"100","cbRing");
}
void cbLoadMainScene() {
	clearScene("/");
	loadScene("levelsets/generic/generic.hvfs","/","cbSceneLoaded");
}
void cbSceneLoaded() {
	if (["ui.home"->isLoaded() as int]) {
		sendEvent("RESUME","sc.control");
		addTickPath("/sector");
	}
	else {
		debugLog("Waiting for ui.home");
		addListener(ON_TIMEOUT_MSEC,"100","cbSceneLoaded");
	}
}
void e_hide() {
	addListener(ON_TIMEOUT_MSEC,"500","cbHide");
}
void cbHide() {  
	HUD.fadeOut(500,"");
	w_bunnylogo.animate("Y","240:0;600:260","");
}
void cbRing() {
	if (HUD.isVisible()) {
		if (pollForComplete()>loadperc) loadperc+=12.5;
		w_loadring.setParam(PARAM_PERCENTAGE,string(loadperc));
		addListener(ON_TIMEOUT_MSEC,"100","cbRing");
	}
}
//for debugging
public int exist() { return 1; }
public void clearRender() {
	WIN.renderClear();
	WIN.setColor(float4(0,0,0,1));
}
public void renderSector() {
	WIN.setColor(float4(0.1,0.6,0.5,1.0));
	WIN.renderSector("/sector");
}
public int isIOS() { return is_ios; }
public int isKindle() { return is_kindle; }
