 /****************************************************************************
 * This file is part of Bunny Maze Application                                *
 *                                                                           *
 * Copyright (c) 2011, Eyelead Software. All rights reserved.                *
 * PROPRIETARY AND CONFIDENTIAL                                              *
 *                                                                           *
 * This file is subject to the terms and conditions of the                   *
 * Commercial Software License Agreement by and between                      *
 * Qualcomm Incorporated and Eyelead Software S.A                            *
 * as made and entered on September 30th, 2011                               *
 *****************************************************************************/
 
#using stdlib game
#hive 1

String APPNAME = "bunnymaze";
vfile parent,bridge;
float bridge_angle;

lists inside;
listvf vfinside;

float RADIUS=2.2;

void defaults() {
	this.setChannel("prox.radius",RADIUS);
	
	bridge_angle=-25.0;	
	bridge.setRot(hpr(0.0,0.0,bridge_angle));
	
	inside.clear();
	vfinside.clear();
	
	//reset listeners
	clearListeners(ON_TICK);
	clearListeners(ON_EVENT);
	clearListeners(ON_ENTITYENTER);
	clearListeners(ON_ENTITYLEAVE);
	clearListeners(ON_TIMEOUT_FRAMES);
}

void init()
{
	//set entity tag
	debugLog("Loading trap: "+getTag());
	parent=vfopen(this.parentPath(),"r");
	bridge=vfopen(this.parentPath()+"/bridge","r");
	inside=lists();
	vfinside=listvf();
	defaults();
}
public void savestate() {
	appPropertySet(APPNAME,getTag()+"_ANGLE",string(bridge_angle));
}
public void loadstate() {
	bridge_angle=float(appPropertyGet(APPNAME,getTag()+"_ANGLE",string(bridge_angle)));
}
public void regEvents() {
	addListener(ON_ENTITYENTER,"e_enter");
	addListener(ON_ENTITYLEAVE,"e_leave");
	addListener(ON_TICK,"tick");
}
public void reset() {
	defaults();
}
public string isLoaded() {
	return string(1);
}
public int validate(string _name,float _heading) {
	if (_name=="hero") {
		if (["hero"->isDoubleJumping() as int]) return 1;
	}
	//Enemies, turn around
	float angle=minangle(_heading,parent.getRot().h()+90.0);
	if (bridge_angle < -24.0) {
		if (angle<20) return 1;
	}
	else if (bridge_angle>24.0) {
		if (angle>160) return 1;
	}
	return 0;
}
void e_enter(string _tag) {
	if (_tag.startsWithNoCase("enemy") || _tag=="hero") {	
		inside.push(_tag);
		vfinside.push(vfopenEntityByTag(_tag));
		if (inside.num()==1) {
			["hero"->playTicTocShort(parent.getPos().xz())];
		}
	}
}
void e_leave(string _tag) {
	int index=inside.findIndex(_tag);
	if (index<inside.num()) {
		inside.delIndex(index);
		vfinside.delIndex(index);
		[_tag->setBaseY(2.33)];
	}
}

void tick() {
	bridge.setRot(hpr(0.0,0.0,bridge_angle));
	if (!inside.num()) {
		if (bridge_angle>0.0) {
			bridge_angle=lerp(bridge_angle,25.0,fElapsed());
		}
		else if (bridge_angle<=0.0) {
			bridge_angle=lerp(bridge_angle,-25.0,fElapsed());
		}
		return;
	}
	int i;
	float3 norm;
	float dist;
	vfile entt;
	string tag;
	int jump;
	float avg=0.0;
	int count=0;
	int num=inside.num();
	for (i=0;i<num;i++) {
		jump=0;
		entt=vfinside.get(i);
		tag=inside.get(i);
		norm=entt.getPos()-parent.getPos();
		norm.y(0.0);
		norm.roth(-(parent.getRot().h()+90.0));
		dist=clamp(norm.z(),-RADIUS,RADIUS);
		dist=-(dist/RADIUS);
		if ( tag=="hero" ) {
			if (!["hero"->isJumping() as int]) {
				avg+=dist;
				count++;
			}
			else jump=1;
		}
		else {
			avg+=dist;
			count++;
		}
		//find y
		if (!jump) {
			float3 ypos=float3(dist*RADIUS,0.0,0.0);
			ypos.rotr(-bridge_angle);
			[tag->setBaseY(max(2.33,3.3+ypos.y()))];
		}
	}
	if (count) {
		avg/=float(count);
		bridge_angle=avg*25.0;
	}
} 
