 /****************************************************************************
 * This file is part of Bunny Maze Application                                *
 *                                                                           *
 * Copyright (c) 2011, Eyelead Software. All rights reserved.                *
 * PROPRIETARY AND CONFIDENTIAL                                              *
 *                                                                           *
 * This file is subject to the terms and conditions of the                   *
 * Commercial Software License Agreement by and between                      *
 * Qualcomm Incorporated and Eyelead Software S.A                            *
 * as made and entered on September 30th, 2011                               *
 *****************************************************************************/

#using stdlib game
#hive 1
String APPNAME = "bunnymaze";

int trap_state=0;
int actoroffset = 1.2;
vfile actor;

public void reset() {  
	trap_state=0;
	actor.setMaster("reset",0);
	this.setPos(float3(0.0,2.0,0.0));
}
void init(){ 
	debugLog("Loading trap: "+getTag());
	this.setChannel("prox.radius",6.8);
	actor=vfopen(this.path()+"/../","r");
	actor.onComplete("reset");
}
public string isLoaded() { 
	return "1";
}
public void regEvents() {  
	addListener(ON_ENTITYENTER,"e_enter");
	addListener(ON_ENTITYLEAVE,"e_leave");
}

int isHeroOff(float radiusmin, float radiusmax){
	float3 pos = ["hero"->getPos() as float3];
	if (pos.y < 4){		
		string dir = ["hero"->getDir() as string];
		float diff;
		if (dir =="E" || dir=="W"){
			diff =  abs(pos.x - actor.getPos().x-actoroffset);
		} else {
			diff = abs(pos.z - actor.getPos().z-actoroffset);
		}			
		
		if (diff > radiusmin && diff < radiusmax ) {
			return 1;
		} 
	}
	return 0;
}

int isHeroOn(float radius){
	float3 pos = ["hero"->getPos() as float3];
	if (pos.y < 4){		
		string dir = ["hero"->getDir() as string];
		float diff;
		if (dir =="E" || dir=="W"){
			diff = abs(pos.x - actor.getPos().x-actoroffset);
		} else {
			diff = abs(pos.z - actor.getPos().z-actoroffset);
		}			
		
		if (diff < radius ) {
			return 1;
		} 
	}
	return 0;
}


void e_checkpoint_closed_rearm(){	
	if (trap_state==1){
		if(isHeroOn(0.7)){	}
		else if(isHeroOff(0.2,3.5)) {
			addListener(ON_TICK,"");
			delListener(ON_TICK,"e_checkpoint_closed_rearm");
		}
	}
	else if (["hero"->isDoubleJumping() as int]) {
		sendEvent("ROADBLOCK_OFF","ui.play");
		}
}

void e_checkpoint_closed(string _tag){
	if (isHeroOn(0.7)){
		sendEvent("ROADBLOCK","ui.play");
		delListener(ON_TICK,"e_checkpoint_closed");
		addListener(ON_TICK,"e_checkpoint_closed_rearm");
	}
}

void e_checkpoint(string _tag){
	delListener(ON_TIMEOUT_FRAMES,"e_checkpoint");
	delListener(ON_TIMEOUT_MSEC,"arm_roadblock");	
	addListener(ON_TICK, "e_checkpoint_closed_rearm");
	addListener(ON_TIMEOUT_MSEC,"2000", "e_rearm");
	float2 pos = actor.getPos().xz();
	["hero"->playSound(pos,"trap-jtrunkhit.audio"];
	
	if (isHeroOn(3.5)) {
		Event e=Event("JTRUNK","hero");		
		["hero"->playSound(pos,"trap-lighthouse.audio"];
		e.addArg(actor.getPos());
		e.sendEvent();
	}
	return;
}

void e_rearm(string _tag){
	float2 pos = actor.getPos().xz();
	delListener(ON_TICK, "e_checkpoint_closed");
	delListener(ON_TICK, "e_checkpoint_closed_rearm");
	sendEvent("ROADBLOCK_OFF","ui.play");
	addListener(ON_TIMEOUT_MSEC,"10","e_allowpass");
	delListener(ON_TIMEOUT_MSEC,"e_rearm");
	actor.setMaster("reset",0);	
	["hero"->playSound(pos,"trap-tictoc-short.audio"];
}

void e_enter(string _tag) {
	float2 pos = actor.getPos().xz();
	if (_tag.startsWithNoCase("rock")) {
		if (trap_state == 1) {
				e_rearm(_tag);
				return;
			}
		}
	if (trap_state < 1){
		if (_tag=="hero") {
			trap_state=1;
			actor.setMaster("hit",0); 
			["hero"->playSound(pos,"trap-jtrunkfall.audio"];
			addListener(ON_TIMEOUT_FRAMES,"22", "e_checkpoint");
		}
	} else {//if trap_state = 1: thy shall not pass
		if (_tag=="hero") {
			addListener(ON_TICK, "e_checkpoint_closed");
		} else {
			[_tag->doValidateLog(getTag())];
		}
	}
}

void e_leave(string _tag) {
	if (_tag=="hero") {
		delListener(ON_TICK, "e_checkpoint_closed");
	}
}

void e_allowpass(){
	delListener(ON_TIMEOUT_FRAMES,"e_allowpass");
	trap_state = 0;
}

public void savestate() {
	appPropertySet(APPNAME,getTag()+"_CLOSED",string(trap_state));
}
public void loadstate() {
	trap_state=int(appPropertyGet(APPNAME,getTag()+"_CLOSED",string(trap_state)));
	if (trap_state==1) {
		actor.setMaster("hit",0);
	}
	else if (trap_state==2) {
		actor.setMaster("reset",0);
	}
}
public int validate() {
	return (trap_state<1);
}
