 /****************************************************************************
 * This file is part of Bunny Maze Application                                *
 *                                                                           *
 * Copyright (c) 2011, Eyelead Software. All rights reserved.                *
 * PROPRIETARY AND CONFIDENTIAL                                              *
 *                                                                           *
 * This file is subject to the terms and conditions of the                   *
 * Commercial Software License Agreement by and between                      *
 * Qualcomm Incorporated and Eyelead Software S.A                            *
 * as made and entered on September 30th, 2011                               *
 *****************************************************************************/

#using stdlib game
#hive 1

int CONST_STATE_HIDDEN=0;
int CONST_STATE_ASCEND=1;
int CONST_STATE_IDLE=2;

Resource res;
vfile actor,hero;
int bLoaded=0;
int state,lock,bunny_inside;
float hero_dist;
string anim_post;

void defaults() {
	this.setChannel("prox.radius",2.5);
	state=0;
	lock=0;
	bunny_inside=0;

	actor.setPos( float3(0,-0.7,0) );
	actor.setScale(float3(0.8,0.8,0.8));
	actor.show();
	
	anim_post="";
	if (["sc.loader"->getTheme() as string]=="city") {
		anim_post="_sewer";
	}

	clearListeners(ON_TICK);
	clearListeners(ON_EVENT);	
	clearListeners(ON_TIMEOUT_FRAMES);
	clearListeners(ON_ENTITYENTER);
	clearListeners(ON_ENTITYLEAVE);
}

void init()
{
	//set entity tag
	debugLog("Loading trap: "+getTag());	
	res=Resource("levelsets/traps/gopher.hvfs","cbResLoaded");	
}
void cbResLoaded() {
	actor = res.use("gopher.actor","cbLoaded");
}
void cbLoaded() {
	bLoaded=1;
	defaults();

	hero=vfopen("/sector/hero","r");
	
	sendEvent("GOPHER_UNLOCK","ui.play");	
	
	fCall("cbReady");
}
public void reset() {
	defaults();
	fCall("cbReady");
}
public void regEvents() {
	addListener(ON_TICK,"TICK","e_tick");
	addListener(ON_ENTITYENTER,"e_enter");
	addListener(ON_ENTITYLEAVE,"e_leave");
	addListener(ON_EVENT,"HIT","e_hit");
}
void e_hit() {
	if (state==CONST_STATE_IDLE) {
		fCall("cbIdle");
	}
}
void e_enter(string _tag) {
	if (_tag=="hero") {
		bunny_inside=1;
		//if (state!=CONST_STATE_HIDDEN) {
		if (state==CONST_STATE_IDLE) {
			if (["hero"->isJumping() as int]) { 
				clearListeners(ON_TIMEOUT_FRAMES);
				fCall("cbIdle");
				sendEvent("GOPHER_ISHIT","ui.play");
				return; 			
			}
			if (state==CONST_STATE_IDLE) {
				actor.setMaster("hello"+anim_post,0);
				["hero"->playHello(this.getPos().xz())];
			}
			lock=1;
			sendEvent("GOPHER_LOCK","ui.play");
			["hero"->playTrapSound()];
		}
	}
	else if (_tag.startsWithNoCase("enemy")) {
		if (state!=CONST_STATE_HIDDEN) {
			Event e=Event("ATTACK",_tag);
			e.addArg(this.getPos());
			e.sendEvent();	
			clearListeners(ON_TIMEOUT_FRAMES);
			fCall("cbIdle");
		}
	}
	else if (_tag.startsWithNoCase("rock")) {	
		clearListeners(ON_TIMEOUT_FRAMES);
		fCall("cbIdle");
	}
}
void e_leave(string _tag) {
	if (_tag=="hero") {
		bunny_inside=0;
		if (lock) {
			lock=0;
			sendEvent("GOPHER_UNLOCK","ui.play");		
		}
	}
}
void e_tick() {
	float3 rot=lookat(this.getPos(),hero.getPos());
	rot.y(rot.y()-90.0);
	rot.x(0.0);
	rot.z(0.0);
	actor.setRot(rot);
	hero_dist=length( (this.getPos()-hero.getPos()).xz() );
	if (bunny_inside) {
		if (["hero"->isDoubleJumping() as int]) {
			e_leave("hero");
			clearListeners(ON_TIMEOUT_FRAMES);
			fCall("cbIdle");
		}
	}
}
void cbIdleState(){
	state=CONST_STATE_IDLE;
}
void cbUp() {	
	state=CONST_STATE_IDLE;
	if (hero_dist<10.0) {
		addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("hello"+anim_post,0)),"cbIdle");
		["hero"->playHello(this.getPos().xz())];
	} else {
		addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("up"+anim_post,0)),"cbIdle");
	}
}
void cbIdle() {
	state=CONST_STATE_ASCEND;
	addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("down"+anim_post,0)),"cbDown");
}
void cbDown() {
	state=CONST_STATE_HIDDEN;
	addListener(ON_TIMEOUT_FRAMES,string(120+rand(90)),"cbReady");
	if (lock) {
		lock=0;
		sendEvent("GOPHER_UNLOCK","ui.play");		
	}
}
void cbReady() {
	if (bunny_inside) {
		cbDown();
		return;
	}
	state=CONST_STATE_ASCEND;
	float frames_to_idle = actor.setMaster("up"+anim_post,0);
	int frames_to_idlehalf = int(frames_to_idle*0.5);
		
	addListener(ON_TIMEOUT_FRAMES,string(frames_to_idle),"cbUp");
	addListener(ON_TIMEOUT_FRAMES,string(frames_to_idlehalf),"cbIdleState");
}
public string isLoaded() {
	return string(bLoaded);
}
public int isHitable() {
	return int(state==CONST_STATE_IDLE);
}
public void savestate() {
}
public void loadstate() {
}
public int isHidden() {
	return int(state==CONST_STATE_HIDDEN);
}
public int isAscend() {
	return int(state==CONST_STATE_ASCEND);
}

