 /****************************************************************************
 * This file is part of Bunny Maze Application                                *
 *                                                                           *
 * Copyright (c) 2011, Eyelead Software. All rights reserved.                *
 * PROPRIETARY AND CONFIDENTIAL                                              *
 *                                                                           *
 * This file is subject to the terms and conditions of the                   *
 * Commercial Software License Agreement by and between                      *
 * Qualcomm Incorporated and Eyelead Software S.A                            *
 * as made and entered on September 30th, 2011                               *
 *****************************************************************************/

#using stdlib game gui sound
#hive 1


String APPNAME = "bunnymaze";

//if script is used for preloading
input int PRELOADER=0;

float NORMALSCALE=0.6;
float DRAGONSCALE=0.8;

//if snapdragon is supported
int SNAPDRAGON=0;

//Constants
float CONST_MAX_JUMP_FRAMES1=20.0;
float CONST_MAX_JUMP_FRAMES2=20.0;
//also defined runtime
float CONST_MAX_JUMP_FRAMES3=35.0;
float CONST_MAX_BLINKTIME=150.0;

float CONST_SPEED_SHOES=1.5;
float CONST_SPEED_SKULL=0.3;

//AI
int CONST_STATE_IDLE=0;
int CONST_STATE_WALK=1;

int CONST_BLENDSTATE_NONE=0;
int CONST_BLENDSTATE_DEADEND=1;
int CONST_BLENDSTATE_JUMP=2;
int CONST_BLENDSTATE_HAPPY=3;
int CONST_BLENDSTATE_HIT=4;
int CONST_BLENDSTATE_TRAP=5;
int CONST_BLENDSTATE_CRASH=6;
int CONST_BLENDSTATE_ICECUBE=7;
int CONST_BLENDSTATE_FALL=8;

//Skull initial positions
float3 CONST_SKULL_POS=float3(-0.86,0.0,0.0);
float3 CONST_SKULL_ROT=float3(-18.75,180.0,-90.0);
float3 CONST_SKULL_SCALE=float3(0.95,0.95,0.95);

Sound snd;
Resource res,resskull;
float speed,speed_mult,speed_trap,jump_scale,hedgehog_scale;
vfile actor,wings,tail,fire,firehit,dragonfx,skull,fx_hitstars,fx_speed,fx_res,fx_sound,fx_dust,jumpland,icecube;
vfile pu_rock,pu_shoe,pu_life,pu_potion;
string orientation,next_orientation,force_next;
int force_next_count;

float3 GOTO_POS;
float GOTO_HEADING,BASE_Y,BLINKTIME; 
float JUMP_Y1,JUMP_Y2,JUMP_Y3,JUMP_TIMEOUT1,JUMP_TIMEOUT2,JUMP_TIMEOUT3,JUMP_HEIGHT1,JUMP_HEIGHT2,JUMP_HEIGHT3,JUMP_THRES1,JUMP_THRES2,JUMP_THRES3,MOVE_DELAY,JUMP_DELAY;
int PU_GHOST,PU_SHOES,TRAP_SKULL;
int PU_WINGS,PU_TAIL,PU_FIRE;

//flags
int state,blendstate,bLoaded=0,bCanPlayJumpSound,bCanPlayTrapSound,bCanPlaySounds,bCanJump;
//icecube timer
int aux_state;
float aux_y,aux_timer;
float3 auxtile_pos;
//list of entities inside
lists l_enemies,l_enemy_jumps,l_quicksands,l_hogs,l_hog_jumps,l_penguins,l_penguin_jumps;

//CAMERA
Camera cam;
float3 CAM_TAR_OFFSET,CAM_TAR_ANGLEOFFSET;
float CAM_SMOOTHNESS,CAM_Z_RUN,CAM_Z_IDLE,PREVCAMORBIT_H,TARGETCAMORBIT_H;
float CAM_FOV,CAM_DEF_FOV,CAM_ACTION_FOV;

void commitCam() {
	float3 v=CAM_TAR_OFFSET;
	v.roth(PREVCAMORBIT_H);
	cam.setTargetOffset(v);
	//cam.setTargetAngleOffset(CAM_TAR_ANGLEOFFSET+float3( -3.0*(max(0.0,this.getPos().y-2.5)/4.0),PREVCAMORBIT_H,PREVCAMORBIT_H*0.2));
	cam.setTargetAngleOffset(CAM_TAR_ANGLEOFFSET+float3( 0.0,PREVCAMORBIT_H,PREVCAMORBIT_H*0.2));
	cam.setFOV(CAM_FOV);
}

public int isDragon() { return PU_WINGS && PU_TAIL && PU_FIRE; }

///// INIT /////
void defaults() {	
	//set proximity
	this.setChannel("col.height",2.0);
	this.setChannel("col.radius",0.5);
	this.setChannel("prox.radius",1.4);

	//speed
	speed = 0.4;
	speed_mult = 1.0;
	speed_trap = 1.0;
	jump_scale = 1.0;
	hedgehog_scale =1.0;
	
	//vfiles
	snd.stop();
	fx_hitstars.hide();
	pu_rock.hide();
	pu_shoe.hide();
	pu_life.hide();
	pu_potion.hide();
	
	//AI
	orientation="N";
	next_orientation="N";
	force_next="";
	force_next_count=0;
	GOTO_HEADING=0.0;
	BASE_Y=2.33;
	state=0;
	blendstate=0;
	BLINKTIME=0.0;
	PU_GHOST=0;
	PU_SHOES=0;
	TRAP_SKULL=0;
	PU_WINGS=0;
	PU_TAIL=0;
	PU_FIRE=0;
	
	//jump
	JUMP_Y1=0.0;
	JUMP_Y2=0.0;
	JUMP_Y3=0.0;
	JUMP_TIMEOUT1=0.0;
	JUMP_TIMEOUT2=0.0;
	JUMP_TIMEOUT3=0.0;
	JUMP_HEIGHT1=0.0;
	JUMP_HEIGHT2=0.0;
	JUMP_HEIGHT3=0.0;
	JUMP_THRES1=0.0;
	JUMP_THRES2=0.0;
	JUMP_THRES3=0.0;
	MOVE_DELAY=0.0;
	JUMP_DELAY=0.0;
	bCanPlayJumpSound=1;
	bCanPlayTrapSound=1;
	bCanPlaySounds=1;
	bCanJump=1;

	l_enemies.clear();
	l_enemy_jumps.clear();
	l_penguins.clear();
	l_penguin_jumps.clear();
	l_quicksands.clear();
	l_hogs.clear();
	l_hog_jumps.clear();

	actor.show();
	actor.setScale(float3(0.6,0.6,0.6));
	actor.setTimeScale(1.0);
	actor.tempShow();
	actor.clearTracks();
	actor.setMaster("idle",1);
	actor.blendClip("dragon_off",0);
	wings.blendClip("dragon_off",0);
	tail.blendClip("dragon_off",0);
	
	icecube.setPos(float3(-2,0,-0.5));
	icecube.setRot( hpr(0.0,0.0,-90.0) );
	icecube.setMaster("reset",0);
	icecube.show();
	icecube.tempHide();
	
	skull.setPos(CONST_SKULL_POS);
	skull.setRot(CONST_SKULL_ROT);
	skull.setScale(CONST_SKULL_SCALE);
	skull.show();	
	skull.tempHide();	
	
	wings.setPos(float3(-1.4,0.3,0.0));
	wings.setRot(hpr(270.0,90.0,180.0));
	wings.setScale(float3(0.7,1.0,1.0));
	wings.setMaster("walk",1);
	wings.show();
	wings.tempHide();
	

	tail.setMaster("walk",1);
	tail.setPos(float3(-0.54,0.11,0.0));
	tail.setRot(hpr(0.0,90.0,-90.0));
	tail.show();
	tail.tempHide();
	
	GOTO_POS=this.getPos();
	
	//camera
	CAM_TAR_OFFSET = float3(0,10.0,14);
	CAM_TAR_ANGLEOFFSET = float3(-32,0,0);
	CAM_SMOOTHNESS = 0.91;
	CAM_Z_RUN = 8.5;
	CAM_Z_IDLE = 14;
	PREVCAMORBIT_H=0,TARGETCAMORBIT_H=0;
	CAM_FOV=61.0;
	CAM_DEF_FOV=61.0;
	CAM_ACTION_FOV=67.0;	
	cam.setNearFar(0.1,100.0);
	cam.setTarget(this);
	cam.setTargetOffset(CAM_TAR_OFFSET);
	cam.setTargetAngleOffset(CAM_TAR_ANGLEOFFSET);
	cam.setSmoothness(CAM_SMOOTHNESS);
	cam.setFOV(CAM_FOV);	
	cam.reset();	
	
	//power ups
	pu_rock.hide();
	pu_shoe.hide();
	pu_life.hide();
	pu_potion.hide();
	pu_rock.setPos(float3(0,2,0));
	pu_shoe.setPos(float3(0,2,0));
	pu_life.setPos(float3(0,2,0));
	pu_potion.setPos(float3(0,2,0));
	
	//effects
	fx_dust.resetParticles();
	fx_dust.setService("emit.rate",0.0);
	fx_speed.resetParticles();
	fx_speed.setService("emit.rate",0.0);
	fx_speed.setService("emit.rate_var",0.0);
	fx_speed.setService("emit.rate_byspeed",0.0);
	fx_res.resetParticles();
	fx_res.setService("emit.rate",0.0);
	
	fire.resetParticles();
	fire.setService("emit.rate",0.0);
	fire.setService("emit.rate_var",0.0);
	
	firehit.resetParticles();
	firehit.setService("emit.rate",0.0);
	
	dragonfx.resetParticles();
	dragonfx.setService("emit.rate",0.0);
	
	jumpland.resetParticles();
	jumpland.setService("emit.rate",0.0);
	
	//reset listeners
	clearListeners(ON_TICK);
	clearListeners(ON_EVENT);
	clearListeners(ON_ENTITYENTER);
	clearListeners(ON_ENTITYLEAVE);
	clearListeners(ON_TIMEOUT_FRAMES);
}


void init()
{
	if (PRELOADER) {
		debugLog("Preloading Bunny");
		setTag("dummy");
		res=Resource("levelsets/generic/bunny.hvfs","");	
		resskull=Resource("levelsets/traps/skull.hvfs","");	
		return;
	}

	SNAPDRAGON=int(appPropertyGet(APPNAME,"snapdragon","0"));
	
	bLoaded=0;

	//set entity tag
	debugLog("Loading bunny");
	setTag("hero");

	//initialize camera
	cam=Camera();
	
	//initialize lists
	l_hogs=lists();
	l_hog_jumps=lists();
	l_enemies=lists();
	l_enemy_jumps=lists();
	l_penguins=lists();
	l_penguin_jumps=lists();
	l_quicksands=lists();
	
	//clean up just in case
	clearScene(this.path()+"/fx");
	clearScene(this.path()+"/chest");
	//mount effects
	system("mkdir "+this.path()+"/fx");
	system("mkdir "+this.path()+"/chest");
	system("cp /assets/pu_shoe.actor "+this.path()+"/chest/pu_shoe.actor");
	system("cp /assets/pu_potion.actor "+this.path()+"/chest/pu_potion.actor");
	system("cp /assets/pu_rock.actor "+this.path()+"/chest/pu_rock.actor");
	system("cp /assets/pu_life.actor "+this.path()+"/chest/pu_life.actor");
	
	pu_rock=vfopen(this.path()+"/chest/pu_rock.actor","r");
	pu_shoe=vfopen(this.path()+"/chest/pu_shoe.actor","r");
	pu_life=vfopen(this.path()+"/chest/pu_life.actor","r");
	pu_potion=vfopen(this.path()+"/chest/pu_potion.actor","r");
	
	loadScene("levelsets/generic/fx.hvfs",this.path()+"/fx","cbFxLoaded");
	loadScene("levelsets/generic/hitstars.hvfs",this.path()+"/fx","cbHitstarsLoaded");
}
void cbHitstarsLoaded() {
	fx_hitstars=vfopen(this.path()+"/fx/hitstars.actor","r");
}
void cbFxLoaded() { 	
	fx_speed=vfopen(this.path()+"/fx/speed.bboard","r");
	fx_res=vfopen(this.path()+"/fx/res.bboard","r");
	fx_dust=vfopen(this.path()+"/fx/dust.bboard","r");
	fire=vfopen(this.path()+"/fx/fire.bboard","r");
	fire.setPos(float3(-0.5,0.0,0.0));

	firehit=vfopen(this.path()+"/fx/firehit.bboard","r");
	firehit.setPos(float3(0.0,0.7,-0.2));

	jumpland=vfopen(this.path()+"/fx/jumpland.bboard","r");
	jumpland.setPos(float3(0.0,0.7,-0.2));

	dragonfx=vfopen(this.path()+"/fx/dragonfx.bboard","r");
	dragonfx.setPos(float3(0.0,1.6,0.0));

	res=Resource("levelsets/generic/bunny.hvfs","cbResLoaded");	
}  
void cbResLoaded() {
	if (SNAPDRAGON) {
		system("matlsetconst dragon float3(1.0,0.1,0.05) "+res.path()+"/male_fat/mbunny_mat");
	}
	else {
		system("matlsetconst dragon float3(1.0,0.5,0.1) "+res.path()+"/male_fat/mbunny_mat");
	}
	tail = res.use("tail.actor","");
	wings = res.use("wings.actor","");
	icecube = res.use("ice_column.actor","");	
	actor = res.use("male.actor","actorloaded");
	fx_sound = vfopen("/sector/bunny.sound","r");
	snd=Sound(fx_sound);
	fCall("e_volume");
}
void actorloaded() {
	resskull=Resource("levelsets/traps/skull.hvfs","cbSkullLoaded");
}
void cbSkullLoaded() {
	skull = resskull.use("skull","skullloaded");
}
void skullloaded() { 
	//make external parenting
	system("mkconn "+actor.path()+" XCLD:"+skull.path());
	system("setconnmeta "+actor.path()+" "+skull.path()+" parent=bn_head01");

	//attach wings
	system("mkconn "+actor.path()+" XCLD:"+wings.path());
	system("setconnmeta "+actor.path()+" "+wings.path()+" parent=bn_chest01");

	//attach tail
	system("mkconn "+actor.path()+" XCLD:"+tail.path());
	system("setconnmeta "+actor.path()+" "+tail.path()+" parent=root_hip01");

	//attach fire
	system("mkconn "+actor.path()+" XCLD:"+fire.path());
	system("setconnmeta "+actor.path()+" "+fire.path()+" parent=hand_r_01");

	//attack icecube
	system("mkconn "+actor.path()+" XCLD:"+icecube.path());
	system("setconnmeta "+actor.path()+" "+icecube.path()+" parent=bn_chest01");

	actor.onComplete("reset");
}
public void reset() {
	if (bLoaded) debugLog("Reset Bunny");
	else {	
		bLoaded=1;
		debugLog("Bunny loaded");
	}
	defaults();
}
public void regEvents() {
	addListener(ON_EVENT,"CHEST","e_chest");
	addListener(ON_EVENT,"GOTO","e_goto");
	addListener(ON_EVENT,"RESET_GOTO","e_reset_goto");
	addListener(ON_EVENT,"STOP","e_stop");
	addListener(ON_EVENT,"DEADEND","e_deadend");
	addListener(ON_EVENT,"VICTORY","e_victory");
	addListener(ON_EVENT,"FAIL","e_fail");
	addListener(ON_EVENT,"HAPPY","e_happy");
	addListener(ON_EVENT,"JUMP","e_jump");
	addListener(ON_EVENT,"SHOES_ON","e_shoes_on");
	addListener(ON_EVENT,"SHOES_OFF","e_shoes_off");
	addListener(ON_EVENT,"GHOST_ON","e_ghost_on");
	addListener(ON_EVENT,"GHOST_OFF","e_ghost_off");
	addListener(ON_EVENT,"WINGS","e_wings");	
	addListener(ON_EVENT,"TAIL","e_tail");	
	addListener(ON_EVENT,"FIRE","e_fire");	
	addListener(ON_EVENT,"CRACK","e_crack");
	addListener(ON_EVENT,"JTRUNK","e_jtrunk");
	addListener(ON_EVENT,"VOLUME","e_volume");	
	addListener(ON_EVENT,"SNOWBALL","e_hit");	
	addListener(ON_EVENT,"HIT","e_hit");	
	addListener(ON_EVENT,"WATER","e_water");
	addListener(ON_ENTITYENTER,"e_enter");
	addListener(ON_ENTITYLEAVE,"e_leave");
	addListener(ON_TICK,"tick");
	
}
public string isLoaded() {
	return string(bLoaded);
}


////// CHEST EFFECT //////

void e_chest(string _pu) {
	if (_pu=="life") {
		pu_life.show();
		addListener(ON_TIMEOUT_FRAMES,string(pu_life.setMaster("bounce",0)),"cbChestLifeOver");
	}
	else if (_pu=="shoe") {
		pu_shoe.show();
		addListener(ON_TIMEOUT_FRAMES,string(pu_shoe.setMaster("bounce",0)),"cbChestShoeOver");
	}
	else if (_pu=="potion") {
		pu_potion.show();
		addListener(ON_TIMEOUT_FRAMES,string(pu_potion.setMaster("bounce",0)),"cbChestPotionOver");
	}
	else if (_pu=="rock") {
		pu_rock.show();
		addListener(ON_TIMEOUT_FRAMES,string(pu_rock.setMaster("bounce",0)),"cbChestRockOver");
	}
}
void cbChestLifeOver() { pu_life.hide(); }
void cbChestShoeOver() { pu_shoe.hide(); }
void cbChestPotionOver() { pu_potion.hide(); }
void cbChestRockOver() { pu_rock.hide(); }

/////// STATE //////
public float3 getPos(){
	return this.getPos();
}

public float getJumpY(){
	return (JUMP_Y1+JUMP_Y2+JUMP_Y3);
}

public void savestate() {
	appPropertySet(APPNAME,"BUNNY_POS",string(this.getPos()));
	appPropertySet(APPNAME,"BUNNY_ROT",string(this.getRot()));
	appPropertySet(APPNAME,"BUNNY_GOTO",string(GOTO_POS));
	appPropertySet(APPNAME,"BUNNY_SKULL",string(TRAP_SKULL));
	appPropertySet(APPNAME,"BUNNY_WINGS",string(PU_WINGS));
	appPropertySet(APPNAME,"BUNNY_TAIL",string(PU_TAIL));
	appPropertySet(APPNAME,"BUNNY_FIRE",string(PU_FIRE));
}
public void loadstate() {
	this.setPos(float3(appPropertyGet(APPNAME,"BUNNY_POS",string(this.getPos()))));
	this.setRot(float3(appPropertyGet(APPNAME,"BUNNY_ROT",string(this.getRot()))));
	GOTO_POS=float3(appPropertyGet(APPNAME,"BUNNY_GOTO",string(GOTO_POS)));
	TRAP_SKULL=int(appPropertyGet(APPNAME,"BUNNY_SKULL",string(TRAP_SKULL)));
	if (TRAP_SKULL) {
		skull.tempShow();
		skull.setPos(CONST_SKULL_POS);
		speed_mult=CONST_SPEED_SKULL;
	}
	if (int(appPropertyGet(APPNAME,"BUNNY_WINGS",string(PU_WINGS)))) fCall("e_wings");
	if (int(appPropertyGet(APPNAME,"BUNNY_TAIL",string(PU_TAIL)))) fCall("e_tail");
	if (int(appPropertyGet(APPNAME,"BUNNY_FIRE",string(PU_FIRE)))) fCall("e_fire");
	cam.reset();
}

////// VOLUME //////

void e_volume() {
	snd.setVolume(float(appPropertyGet(APPNAME,"sfxvol","1.0")));
}
public void playAttackSound(float2 _src) {
	if ( length( this.getPos().xz() - _src )<20.0 ) 	
		snd.fplay("sounds/attacked.audio"); 
}
public void playHitSound(float2 _src) {
	if ( length( this.getPos().xz() - _src )<20.0 ) 	
		snd.fplay("sounds/bunny-attack.audio");
}

public void playTrapSound() {
	if (!bCanPlayTrapSound) return;
	bCanPlayTrapSound=0;
	snd.fplay("sounds/trap-generic.audio");
	addListener(ON_TIMEOUT_FRAMES,string(2*getFps()),"cbResetTrapSound");
}
void cbResetTrapSound() { bCanPlayTrapSound=1;}
public void playTicToc(float2 _src) {
	if (!bCanPlaySounds) return;
	if ( length( this.getPos().xz() - _src )>12.0 ) return;
	snd.play("sounds/trap-tictoc.audio");
}
public void playTicTocShort(float2 _src) {
	if (!bCanPlaySounds) return;
	if ( length( this.getPos().xz() - _src )>12.0 ) return;
	snd.play("sounds/trap-tictoc-short.audio");
}
public void playSplash(float2 _src) {
	if ( length( this.getPos().xz() - _src )>12.0 ) return;
	snd.play("sounds/trap-splash.audio");
}
public void playHello(float2 _src) {
	if (!bCanPlaySounds) return;
	if ( length( this.getPos().xz() - _src )>6.0 ) return;
	snd.play("sounds/gopher-hello.audio");
}
public void playIcecrack(int _fall) {
	if (_fall) snd.play("sounds/trap-icecrack-fallcrack.audio");
	else snd.play("sounds/trap-icecrack-cracking.audio");
}
public void playPenguin(float2 _src,string _type) {
	if (!bCanPlaySounds) return;
	if ( length( this.getPos().xz() - _src )>6.0 ) return;
	snd.play("sounds/pingu-"+_type+".audio");
}
public void playSound(float2 _src,string soundfile) {
	if ( length( this.getPos().xz() - _src )>12.0 ) return;
	snd.play("sounds/"+soundfile);
}
public int doValidateLog(string _tag) {
	if (![_tag->validate(this.getRot().h()) as int]) {
		playTrapSound();
		return 0;
	}
	return 1;
}
///// HELPERS /////

void resetListeners() {
	delListener(ON_TIMEOUT_FRAMES,"cbFinishHappy");	
	delListener(ON_TIMEOUT_FRAMES,"cbFinishJump");
	delListener(ON_TIMEOUT_FRAMES,"cbFinishHit");
	delListener(ON_TIMEOUT_FRAMES,"cbFinishTrap");
}
void resetBlends() {
	resetListeners();
	JUMP_Y1=0;
	JUMP_Y2=0;
	JUMP_Y3=0;
	JUMP_TIMEOUT1=0;
	JUMP_TIMEOUT2=0;
	JUMP_TIMEOUT3=0;
}
void restoreFromBlend() {
	if (state==CONST_STATE_WALK) {
		if (PU_SHOES) {
			actor.setMaster("run",1);
			wings.setMaster("run",1);
			tail.setMaster("walk",1);
		}
		else {
			actor.setMaster("walk",1);
			wings.setMaster("walk",1);			
			tail.setMaster("walk",1);
		}
	}
	else if (state==CONST_STATE_IDLE) {
		actor.setMaster("idle",1);
		wings.setMaster("walk",1);
		tail.setMaster("walk",1);
	}
}

////// COLLECT POWERUP //////

void e_happy() {
	if (blendstate!=CONST_BLENDSTATE_NONE) return;
	resetBlends();	
	string timeout=string(actor.setMaster("happy",0));
	wings.setMaster("happy",0);	
	tail.setMaster("walk",0);
	addListener(ON_TIMEOUT_FRAMES,timeout,"cbFinishHappy");	
	blendstate=CONST_BLENDSTATE_HAPPY;
	fx_res.setService("emit.rate",0.5);
	delListener(ON_TIMEOUT_FRAMES,"cbFinishResFX");	
	addListener(ON_TIMEOUT_FRAMES,timeout,"cbFinishResFX");	
}
void e_crack(float3 _tilepos) {
	if (blendstate==CONST_BLENDSTATE_ICECUBE) return;
	resetBlends();	
	auxtile_pos=_tilepos;
	aux_timer=0;
	aux_state=0;	
	blendstate=CONST_BLENDSTATE_ICECUBE;
	icecube.setMaster("reset",0);
	actor.setMaster("run",1);
	actor.setTimeScale(1.5);
}


public void hitStars_show(){
	fx_hitstars.show();
	fx_hitstars.setMaster("hit",0);
}

public void hitStars_hide(){
	fx_hitstars.hide();
}

void e_jtrunk(float3 _tilepos) {
	if (blendstate==CONST_BLENDSTATE_TRAP) return;
	auxtile_pos=_tilepos;
	resetBlends();	
	blendstate=CONST_BLENDSTATE_TRAP;
	actor.setMaster("walk",1);
	actor.setTimeScale(0.0);
	addListener(ON_TIMEOUT_MSEC,"700","e_trunkrevive");
	hitStars_show();
	aux_state=0;
	actor.setScale(float3(0.1,NORMALSCALE,NORMALSCALE));
}

void e_trunkrevive(){	
		addListener(ON_TICK, "e_squashrestore");	
}

void e_squashrestore(){
	float fac = float(aux_state) * 0.05; //fac = [0,1]
	float factor = 0.1+0.9*fac*fac + 0.2*( (1-fac)*sin(fac*3.141*8));
	if ( aux_state < 20) {
		aux_state ++ ;
		actor.setScale(float3(factor,NORMALSCALE,NORMALSCALE));	
	}
	else {
		actor.setScale(float3(NORMALSCALE,NORMALSCALE,NORMALSCALE));		
		actor.setTimeScale(1.0);
		blendstate=CONST_BLENDSTATE_NONE;
		restoreFromBlend();
		delListener(ON_TICK, "e_squashrestore");
	}

}

void cbFinishIceExplode() {
	icecube.setMaster("reset",0);
	icecube.tempHide();
}
void cbFinishHappy() {
	blendstate=CONST_BLENDSTATE_NONE;
	restoreFromBlend();
}
void cbFinishResFX() {	
	fx_res.setService("emit.rate",0.0);
}

////// JUMP //////
void doDust(int _jump) {
	delListener(ON_TIMEOUT_FRAMES,"cbFinishDust");
	if (((JUMP_Y1+JUMP_Y2+JUMP_Y3)>0.2) && (_jump!=2)) return;
	if (_jump) {
		fx_dust.setPos(float3(0,1.5,0));
		fx_dust.setService("emit.rate",5.0);
		fx_dust.setService("emit.radius.min",1.5);
		fx_dust.setService("emit.radius.max",2.0);
		addListener(ON_TIMEOUT_FRAMES,"4","cbFinishDust");
	}
	else {
		fx_dust.setPos(float3(0,0.5,0));
		fx_dust.setService("emit.rate",1.0);
		fx_dust.setService("emit.radius.min",0.5);
		fx_dust.setService("emit.radius.max",1.0);
		addListener(ON_TIMEOUT_FRAMES,"40","cbFinishDust");
	}
}
void cbFinishDust() {
	if ((state==CONST_STATE_IDLE) || (blendstate!=CONST_BLENDSTATE_NONE)) {
		fx_dust.setService("emit.rate",0.0);
		fx_dust.setService("emit.rate_var",0.0);
	}
	else {
		fx_dust.setService("emit.rate",0.2);
		fx_dust.setService("emit.rate_var",0.1);
	}
}
void cbResetTrap() {
	TRAP_SKULL=4;
}
void tick_restore_y(){
	float frames=getFps()*0.1;
	float3 heropos = this.getPos();
	if (aux_timer>=frames) {
		heropos.y(BASE_Y);
		delListener(ON_TICK, "tick_restore_y");
	}
	else {
		heropos.y( lerp(-1.0,BASE_Y,aux_timer/frames) );		
		aux_timer++;
	}
	this.setPos(heropos);
}
void e_jump(int _msec) {
	if (blendstate==CONST_BLENDSTATE_ICECUBE) {
		if ( aux_state>4) {		
			blendstate=CONST_BLENDSTATE_NONE;
			restoreFromBlend();
			float3 rot=actor.getRot();
			rot.z(0.0);
			actor.setRot(rot);
			actor.setTimeScale(1.0);	
			wings.setTimeScale(1.0);	
			tail.setTimeScale(1.0);	
			addListener(ON_TIMEOUT_FRAMES,string(icecube.setMaster("explode",0)),"cbFinishIceExplode");
		}
	} else if (blendstate==CONST_BLENDSTATE_FALL) {
		if ( aux_state >3) {
			aux_timer=0;
			addListener(ON_TICK, "tick_restore_y");
			blendstate=CONST_BLENDSTATE_NONE;
			restoreFromBlend();
			float3 rot=actor.getRot();
			rot.z(0.0);
			actor.setRot(rot);
			actor.setTimeScale(1.0);	
			wings.setTimeScale(1.0);	
			tail.setTimeScale(1.0);	
		}
	}
	
	if ((blendstate!=CONST_BLENDSTATE_NONE) && (blendstate!=CONST_BLENDSTATE_JUMP) && (blendstate!=CONST_BLENDSTATE_HAPPY)) return;
	if (!bCanJump) return;

	if (TRAP_SKULL) {
		TRAP_SKULL--;
		if (!TRAP_SKULL) {
			skull.temphide();
			if (PU_SHOES) speed_mult=CONST_SPEED_SHOES;
			else speed_mult=1.0;
			delListener(ON_TIMEOUT_FRAMES,"cbResetTrap");
		}
		else {
			JUMP_HEIGHT1=1.0;
			JUMP_THRES1=0.0;
			JUMP_TIMEOUT1=CONST_MAX_JUMP_FRAMES1;
			delListener(ON_TIMEOUT_FRAMES,"cbResetTrap");
			addListener(ON_TIMEOUT_FRAMES,string(1.0*getFPS()),"cbResetTrap");
			return;
		}
	}
	
	if (jump_scale<0.5) {
		int num=l_quicksands.num();
		if (num) {
			int i;
			for (i=0;i<num;i++) {
				sendEvent("JUMP",l_quicksands.get(i));
			}			
		}
		return;
	}
	
	//double jump
	if (blendstate==CONST_BLENDSTATE_JUMP) {
		if (JUMP_Y2==0 && JUMP_TIMEOUT1>=(CONST_MAX_JUMP_FRAMES1*0.125) && JUMP_TIMEOUT1<=(CONST_MAX_JUMP_FRAMES1*0.9)) {
			if (!PU_WINGS) actor.setMaster("jump",0);
			else actor.setMaster("fly",1);
			wings.setMaster("fly",0);
			tail.setMaster("fly",0);
			JUMP_TIMEOUT2=CONST_MAX_JUMP_FRAMES2;
			JUMP_HEIGHT2=4.5;
			JUMP_THRES1=0.0;
			JUMP_THRES3=0.0;
			JUMP_THRES2=2.0;
			delListener(ON_TIMEOUT_FRAMES,"cbFinishJump");
			addListener(ON_TIMEOUT_FRAMES,string(CONST_MAX_JUMP_FRAMES2),"cbFinishJump");	
			return;
		}
		else if (PU_WINGS && JUMP_Y3==0 && JUMP_TIMEOUT2>=(CONST_MAX_JUMP_FRAMES2*0.125) && JUMP_TIMEOUT2<=(CONST_MAX_JUMP_FRAMES2*0.9)) {
			wings.setMaster("fly",0);
			tail.setMaster("fly",0);
			if (isDragon()) CONST_MAX_JUMP_FRAMES3=35.0;
			else CONST_MAX_JUMP_FRAMES3=25.0;
			JUMP_TIMEOUT3=CONST_MAX_JUMP_FRAMES3;
			JUMP_HEIGHT3=2.0;
			JUMP_THRES1=0.0;
			JUMP_THRES2=0.0;
			JUMP_THRES3=0.5;
			delListener(ON_TIMEOUT_FRAMES,"cbFinishJump");
			addListener(ON_TIMEOUT_FRAMES,string(CONST_MAX_JUMP_FRAMES3),"cbFinishJump");	
			return;
		}
		else { return; }
	}
	resetBlends();
	blendstate=CONST_BLENDSTATE_JUMP;
	JUMP_Y1=0;
	JUMP_Y2=0;
	JUMP_Y3=0;
	JUMP_HEIGHT1=4.5;
	JUMP_HEIGHT2=0.0;
	JUMP_HEIGHT3=0.0;
	JUMP_THRES1=2.0;
	JUMP_THRES2=0.0; 
	JUMP_THRES3=0.0; 
	fx_dust.setService("emit.rate",0.0);
	if (bCanPlayJumpSound) {
		bCanPlayJumpSound=0;
		snd.fplay("sounds/bunny-jump"+string(1+rand(2))+".audio");
		addListener(ON_TIMEOUT_FRAMES,string(getFPS()),"cbResetJumpSound");	
	}
	if (!PU_WINGS) actor.setMaster("jump2",0);
	else actor.setMaster("fly",1);
	wings.setMaster("fly",0);	
	tail.setMaster("fly",0);
	addListener(ON_TIMEOUT_FRAMES,string(CONST_MAX_JUMP_FRAMES1),"cbFinishJump");	
	JUMP_TIMEOUT1=CONST_MAX_JUMP_FRAMES1;
	JUMP_TIMEOUT2=0;
	JUMP_TIMEOUT3=0; 
}
void cbResetJumpSound() {
	bCanPlayJumpSound=1;
}
void e_walk() {
	if (state!=CONST_STATE_IDLE) {
		doDust(1);
		if (blendstate==CONST_BLENDSTATE_NONE) {
			actor.setMaster("walk",1);
			wings.setMaster("walk",1);
			tail.setMaster("walk",1);
		}
		state=CONST_STATE_WALK;
	}
	
	//else{
//		state = CONST_STATE_WALK;
		//if (blendstate==CONST_BLENDSTATE_NONE) {
//			actor.setMaster("walk",1);
			//wings.setMaster("walk",1);
			//tail.setMaster("walk",1);
		//}
	//}
}
void cbFinishJump() {
	blendstate=CONST_BLENDSTATE_NONE;
	restoreFromBlend();
	
	//kill enemies
	int bPlayAttackSound=0;
	int i;
	int count_clunnies_hit=0,count_clunnies_jumped=0;
	int num=l_enemies.num();
	string tag;
	for (i=0;i<num;i++) {
		int index=l_enemy_jumps.findIndex(l_enemies.get(i));
		if (index<l_enemy_jumps.num()) {
			l_enemy_jumps.delIndex(index);
		}
		tag=l_enemies.get(i);
		if (![tag->isHit() as int]) {
			//sendEvent("CLUNNY_ISHIT","ui.play");
			count_clunnies_hit++;
			sendEvent("HIT",tag);
			MOVE_DELAY=20.0;
			bPlayAttackSound=1;
		}
	}
	num=l_enemy_jumps.num();
	for (i=0;i<num;i++) {
		count_clunnies_jumped++;
		//sendEvent("CLUNNY_ISJUMPED","ui.play");		
	}
	l_enemy_jumps.clear();
	
	if ((count_clunnies_hit+count_clunnies_jumped)>0) {
		Event e=Event("CLUNNY_ISHIT","ui.play");
		e.addArg(count_clunnies_hit);
		e.addArg(count_clunnies_jumped);
		e.sendEvent();
	}

	//kill penguins
	num=l_penguins.num();
	for (i=0;i<num;i++) {
		tag=l_penguins.get(i);
		int index=l_penguin_jumps.findIndex(tag);
		if (index<l_penguin_jumps.num()) {
			l_penguin_jumps.delIndex(index);
		}
		if (![tag->isHit() as int]) {	
			sendEvent("HEDGE_ISHIT","ui.play");
			sendEvent("HIT",tag);	
			MOVE_DELAY=20.0;
			bPlayAttackSound=1;
		}
	}
	num=l_penguin_jumps.num();
	for (i=0;i<num;i++) {
		sendEvent("HEDGE_ISJUMPED","ui.play");		
	}
	l_penguin_jumps.clear();
	
	//kill hedgehogs
	num=l_hogs.num();
	for (i=0;i<num;i++) {
		tag=l_hogs.get(i);
		int index=l_hog_jumps.findIndex(tag);
		if (index<l_hog_jumps.num()) {
			l_hog_jumps.delIndex(index);
		}
		if ([tag->isHitable() as int]) {	
			sendEvent("HEDGE_ISHIT","ui.play");  ///////////////////////////////
			sendEvent("HIT",tag);
			MOVE_DELAY=10.0;
			sendEvent("UNFORCESTOP","ui.play");
			e_walk();			
			bPlayAttackSound=1;
		}
	}
	num=l_hog_jumps.num();
	for (i=0;i<num;i++) {
		sendEvent("HEDGE_ISJUMPED","ui.play");
	}
	l_hog_jumps.clear();
	
	if (bPlayAttackSound) {
		snd.fplay("sounds/bunny-attack.audio");
	}
	
	//do fire attack
	if (PU_FIRE) {
		snd.fplay("sounds/dragon-firehit.audio");						
		firehit.setService("emit.rate",25.0);
		addListener(ON_TIMEOUT_FRAMES,"2","cbFinishFireHit");
		sendEvent("DO_FIREHIT","ui.play");	
	}
	else {
		jumpland.setService("emit.rate",10.0);
		addListener(ON_TIMEOUT_FRAMES,"2","cbFinishFireHit");
	}
}
void cbFinishFireHit() {
	firehit.setService("emit.rate",0.0);
	jumpland.setService("emit.rate",0.0);
}

/////// POWER UPS ///////

void e_shoes_on() { 
	if (state==CONST_STATE_WALK && blendstate==CONST_BLENDSTATE_NONE) {
		actor.setMaster("run",1);
		wings.setMaster("run",1);
		tail.setMaster("walk",1);
	}
	fx_speed.setService("emit.rate",3.0);
	fx_speed.setService("emit.rate_var",1.0);
	fx_speed.setService("emit.rate_byspeed",5.0);
	if (!TRAP_SKULL) speed_mult = CONST_SPEED_SHOES;
	PU_SHOES=1;
}
void e_shoes_off() {
	if (state==CONST_STATE_WALK && blendstate==CONST_BLENDSTATE_NONE) {
		actor.setMaster("walk",1);
		wings.setMaster("walk",1);		
		tail.setMaster("walk",1);
	}
	fx_speed.setService("emit.rate",0.0);
	fx_speed.setService("emit.rate_var",0.0);
	fx_speed.setService("emit.rate_byspeed",0.0);
	speed_mult = 1.0; 
	PU_SHOES=0;
}

void e_ghost_on() {
	if (isDragon()) {
		actor.blendClip("dragonghost_on",0);
		tail.blendClip("dragonghost_on",0);
		wings.blendClip("dragonghost_on",0);
	}
	else {
		actor.blendClip("ghost_on",0);
		tail.blendClip("ghost_on",0);
		wings.blendClip("ghost_on",0);
	}
	PU_GHOST=1;
}
void e_ghost_off() {
	if (isDragon()) {
		actor.blendClip("dragonghost_off",0);
		tail.blendClip("dragonghost_off",0);
		wings.blendClip("dragonghost_off",0);
	}
	else {
		actor.blendClip("ghost_off",0);
		tail.blendClip("ghost_off",0);
		wings.blendClip("ghost_off",0);
	}
	PU_GHOST=0;
}

////// DRIVE //////

void e_reset_goto() {
	GOTO_POS=this.getPos();
}
void cbTailHit() {
	sendEvent("DO_TAILHIT","ui.play");
}

void e_stop() {
	if (state!=CONST_STATE_IDLE) {
		doDust(1);
		if (blendstate==CONST_BLENDSTATE_NONE) {
			actor.setMaster("idle",1);
			wings.setMaster("walk",1);
			tail.setMaster("walk",1);
		}
		state=CONST_STATE_IDLE;
	}
}
void e_goto(float3 _newpos,float _angle) {
	if (minangle(GOTO_HEADING,_angle)>80) {
		doDust(1);
		if (PU_TAIL) {
			if ( minangle(GOTO_HEADING,_angle)>150 ) {
				if (blendstate==CONST_BLENDSTATE_NONE) {
					resetBlends();
					string timeout;
					float myheading=normalizeAngle(this.getRot().h());
					float tangle=normalizeangle(turnangle(myheading,_angle,abs(minangle(myheading,_angle))*0.25));
					if (tangle>myheading) { 
						addListener(ON_TIMEOUT_FRAMES,string(tail.setMaster("hit_left",0)),"cbTailGrow");
						timeout=string(actor.setMaster("happy_left",0));
					}
					else {
						addListener(ON_TIMEOUT_FRAMES,string(tail.setMaster("hit_right",0)),"cbTailGrow");
						timeout=string(actor.setMaster("happy_right",0));
					}
					wings.setMaster("happy",0);	
					snd.fplay("sounds/dragon-tailhit.audio");					
					addListener(ON_TIMEOUT_FRAMES,timeout,"cbFinishHappy");	
					blendstate=CONST_BLENDSTATE_HAPPY;
					addListener(ON_TIMEOUT_FRAMES,"1","cbTailHit");
				}
			}
		}
	}
	GOTO_POS=_newpos;
	GOTO_HEADING=_angle;
	if (state!=CONST_STATE_WALK) {
		if (blendstate==CONST_BLENDSTATE_NONE) {
			doDust(0);
			if (PU_SHOES) {
				actor.setMaster("run",1);
				wings.setMaster("run",1);
				tail.setMaster("walk",1);
			}
			else {
				actor.setMaster("walk",1);
				wings.setMaster("walk",1);				
				tail.setMaster("walk",1);
			}
		}
		state=CONST_STATE_WALK;
	}
}


///// DEADEND ////

public void beginLevel() {
	addListener(ON_TIMEOUT_MSEC,"500","startBegin");
	bCanPlaySounds= 1;
}
public void endLevel() {
	bCanPlaySounds = 0;
}
void startBegin() {	
	resetBlends();
	blendstate=CONST_BLENDSTATE_DEADEND;
	actor.tempshow();
	addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("jump_out",0)),"cbFinishBeginLevel");
	snd.fplay("sounds/jump_out.audio");
}
void cbFinishBeginLevel() {
	blendstate=CONST_BLENDSTATE_NONE;
	state=CONST_STATE_IDLE;
	actor.setMaster("idle",1);
}
void e_deadend(float3 _newpos) {
	resetBlends();
	addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("jump_in",0)),"cbFinishDeadEndIn");
	wings.setMaster("jump_in",0);
	tail.setMaster("walk",0);
	blendstate=CONST_BLENDSTATE_DEADEND;
	snd.fplay("sounds/jump_in.audio");
}
void cbFinishDeadEndIn() {
	GOTO_POS.y( BASE_Y );
	this.setRot(hpr(GOTO_HEADING,0.0,0.0));
	this.setPos(GOTO_POS);
	addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("jump_out",0)),"cbFinishDeadEndOut");
	wings.setMaster("jump_out",0);
	tail.setMaster("walk",0);
	snd.fplay("sounds/jump_out.audio");
}
void cbFinishDeadEndOut() {
	blendstate=CONST_BLENDSTATE_NONE;	
	state=CONST_STATE_IDLE;
	actor.setMaster("idle",1);
	wings.setMaster("walk",1);
	tail.setMaster("walk",1);
	["ui.play"->heroJumpedOut()];
}

////// VICTORY //////

void setupDebrief() {
	resetBlends();
	float3 p=this.getPos();
	p.y( BASE_Y );
	this.setPos(p);

	actor.setTimeScale(1.0);
	wings.setTimeScale(1.0);
	tail.setTimeScale(1.0);
	
	if (isDragon()) {
		actor.blendClip("dragon_on",0);
		tail.blendClip("dragon_on",0);
		wings.blendClip("dragon_on",0);
	}
	else {
		actor.blendClip("ghost_off",0);
		tail.blendClip("ghost_off",0);
		wings.blendClip("ghost_off",0);
	}
	actor.tempShow();
	if (PU_WINGS) wings.tempShow();
	if (PU_TAIL) tail.tempShow();

	fx_dust.setService("emit.rate",0.0);
	fx_speed.setService("emit.rate",0.0);
	fx_speed.setService("emit.rate_var",0.0);
	fx_speed.setService("emit.rate_byspeed",0.0);
	fx_res.setService("emit.rate",0.0);		
	fire.setService("emit.rate",0.0);
	fire.setService("emit.rate_var",0.0);
	firehit.setService("emit.rate",0.0);
	dragonfx.setService("emit.rate",0.0);
	jumpland.setService("emit.rate",0.0);		
		
	pu_life.hide();
	pu_shoe.hide();
	pu_potion.hide();
	pu_rock.hide();
	icecube.tempHide();
	
	clearListeners(ON_TICK);
	addListener(ON_TICK,"tickvictory");	
	
	clearListeners(ON_ENTITYENTER);
	clearListeners(ON_ENTITYLEAVE);
	clearListeners(ON_TIMEOUT_FRAMES);
	clearListeners(ON_EVENT);	
}

void e_victory() {
	actor.setTimeScale(1.0);
	wings.setTimeScale(1.0);
	tail.setTimeScale(1.0);
	setupDebrief();
	fx_res.setService("emit.rate",0.5);
	snd.fplay("sounds/victory.audio");
	addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("victory",0)),"cbFinishResFX");	
	wings.setMaster("victory",0);
	tail.setMaster("victory",0);
}

/////// FAIL ///////
void e_fail() {
	setupDebrief();
	actor.setMaster("loose",0);
	wings.setMaster("loose",0);
	tail.setMaster("loose",0);
	snd.fplay("sounds/lose.audio");
}


/////// PROXIMITY //////
void e_water() {
	fCall("e_enter(water)");
}
void e_hit() {
	print(" hero.e_hit");
	if (blendstate==CONST_BLENDSTATE_ICECUBE) return;
	if (blendstate==CONST_BLENDSTATE_FALL) return;
	if ((blendstate!=CONST_BLENDSTATE_DEADEND)  && (BLINKTIME<=0.0) && (PU_GHOST==0)) {
		resetBlends();
		BLINKTIME=CONST_MAX_BLINKTIME;
		blendstate=CONST_BLENDSTATE_HIT;
		addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("attacked",0)),"cbFinishHit");
		wings.setMaster("attacked",0);				
		tail.setMaster("attacked",0);
		addListener(ON_TICK,"tickBlink");
		fx_hitstars.show();
		fx_hitstars.setMaster("hit",0);
		float3 p=this.getPos();
		p.y( BASE_Y );
		this.setPos(p);
		snd.fplay("sounds/attacked.audio"); 	
		snd.play("sounds/pingu-ballhit.audio"); 
		doDust(0);		
	} 
}
void e_enter(string _tag) {
	if (blendstate==CONST_BLENDSTATE_ICECUBE) return;
	if (blendstate==CONST_BLENDSTATE_FALL) return;
	
	if (_tag.startsWithNoCase("enemy")) {
		l_enemies.push(_tag);
		if (((JUMP_Y1+JUMP_Y2+JUMP_Y3)<0.5) && (blendstate!=CONST_BLENDSTATE_DEADEND)  && (BLINKTIME<=0.0) && (PU_GHOST==0) && ([_tag->isHit() as int]==0)) {
			if (TRAP_SKULL || isDragon()) {
				//make clunny to do attack animation
				Event e=Event("ATTACK",_tag);
				e.addArg(this.getPos());
				e.sendEvent();
				
				sendEvent("DRAGON_HIT",_tag);
				MOVE_DELAY=20.0;
				
				snd.fplay("sounds/bunny-attack.audio");
			}
			else {
				resetBlends();
				BLINKTIME=CONST_MAX_BLINKTIME;
				blendstate=CONST_BLENDSTATE_HIT;
				addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("attacked",0)),"cbFinishHit");
				wings.setMaster("attacked",0);				
				tail.setMaster("attacked",0);
				addListener(ON_TICK,"tickBlink");
				fx_hitstars.show();
				fx_hitstars.setMaster("hit",0);
				float3 p=this.getPos();
				p.y( BASE_Y );
				this.setPos(p);
				//make clunny to do attack animation
				Event e=Event("ATTACK",_tag);
				e.addArg(this.getPos());
				e.sendEvent();
				//inform ui.play script to remove carrots
				sendEvent("BUNNY_ISHIT","ui.play");
				snd.fplay("sounds/attacked.audio"); 
				doDust(0);
			}
		}
		else if ((JUMP_Y1+JUMP_Y2+JUMP_Y3)>0.5) {
			l_enemy_jumps.push(_tag);
		}
	}
	else if (_tag.startsWithNoCase("snowball")) {
		if (((JUMP_Y1+JUMP_Y2+JUMP_Y3)<1.0) && (blendstate!=CONST_BLENDSTATE_DEADEND) && (blendstate!=CONST_BLENDSTATE_ICECUBE) && (BLINKTIME<=0.0) && (PU_GHOST==0) && [_tag->isValid() as int]) {
			resetBlends();
			BLINKTIME=CONST_MAX_BLINKTIME;
			blendstate=CONST_BLENDSTATE_HIT;
			addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("attacked",0)),"cbFinishHit");
			wings.setMaster("attacked",0);				
			tail.setMaster("attacked",0);
			addListener(ON_TICK,"tickBlink");
			fx_hitstars.show();
			fx_hitstars.setMaster("hit",0);
			float3 p=this.getPos();
			p.y( BASE_Y );
			this.setPos(p);
			snd.fplay("sounds/attacked.audio"); 
			snd.play("sounds/pingu-ballhit.audio"); 
			doDust(0);	
		}
	}
	else if (_tag.startsWithNoCase("penguinsit")) {
	}
	else if (_tag.startsWithNoCase("penguin")) {
		l_penguins.push(_tag);
		if ((JUMP_Y1+JUMP_Y2+JUMP_Y3)>0.5) {
			l_penguin_jumps.push(_tag);
		}
		else {
			sendEvent("ROTATE",_tag);
		}
	}
	else if (_tag.startsWithNoCase("hedgehog")) {
		l_hogs.push(_tag);
		if ((JUMP_Y1+JUMP_Y2+JUMP_Y3)>1.0) {
			if ( ![_tag->hasNeedles() as int])  {
				l_hog_jumps.push(_tag);
			}
			
		}	
	}
	else if (_tag.startsWithNoCase("quicksand") || _tag.startsWithNoCase("jbridge")) {
		l_quicksands.push(_tag);			
		playTrapSound();
	}
	else if (_tag.startsWithNoCase("banana")) {
		if ((JUMP_Y1+JUMP_Y2+JUMP_Y3)<0.5 && [_tag->isActive() as int]) {
			//inform trap of impact
			Event e=Event("TRAP",_tag);
			e.addArg(GOTO_HEADING);
			e.sendEvent();
			
			doDust(0);
			resetBlends();
			blendstate=CONST_BLENDSTATE_TRAP;		
			addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("banana",0)),"cbFinishTrap");		
			wings.setMaster("banana",0);
			tail.setMaster("walk",0);
			GOTO_POS.y(BASE_Y);
			this.setPos(GOTO_POS);
			
			snd.fplay("sounds/trap-banana.audio");
		}
	}
	else if (_tag.startsWithNoCase("water")) {
			doDust(0);
			resetBlends();
			blendstate=CONST_BLENDSTATE_TRAP;			
			addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("banana",0)),"cbFinishTrap");		
			wings.setMaster("banana",0);
			tail.setMaster("walk",0);
			GOTO_POS.y(BASE_Y);
			
			snd.fplay("sounds/trap-banana.audio");	
	}
	else if (_tag.startsWithNoCase("skull")) {
		if ((JUMP_Y1+JUMP_Y2+JUMP_Y3)<0.5 && [_tag->isActive() as int]) {
			playTrapSound();
			skull.tempShow();
			skull.setPos(CONST_SKULL_POS);
			speed_mult=CONST_SPEED_SKULL;
			TRAP_SKULL=4;
			sendEvent("TRAP",_tag);
		}	
	}
	else if (_tag.startsWithNoCase("lighthouse")) {
		bCanJump=0;
		if ((JUMP_Y1+JUMP_Y2+JUMP_Y3)>2.0 && (blendstate==CONST_BLENDSTATE_NONE || blendstate==CONST_BLENDSTATE_JUMP)) {
			resetListeners();
			blendstate=CONST_BLENDSTATE_CRASH;
			JUMP_DELAY=25.0;
			addListener(ON_TIMEOUT_FRAMES,"35","cbFinishHit");
			actor.setMaster("crush",0);
			wings.setMaster("crush",0);
			tail.setMaster("crush",0);
			fx_hitstars.show();
			fx_hitstars.setMaster("hit",0);
			snd.fplay("sounds/trap-lighthouse.audio");
		}
	}
	else if (_tag.startsWithNoCase("duck")) {
		bCanJump=0;
		if ((JUMP_Y1+JUMP_Y2+JUMP_Y3)>2.0 && (blendstate==CONST_BLENDSTATE_NONE || blendstate==CONST_BLENDSTATE_JUMP) && [_tag->validate(0.0) as int]) {
			resetListeners();
			blendstate=CONST_BLENDSTATE_CRASH;
			JUMP_DELAY=25.0;
			addListener(ON_TIMEOUT_FRAMES,"35","cbFinishHit");
			actor.setMaster("crush",0);
			wings.setMaster("crush",0);
			tail.setMaster("crush",0);
			fx_hitstars.show();
			fx_hitstars.setMaster("hit",0);
			snd.fplay("sounds/trap-lighthouse.audio");
		}
	}	
}
void e_leave(string _tag) {
	if (_tag.startsWithNoCase("enemy")) {
		l_enemies.delIndex(l_enemies.findIndex(_tag));
	}
	else if (_tag.startsWithNoCase("hedgehog")) { 
		l_hogs.delIndex(l_hogs.findIndex(_tag));
	}
	else if (_tag.startsWithNoCase("penguinsit")) { 
	}
	else if (_tag.startsWithNoCase("penguin")) { 
		l_penguins.delIndex(l_penguins.findIndex(_tag));
	}
	else if (_tag.startsWithNoCase("quicksand") || _tag.startsWithNoCase("jbridge")) {
		l_quicksands.delIndex(l_quicksands.findIndex(_tag));
	}
	else if (_tag.startsWithNoCase("lighthouse")) {
		bCanJump=1;
	}
	else if (_tag.startsWithNoCase("duck")) {
		bCanJump=1;
	}
}
void cbFinishHit() {
	blendstate=CONST_BLENDSTATE_NONE;
	restoreFromBlend();
}
void cbFinishTrap() {
	blendstate=CONST_BLENDSTATE_NONE;
	restoreFromBlend();
}

//////// UI.PLAY INTERFACING //////

public int isOnTarget() {
	return int((this.getPos().xz()-GOTO_POS.xz()).length()<(speed*speed_mult*fElapsed()*1.1)) || (state==CONST_STATE_IDLE) || (hedgehog_scale<0.5);
}
public int isJumping() {
	return int((JUMP_Y1+JUMP_Y2+JUMP_Y3)>0.25);
}
public int isDoubleJumping() {
	return int((2.0*JUMP_TIMEOUT2)>CONST_MAX_JUMP_FRAMES2) || int((2.0*JUMP_TIMEOUT3)>CONST_MAX_JUMP_FRAMES3);
}
public int isGod() {
	return int( (BLINKTIME>0.0) || PU_GHOST || TRAP_SKULL);
}
public int hasWings() { return PU_WINGS; }
public int hasTail()  { return PU_TAIL ; }
public int hasFire()  { return PU_FIRE ; }
public int isMoving() {
	if (state != CONST_STATE_WALK) { return 0; }
	if ((blendstate==CONST_BLENDSTATE_DEADEND) ||
		(blendstate==CONST_BLENDSTATE_TRAP)    ||
		(blendstate==CONST_BLENDSTATE_HIT)     ||
		(blendstate==CONST_BLENDSTATE_ICECUBE) ||
		(blendstate==CONST_BLENDSTATE_CRASH)   ||
		(blendstate==CONST_BLENDSTATE_FALL)
		) { return 0; }
	return 1;
}

// direction control

public string getStepWidth() { return string(speed*speed_mult*fElapsed()*1.2)); }
public string getDir() { return orientation; }
public string getNextDir() { return next_orientation; }

public string getForceNext() { return force_next; }
public string resetForceNext() { 
	force_next_count=max(0,force_next_count-1);
	if (force_next_count<=0) force_next=""; 
}
public void setForceNext(string _dir) { force_next=_dir;force_next_count++; }

public void setDir(string _dir) { 
	orientation=_dir; 
}
public void setNextDir(string _dir) { next_orientation=_dir; }
public void setNextLeft() { 
	if (orientation=="N") {
		next_orientation="W";
	}
	else if (orientation=="S") {
		next_orientation="E";
	}
	else if (orientation=="W") {
		next_orientation="S";
	}
	else if (orientation=="E") {
		next_orientation="N";	
	}
}
public void setNextRight() { 
	if (orientation=="N") {
		next_orientation="E";
	}
	else if (orientation=="S") {
		next_orientation="W";
	}
	else if (orientation=="W") {
		next_orientation="N";
	}
	else if (orientation=="E") {
		next_orientation="S";	
	}
}
public void setNextBack() { 
	if (orientation=="N") {
		next_orientation="S";
	}
	else if (orientation=="S") {
		next_orientation="N";
	}
	else if (orientation=="W") {
		next_orientation="E";
	}
	else if (orientation=="E") {
		next_orientation="W";	
	}
}
public void setBaseY(float _y) { BASE_Y=_y; }
public void setSpeedTrap(float _s) { speed_trap=_s; }
public void setJumpScale(float _s) { jump_scale=_s; }
public void setTimeScale(float _s) { actor.setTimeScale(_s); }

////// TICK ////////

///// TICKING /////
void tickBlink() {
	BLINKTIME-=fElapsed();
	if (BLINKTIME<=0.0) {
		actor.tempShow();
		if (PU_WINGS) wings.tempShow();
		if (PU_TAIL) tail.tempShow();
		delListener(ON_TICK,"tickBlink");
		return;
	}
	if (int((int(CONST_MAX_BLINKTIME)-int(BLINKTIME))%5)) {
		actor.tempShow();
		if (PU_WINGS) wings.tempShow();
		if (PU_TAIL) tail.tempShow();
	}
	else {
		actor.tempHide();
		wings.tempHide();
		tail.tempHide();
	}
}
void tickVictory() {
	CAM_TAR_OFFSET.z( lerp(CAM_TAR_OFFSET.z(),CAM_Z_IDLE,fElapsed()*0.1) );
	CAM_FOV=lerp(CAM_FOV,CAM_DEF_FOV,fElapsed()*0.1);
	TARGETCAMORBIT_H=0;	
	PREVCAMORBIT_H=lerp(PREVCAMORBIT_H,TARGETCAMORBIT_H,fElapsed()*0.1);
	commitCam();
}
void tick() {
	float TURN_ANGLE=clamp(minangle(GOTO_HEADING,this.getRot().h()),0.0,180.0)/180.0;
	//update camera
	TARGETCAMORBIT_H=0;	
	if (isMoving() && (TRAP_SKULL==0)) {
		CAM_TAR_OFFSET.z( lerp(CAM_TAR_OFFSET.z(),CAM_Z_RUN,fElapsed()*0.1) );
		if (orientation=="N") {
			if (next_orientation=="E") TARGETCAMORBIT_H=-30.0;	
			else if (next_orientation=="W") TARGETCAMORBIT_H=30.0;	
		}
		else if (orientation=="S") {
			if (next_orientation=="E") TARGETCAMORBIT_H=30.0;	
			else if (next_orientation=="W") TARGETCAMORBIT_H=-30.0;			
		}
		else if (orientation=="W") {
			if (next_orientation=="N") TARGETCAMORBIT_H=-30.0;	
			else if (next_orientation=="S") TARGETCAMORBIT_H=30.0;			
		}
		else if (orientation=="E") {
			if (next_orientation=="N") TARGETCAMORBIT_H=30.0;	
			else if (next_orientation=="S") TARGETCAMORBIT_H=-30.0;			
		}		
	}
	else {
		CAM_TAR_OFFSET.z( lerp(CAM_TAR_OFFSET.z(),CAM_Z_IDLE,fElapsed()*0.1) );
	}
	//fov
	if (speed_mult>1.0)  CAM_FOV=lerp(CAM_FOV,CAM_ACTION_FOV,fElapsed()*0.1);
	else CAM_FOV=lerp(CAM_FOV,CAM_DEF_FOV,fElapsed()*0.1);
	//orbit
	PREVCAMORBIT_H=lerp(PREVCAMORBIT_H,TARGETCAMORBIT_H,fElapsed()*0.1);
	commitCam();
		
	if (blendstate==CONST_BLENDSTATE_DEADEND) { return; }
	
	//// SNOW - CRACK FALL ANIMATION
	float3 heropos=this.getPos();	
	if (blendstate==CONST_BLENDSTATE_ICECUBE) { 
		aux_timer+=fElapsed();
		//goto center
		if (aux_state==0) {
			float icespeed=speed*speed_mult*fElapsed();
			if (length( (heropos-auxtile_pos).xz() ) <= icespeed) {		
				heropos.xz( auxtile_pos.xz() );
				aux_state++;
				aux_timer=0;
				snd.play("sounds/trap-icecrack-fall.audio");
			}
			else {
				float3 dir = auxtile_pos - heropos;
				dir.y(0.0);
				dir.normalize();
				heropos=heropos+dir*icespeed;
			}			
		}
		//wait running
		else if (aux_state==1) {
			if (aux_timer>=(getFPS()*0.5)) {
				aux_state++;
				aux_timer=0;
				actor.setTimeScale(0.0);
				wings.setTimeScale(0.0);
				tail.setTimeScale(0.0);
			}
			return;
		}
		//wait
		else if (aux_state==2) {
			if (aux_timer>=(getFPS()*0.3)) {
				aux_state++;
				aux_timer=0;
				aux_y=heropos.y();				
			}
			return;
		}
		//decend
		else if (aux_state==3) {
			float frames=getFps()*0.1;
			if (aux_timer>frames) {
				aux_state++;
				aux_timer=0;
				icecube.tempShow();
			}
			else {
				heropos.y( lerp(aux_y,-1.0,aux_timer/frames) );
			}
		}
		//ascend
		else if (aux_state==4) {			
			float frames=getFps()*2;
			if (aux_timer>=frames) {
				aux_state++;
			}
			else {
				heropos.y( lerp(-1.0,BASE_Y,aux_timer/frames) );
			}
		}
		//float
		else {
			heropos.y( BASE_Y+0.125*cos(aux_timer*0.125)  );
		}
		if (aux_state>=4) {
			float3 rot=actor.getRot();
			rot.z(cos(aux_timer*0.2)*4.0);
			actor.setRot(rot);
		}
		this.setPos(heropos);
		return; 
	}
	
	//// JUNGLE - FALL aNIMATION
	if (blendstate==CONST_BLENDSTATE_FALL) { 
		aux_timer+=fElapsed();
		//goto center
		if (aux_state==0) {
			float auxspeed=speed*speed_mult*fElapsed();
			if (length( (heropos-auxtile_pos).xz() ) <= auxspeed) {		
				heropos.xz( auxtile_pos.xz() ); aux_state++; aux_timer=0;
				snd.play("sounds/trap-icecrack-fall.audio");
			}
			else {
				float3 dir = auxtile_pos - heropos; dir.y(0.0); dir.normalize();
				heropos=heropos+dir*auxspeed;
			}			
		}
		//wait running
		else if (aux_state==1) {
			if (aux_timer>=(getFPS()*0.5)) {
				aux_state++; aux_timer=0; 
				actor.setTimeScale(0.0); wings.setTimeScale(0.0); tail.setTimeScale(0.0);
			}
			return;
		}
		//wait
		else if (aux_state==2) {
			if (aux_timer>=(getFPS()*0.3)) {
				aux_state++; aux_timer=0;aux_y=heropos.y();				
			}
			return;
		}
		//decend
		else if (aux_state==3) {
			float frames=getFps()*0.1;
			if (aux_timer>frames) {
				aux_state++;aux_timer=0;
			}
			else {
				heropos.y( lerp(aux_y,-1.0,aux_timer/frames) );
				actor.setTimeScale(0.5); wings.setTimeScale(0.5); tail.setTimeScale(0.5);
			}
		}
		this.setPos(heropos);
		return; 
	}
	////
	
	if (JUMP_DELAY>=0) JUMP_DELAY-=fElapsed();

	//jump
	if (JUMP_DELAY<=0) {
		if (JUMP_TIMEOUT1>0) {
			if (TRAP_SKULL)	JUMP_TIMEOUT1-=fElapsed()*2.0;
			else JUMP_TIMEOUT1-=fElapsed();
			if (JUMP_TIMEOUT1<JUMP_THRES1) {
				JUMP_TIMEOUT1=0;
				if (JUMP_TIMEOUT2<=0) {
					doDust(1);
				}
			}
			JUMP_Y1=sin(radians((JUMP_TIMEOUT1/CONST_MAX_JUMP_FRAMES1)*180.0))*JUMP_HEIGHT1*jump_scale;
			if (TRAP_SKULL) {
				skull.setPos(CONST_SKULL_POS+float3(JUMP_Y1*(4.0-TRAP_SKULL)*1.5,0.0,0.0));
			}
		}
		else {
			JUMP_Y1=0;
		}
		if (JUMP_TIMEOUT2>0) {
			JUMP_TIMEOUT2-=fElapsed();
			if (JUMP_TIMEOUT2<JUMP_THRES2) {
				JUMP_TIMEOUT2=0;
				doDust(1);
			}
			JUMP_Y2=sin(radians((JUMP_TIMEOUT2/CONST_MAX_JUMP_FRAMES2)*180.0))*JUMP_HEIGHT2*jump_scale;
		}
		else {
			JUMP_Y2=0;
		}
		if (JUMP_TIMEOUT3>0) {
			JUMP_TIMEOUT3-=fElapsed();
			if (JUMP_TIMEOUT3<JUMP_THRES3) {
				JUMP_TIMEOUT3=0;
				doDust(1);
			}
			JUMP_Y3=sin(radians((JUMP_TIMEOUT3/CONST_MAX_JUMP_FRAMES3)*180.0))*JUMP_HEIGHT3*jump_scale;
		}
		else {
			JUMP_Y3=0;
		}	
	}
	float JUMP_Y=min(JUMP_Y1+JUMP_Y2+JUMP_Y3,4.7);

	hedgehog_scale=1.0;
	int must_force_stop=0;
	int num=l_hogs.num();
	string tag;
	if (num) {
		float gotoangle=lookat(heropos,GOTO_POS).h();
		float heading=this.getRot().h();
		int i;
		for (i=0;i<num;i++) {
			tag=l_hogs.get(i);
			if (![tag->isHit() as int]) {
				if ([tag->hasNeedles() as int] || (JUMP_Y<1.0)) { 
					[tag->validateBunny(heropos.xz(),heading)];
					float hogangle=[tag->getBunnyAngle(heropos.xz()) as float];
					if (!["hero"->isDoubleJumping() as int]){ //if we are double jumping, we can pass above the needles
						if (minangle(hogangle,gotoangle)<90.0) { 
							hedgehog_scale=0.0;
							if ( (orientation=="N" && next_orientation=="S") ||
								 (orientation=="S" && next_orientation=="N") || 
								(orientation=="W" && next_orientation=="E") ||
								(orientation=="E" && next_orientation=="W") ) 
							{											
								["ui.play"->forceBunnyBack()];
							}	
							else if (state!=CONST_STATE_IDLE) {
								e_stop();
								sendEvent("FORCESTOP","ui.play");
							}
							break;
						}
					}
				}
			} 
		}
	}
	float3 oldp = heropos;
	float3 target_pos=GOTO_POS;
	float heading;	
	
	if (blendstate==CONST_BLENDSTATE_TRAP || 
		blendstate==CONST_BLENDSTATE_HIT) 
	{
		target_pos=oldp;
	}

	float curspeed=hedgehog_scale*speed*speed_mult*speed_trap*fElapsed()*max(0.1,(1.0-TURN_ANGLE));
	if (MOVE_DELAY>=0) {
		MOVE_DELAY-=fElapsed();
		curspeed=0;
	}
	if ((blendstate==CONST_BLENDSTATE_FALL)||(blendstate==CONST_BLENDSTATE_CRASH) || (bCanJump==0 && JUMP_Y>0.1)) {
		curspeed=0.0;
	}
	
	if (length( (oldp-target_pos).xz() ) <= curspeed) {		
		target_pos.y( BASE_Y+JUMP_Y );
		heading = GOTO_HEADING;
		float3 dir = target_pos - oldp;
		this.setPos(oldp+dir*fElapsed());
	}
	else {
		heading = normalizeAngle(oldp.lookat(target_pos).h);
		float3 dir = target_pos - oldp;
		dir = float3(dir.x,0,dir.z);
		dir.normalize();
		float3 fin = oldp + dir*curspeed;
		fin.y( BASE_Y+JUMP_Y );
		this.setPos(fin);
	}
	
	float3 r = this.getRot();
	if (minangle(heading,r.h) > 0.1)
	{
		float tar_h = turnangle(r.h,heading,abs(minangle(r.h,heading))*0.25*fElapsed());
		this.setRot(hpr(normalizeAngle(tar_h),0.0,0.0));
	}
	else
	{
		this.setRot(hpr(heading,0.0,0.0));
	}		
}
 
/////// BRANDING //////

void cbDragonFX() {
	dragonfx.setService("emit.rate",0.0);
}
void enableDragon() {
	if (PU_GHOST) {
		actor.blendClip("dragonghost_on",0);
		tail.blendClip("dragonghost_on",0);
		wings.blendClip("dragonghost_on",0);
	}
	else {
		actor.blendClip("dragon_on",0);
		tail.blendClip("dragon_on",0);
		wings.blendClip("dragon_on",0);
	}
	if (SNAPDRAGON) {
		addListener(ON_TIMEOUT_FRAMES,"1","switchToSnapdragon");
	}
	actor.setScale(float3(0.7,0.7,0.7));	
	wings.setScale(float3(1.2,1.0,1.0));	
	dragonfx.setService("emit.rate",50.0);
	addListener(ON_TIMEOUT_FRAMES,"110","cbDragonFX");
}
void switchToSnapdragon() {
	["ui.play"->switchToSnapdragon()];
}
void e_wings() {
	if (!PU_WINGS) {
		snd.fplay("sounds/dragon-item.audio"); 
		PU_WINGS=1;
		e_happy();
		wings.tempshow();
		addListener(ON_TIMEOUT_FRAMES,string(wings.blendClip("grow",0)),"cbWingsGrow");
		if (isDragon()) {
			enableDragon();
		}
		else {
			dragonfx.setService("emit.rate",10.0);
			addListener(ON_TIMEOUT_FRAMES,"30","cbDragonFX");
		}
	}
}
void e_tail() {
	if (!PU_TAIL) {
		snd.fplay("sounds/dragon-item.audio"); 
		PU_TAIL=1;
		e_happy();
		tail.tempshow();
		addListener(ON_TIMEOUT_FRAMES,string(tail.blendClip("grow",0)),"cbTailGrow");
		if (isDragon()) {
			enableDragon();
		}
		else {
			dragonfx.setService("emit.rate",10.0);
			addListener(ON_TIMEOUT_FRAMES,"30","cbDragonFX");
		}
	}
}
void e_fire() {
	if (!PU_FIRE) {
		snd.fplay("sounds/dragon-item.audio"); 
		PU_FIRE=1;
		e_happy();
		fire.setService("emit.rate",0.8);
		fire.setService("emit.rate_var",0.3);
		if (isDragon()) {
			enableDragon();
		}
		else {
			dragonfx.setService("emit.rate",10.0);
			addListener(ON_TIMEOUT_FRAMES,"30","cbDragonFX");
		}				
	}
}
void cbWingsGrow() {
	wings.setMaster("walk",1);
}
void cbTailGrow() {
	tail.setMaster("walk",1);
}
