 /****************************************************************************
 * This file is part of Bunny Maze Application                                *
 *                                                                           *
 * Copyright (c) 2011, Eyelead Software. All rights reserved.                *
 * PROPRIETARY AND CONFIDENTIAL                                              *
 *                                                                           *
 * This file is subject to the terms and conditions of the                   *
 * Commercial Software License Agreement by and between                      *
 * Qualcomm Incorporated and Eyelead Software S.A                            *
 * as made and entered on September 30th, 2011                               *
 *****************************************************************************/

#using stdlib game
#hive 1


String APPNAME = "bunnymaze";

//if script is used for preloading
input int PRELOADER=0;

input float speed = 0.2;
input float speed_mult=1.0;
input string theme="forest";
float speed_trap;

vfile actor,fx_hitstars;
Resource res;
int state,blendstate;
float3 GOTO_POS,PREV_GOTO_POS;
float BASE_Y;

int CONST_STATE_IDLE=0;
int CONST_STATE_WALK=1;

int CONST_BLENDSTATE_NONE=0;
int CONST_BLENDSTATE_ATTACK=1;
int CONST_BLENDSTATE_HIT=2;

int bLoaded=0;

string force_next;
int force_next_count;

void defaults() {
	force_next="";
	force_next_count=0;

	
	speed_trap=1.0;
	state=0;
	blendstate=0;
	BASE_Y=2.33;

	this.setChannel("col.height",3.0);
	this.setChannel("col.radius",0.5);
	this.setChannel("prox.radius",0.7);
	fx_hitstars.hide();
	
	//assuming that the vfile is an actor object, set its initial animation
	actor.setMaster("idle",1);
	//make vfile visible
	actor.show();
	actor.setScale(float3(0.7,0.7,0.7));
	actor.setPos(float3(0,0,0));
	actor.setTimeScale(1.0);

	GOTO_POS=this.getPos();
	PREV_GOTO_POS=GOTO_POS;
	
	clearListeners(ON_TICK);
	clearListeners(ON_EVENT);	
	clearListeners(ON_TIMEOUT_FRAMES);
	clearListeners(ON_ENTITYENTER);
	clearListeners(ON_ENTITYLEAVE);
}

void init()
{
	if (PRELOADER) {
		debugLog("Preloading enemy "+theme); 
		setTag("dummy");
		res=Resource("levelsets/"+theme+"/enemy."+theme+".hvfs","");	
		return;
	}

	bLoaded=0;

	//set entity tag
	debugLog("Loading enemy: "+getTag());
	
	clearScene(this.path()+"/fx");
	system("mkdir "+this.path()+"/fx");
	loadScene("levelsets/generic/hitstars.hvfs",this.path()+"/fx","cbHitstarsLoaded");	
}
void cbHitstarsLoaded() {
	fx_hitstars=vfopen(this.path()+"/fx/hitstars.actor","r");
	res=Resource("levelsets/"+theme+"/enemy."+theme+".hvfs","cbResLoaded");	
}
void cbResLoaded() {
	actor = res.use("enemy.actor","reset");
}
public void regEvents() {
	addListener(ON_EVENT,"GOTO","e_goto");
	addListener(ON_EVENT,"STOP","e_stop");
	addListener(ON_EVENT,"HIT","e_hit");
	addListener(ON_EVENT,"DRAGON_HIT","e_dragonhit");
	addListener(ON_EVENT,"ATTACK","e_attack");
	addListener(ON_EVENT,"SNOWBALL","e_snowball");
	addListener(ON_TICK,"tick");
}
public void reset() {
	if (bLoaded) {
		debugLog("Reset "+getTag());
	}
	else {
		bLoaded=1;
		debugLog(getTag()+" loaded");
	}
	defaults();
}
public string isLoaded() {
	return string(bLoaded);
}
public void savestate() {
	appPropertySet(APPNAME,"ENEMY_POS:"+getTag(),string(this.getPos()));
	appPropertySet(APPNAME,"ENEMY_ROT:"+getTag(),string(this.getRot()));
	appPropertySet(APPNAME,"ENEMY_GOTO:"+getTag(),string(GOTO_POS));
	appPropertySet(APPNAME,"ENEMY_STATE:"+getTag(),string(state));
	appPropertySet(APPNAME,"ENEMY_FORCENEXT:"+getTag(),force_next);
	appPropertySet(APPNAME,"ENEMY_FORCENEXTCOUNT:"+getTag(),string(force_next_count));
}
public void loadstate() {
	this.setPos(float3(appPropertyGet(APPNAME,"ENEMY_POS:"+getTag(),string(this.getPos()))));
	this.setRot(float3(appPropertyGet(APPNAME,"ENEMY_ROT:"+getTag(),string(this.getRot()))));
	GOTO_POS=float3(appPropertyGet(APPNAME,"ENEMY_GOTO:"+getTag(),string(GOTO_POS)));
	state=int(appPropertyGet(APPNAME,"ENEMY_STATE:"+getTag(),string(state)));
	force_next=appPropertyGet(APPNAME,"ENEMY_FORCENEXT:"+getTag(),force_next);
	force_next_count=int(appPropertyGet(APPNAME,"ENEMY_FORCENEXTCOUNT:"+getTag(),string(force_next_count)));
	fCall("cbFinishAttack");
}
void e_attack(float3 _enemypos) {
	this.setRot(hpr(this.getPos().lookat(_enemypos).h(),0.0,0.0));
	addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("attack",0)-7),"cbFinishAttack");
	blendstate=CONST_BLENDSTATE_ATTACK;
}
void cbFinishAttack() {
	blendstate=CONST_BLENDSTATE_NONE;
	if (state==CONST_STATE_IDLE) { actor.setMaster("idle",1); }
	else if (state==CONST_STATE_WALK) { actor.setMaster("walk",1); }
}
void e_hit() {
	if (blendstate!=CONST_BLENDSTATE_HIT) {
		//inform gameplay script that clunny is down
		blendstate=CONST_BLENDSTATE_HIT;
		delListener(ON_TIMEOUT_FRAMES,"cbFinishHit");
		delListener(ON_TIMEOUT_FRAMES,"cbFinishAttack");
		addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("fall",0)),"cbFinishHit");	
		fx_hitstars.show();
		fx_hitstars.setMaster("hitclunny",0);
	}
}
void e_snowball() {
	if (blendstate!=CONST_BLENDSTATE_HIT) {
		//inform gameplay script that clunny is down
		blendstate=CONST_BLENDSTATE_HIT;
		delListener(ON_TIMEOUT_FRAMES,"cbFinishHit");
		delListener(ON_TIMEOUT_FRAMES,"cbFinishAttack");
		addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("fall",0)),"cbFinishHit");	
		fx_hitstars.show();
		fx_hitstars.setMaster("hitclunny",0);
	}
}
void e_dragonhit() {
	
	addListener(ON_TIMEOUT_FRAMES,"8","e_hit");
}
void cbFinishHit() {
	cbFinishAttack();
}
void e_stop() {
	GOTO_POS=this.getPos();
	if (state!=CONST_STATE_IDLE) {
		if (blendstate==CONST_BLENDSTATE_NONE) {
			actor.setMaster("idle",1);
		}
		state=CONST_STATE_IDLE;
	}
}
void e_goto(float3 _newpos) {
	PREV_GOTO_POS=GOTO_POS;
	GOTO_POS=_newpos;
	if (state!=CONST_STATE_WALK) {
		if (blendstate==CONST_BLENDSTATE_NONE) actor.setMaster("walk",1);
		state=CONST_STATE_WALK;
	}
}
public int doValidateLog(string _tag) {
	if (![_tag->validate(this.getRot().h()) as int]) {
		e_goto(PREV_GOTO_POS);
		return 0;
	}
	return 1;
}

//AI
public string getStepWidth() { return string(speed*speed_mult*fElapsed()*1.2)); }
public float3 getGOTO() {
	return GOTO_POS;
}
public float3 getPREVGOTO() {
	return PREV_GOTO_POS;
}
public string getDir() {
	int heading=int(normalizeangle(this.getRot().h()));
	if (heading>45 && heading<135) return "W";
	if (heading>=135 && heading<225) return "S";
	if (heading>=225 && heading<315) return "E";
	return "N";
}
public int isOnTarget() {
	return int((this.getPos().xz()-GOTO_POS.xz()).length()<(speed*speed_mult*speed_trap*fElapsed()*1.1));
}
public int isHit() {
	return int(blendstate==CONST_BLENDSTATE_HIT);
}
public int isHitable() {
	return !isHit();
}
void tick() {
	if (blendstate==CONST_BLENDSTATE_HIT || blendstate==CONST_BLENDSTATE_ATTACK) return;

	float3 oldp = this.getPos();
	if (length( (oldp-GOTO_POS).xz() ) <= speed*speed_mult*speed_trap*fElapsed()) {
		this.setPos(float3(GOTO_POS.x(),BASE_Y,GOTO_POS.z()));
	}
	else {
		float CUR_HEADING = normalizeAngle(oldp.lookat(GOTO_POS).h);
		float3 dir = GOTO_POS - oldp;
		dir = float3(dir.x,0,dir.z);
		dir.normalize();

		float3 fin = oldp + dir*(speed*speed_mult*speed_trap*fElapsed());
		fin.y( BASE_Y );
		this.setPos(fin);
		
		float3 r = this.getRot();
		if (minangle(CUR_HEADING,r.h) > 5)
		{
			float tar_h = turnangle(r.h,CUR_HEADING,abs(minangle(r.h,CUR_HEADING))*0.25*fElapsed());
			this.setRot(hpr(normalizeAngle(tar_h),0.0,0.0));
		}
		else
		{
			this.setRot(hpr(CUR_HEADING,0.0,0.0));
		}			
	}
}
public void setBaseY(float _y) { BASE_Y=_y; }
public void setSpeedTrap(float _s) { speed_trap=_s; }
public void setTimeScale(float _s) { actor.setTimeScale(_s); }

public string getForceNext() { return force_next; }
public string resetForceNext() { 
	force_next_count=max(0,force_next_count-1);
	if (force_next_count<=0) force_next=""; 
}
public void setForceNext(string _dir) { force_next=_dir;force_next_count++; }


