 /****************************************************************************
 * This file is part of Bunny Maze Application                                *
 *                                                                           *
 * Copyright (c) 2011, Eyelead Software. All rights reserved.                *
 * PROPRIETARY AND CONFIDENTIAL                                              *
 *                                                                           *
 * This file is subject to the terms and conditions of the                   *
 * Commercial Software License Agreement by and between                      *
 * Qualcomm Incorporated and Eyelead Software S.A                            *
 * as made and entered on September 30th, 2011                               *
 *****************************************************************************/

#using stdlib game
#hive 1

String APPNAME = "bunnymaze";

//if script is used for preloading
input int PRELOADER=0;

input float speed = 0.05;
input float speed_mult=1.0;
float speed_trap;

vfile actor,fx_hitstars;
Resource res;
int state;
float3 GOTO_POS,PREV_GOTO_POS;
float BASE_Y;

int CONST_STATE_IDLE=0;
int CONST_STATE_WALKDOWN=1;
int CONST_STATE_SPIKESUP=2;
int CONST_STATE_WALKUP=3;
int CONST_STATE_SPIKESDOWN=4;
int CONST_STATE_HIT=5;
int CONST_STATE_RECOVER=6;

int bLoaded=0,bNeedles,bBunnyInside;

string force_next;
int force_next_count,inside_counter;

void defaults() {
	bBunnyInside=0;
	speed_trap=1.0;
	state=0;
	BASE_Y=2.33;
	bNeedles=0;
	force_next="";
	force_next_count=0;
	inside_counter=0;
	
	this.setChannel("col.height",3.0);
	this.setChannel("col.radius",1.1);
	this.setChannel("prox.radius",2.0);
	
	fx_hitstars.hide();
	
	GOTO_POS=this.getPos();
	PREV_GOTO_POS=GOTO_POS;

	//assuming that the vfile is an actor object, set its initial animation
	actor.setMaster("idle",1);
	//make vfile visible
	actor.show();
	actor.setScale(float3(0.5,0.5,0.5));	
	actor.setTimeScale(1.0); 
	
	clearListeners(ON_TICK);
	clearListeners(ON_EVENT);	
	clearListeners(ON_TIMEOUT_FRAMES);
	clearListeners(ON_ENTITYENTER);
	clearListeners(ON_ENTITYLEAVE);
}

void init()
{
	if (PRELOADER) {
		debugLog("Preloading Hedgehog");
		setTag("dummy");
		res=Resource("levelsets/traps/hedgehog.hvfs","");	
		return;
	}
	bLoaded=0;

	//set entity tag
	debugLog("Loading enemy: "+getTag());

	clearScene(this.path()+"/fx");
	system("mkdir "+this.path()+"/fx");
	loadScene("levelsets/generic/hitstars.hvfs",this.path()+"/fx","cbHitstarsLoaded");	
}
void cbHitstarsLoaded() {
	fx_hitstars=vfopen(this.path()+"/fx/hitstars.actor","r");
	res=Resource("levelsets/traps/hedgehog.hvfs","cbResLoaded");	
}
void cbResLoaded() {
	actor = res.use("hedgehog.actor","reset");
}
public void reset() {
	if (bLoaded) {
		debugLog("Reset "+getTag());
	}
	else {
		bLoaded=1;
		debugLog(getTag()+" loaded");
	}
	defaults();
}
public void regEvents() {
	addListener(ON_ENTITYENTER,"e_enter");
	addListener(ON_ENTITYLEAVE,"e_leave");
	addListener(ON_EVENT,"GOTO","e_goto");
	addListener(ON_EVENT,"STOP","e_stop");
	addListener(ON_EVENT,"HIT","e_hit");
	addListener(ON_EVENT,"FORCE_HIT","cbHit");
	addListener(ON_TICK,"tick");
}
public string isLoaded() {
	return string(bLoaded);
}
public void savestate() {
	appPropertySet(APPNAME,"ENEMY_POS:"+getTag(),string(this.getPos()));
	appPropertySet(APPNAME,"ENEMY_ROT:"+getTag(),string(this.getRot()));
	appPropertySet(APPNAME,"ENEMY_GOTO:"+getTag(),string(GOTO_POS));
	appPropertySet(APPNAME,"ENEMY_STATE:"+getTag(),string(state));
}
public void loadstate() {
	this.setPos(float3(appPropertyGet(APPNAME,"ENEMY_POS:"+getTag(),string(this.getPos()))));
	this.setRot(float3(appPropertyGet(APPNAME,"ENEMY_ROT:"+getTag(),string(this.getRot()))));
	GOTO_POS=float3(appPropertyGet(APPNAME,"ENEMY_GOTO:"+getTag(),string(GOTO_POS)));
	state=int(appPropertyGet(APPNAME,"ENEMY_STATE:"+getTag(),string(state)));
	
	if (state==CONST_STATE_WALKDOWN) fCall("cbWalkDown");
	else if (state==CONST_STATE_SPIKESUP) fCall("cbSpikesUp");
	else if (state==CONST_STATE_WALKUP) fCall("cbWalkUp");
	else if (state==CONST_STATE_SPIKESDOWN) fCall("cbSpikesDown");
	else if (state==CONST_STATE_RECOVER) fCall("cbHitWait");
	else if (state==CONST_STATE_HIT) fCall("cbHit");
}
void e_stop() {
	GOTO_POS=this.getPos();
	if (state!=CONST_STATE_IDLE) {
		actor.setMaster("idle",1);
		state=CONST_STATE_IDLE;
		clearListeners(ON_TIMEOUT_FRAMES);
	}
}
void e_goto(float3 _newpos) {
	PREV_GOTO_POS=GOTO_POS;
	GOTO_POS=_newpos;
	if (state==CONST_STATE_IDLE) {
		clearListeners(ON_TIMEOUT_FRAMES);
		fCall("cbWalkDown");
	}
}
void cbNeedlesUp() {
	bNeedles=1;
}
void cbNeedlesDown() {
	bNeedles=0;
}
void cbSpikesUp() {
	if (bBunnyInside) {
		fCall("cbWalkDown");
		return;
	}

	int frames=int(actor.setMaster("spikes_up",0));
	clearListeners(ON_TIMEOUT_FRAMES);
	addListener(ON_TIMEOUT_FRAMES,string(frames/2),"cbNeedlesUp");
	addListener(ON_TIMEOUT_FRAMES,string(frames),"cbWalkUp");
	state=CONST_STATE_SPIKESUP;
}
void cbWalkUp() {
	actor.setMaster("walk2",1);
	clearListeners(ON_TIMEOUT_FRAMES);
	addListener(ON_TIMEOUT_FRAMES,string((1+rand(2))*getFPS()),"cbSpikesDown");
	state=CONST_STATE_WALKUP;
	bNeedles=1;
}
void cbSpikesDown() {	
	clearListeners(ON_TIMEOUT_FRAMES);
	addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("spikes_down",0)),"cbWalkDown");
	state=CONST_STATE_SPIKESDOWN;
}
void cbWalkDown() {
	actor.setMaster("walk",1);
	clearListeners(ON_TIMEOUT_FRAMES);
	addListener(ON_TIMEOUT_FRAMES,string((5+rand(5))*getFPS()),"cbSpikesUp");
	state=CONST_STATE_WALKDOWN;
	bNeedles=0;
}
void cbHit() {	
	fx_hitstars.show();
	fx_hitstars.setMaster("hit",0);
	clearListeners(ON_TIMEOUT_FRAMES);
	addListener(ON_TIMEOUT_FRAMES,string(max(60,actor.setMaster("fall",0))),"cbHitWait");
	state=CONST_STATE_HIT;
	bNeedles=0;
}
void cbHitWait() {	
	if (inside_counter) addListener(ON_TIMEOUT_FRAMES,"10","cbHitWait");
	else {
		state=CONST_STATE_RECOVER;
		addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("recover",0)),"cbWalkDown");
	}
}
void e_hit() {
	if (bNeedles || (state==CONST_STATE_HIT)) return;
	cbHit();
}
public int isHitable() {
	return ((!bNeedles) && (state!=CONST_STATE_HIT));
}
public int isHit() {
	return int(state==CONST_STATE_HIT);
}
public int hasNeedles() {
	return bNeedles;
}
void e_enter(string _tag) {	
	if (_tag=="hero") {
		inside_counter++;
		if (state==CONST_STATE_HIT) return;
		if (["hero"->isJumping() as int] && (state==CONST_STATE_WALKDOWN)) {
			cbSpikesUp();
		} else {
			["hero"->playTrapSound()];
		}
	} 
	else if (_tag.startsWithNoCase("enemy")) {
		inside_counter++;
		if (state==CONST_STATE_HIT) return;		
		Event e=Event("ATTACK",_tag);
		e.addArg(this.getPos());
		e.sendEvent();	
		cbHit();
		["hero"->playAttackSound(this.getPos().xz())];
	}
}
void e_leave(string _tag) { 
	if (_tag=="hero") {
		inside_counter--;
		bBunnyInside=0;
	}
	else if (_tag.startsWithNoCase("enemy")) {
		inside_counter--;	
	}
}
public int doValidateLog(string _tag) {
	if (![_tag->validate(this.getRot().h()) as int]) {
		e_goto(PREV_GOTO_POS);
		return 0;
	}
	return 1;
}
public string getBunnyAngle(float2 _xz) { 
	return string(lookat(float3(_xz.x(),BASE_Y,_xz.y()),this.getPos()).h());
}
public void validateBunny(float2 _xz, float _heading) {
	if (state==CONST_STATE_HIT) return;
	float lookangle=lookat(float3(_xz.x(),BASE_Y,_xz.y()),this.getPos()).h();
	//float bunnyangle=minangle( lookat(float3(_x,BASE_Y,_z),this.getPos()).h(), _heading);
	int ret=(minangle(lookangle,_heading)>45.0);
	if (!ret) {
		if (minangle(lookangle,this.getRot().h())>135) {
			float prev_angle=lookat(this.getPos(),PREV_GOTO_POS).h();
			if (minangle(prev_angle,lookangle)>135) bBunnyInside=1;
			else e_goto(PREV_GOTO_POS);
		}
	}
}

//AI
public string getStepWidth() { return string(speed*speed_mult*fElapsed()*1.2)); }
public float3 getGOTO() {
	return GOTO_POS;
}
public float3 getPREVGOTO() {
	return PREV_GOTO_POS;
}
public string getDir() {
	int heading=int(normalizeangle(this.getRot().h()));
	if (heading>45 && heading<135) return "W";
	if (heading>=135 && heading<225) return "S";
	if (heading>=225 && heading<315) return "E";
	return "N";
}
public int isOnTarget() {
	return int((this.getPos().xz()-GOTO_POS.xz()).length()<(speed*speed_mult*speed_trap*fElapsed()*1.1));
}
void tick() {
	if ((state!=CONST_STATE_WALKDOWN) && (state!=CONST_STATE_WALKUP)) return;
	if (bBunnyInside) return;
	float3 oldp = this.getPos();
	if (length( (oldp-GOTO_POS).xz() ) <= speed*speed_mult*speed_trap*fElapsed()) {
		this.setPos(float3(GOTO_POS.x(),BASE_Y,GOTO_POS.z()));
	}
	else {
		float CUR_HEADING = normalizeAngle(oldp.lookat(GOTO_POS).h);
		float3 dir = GOTO_POS - oldp;
		dir = float3(dir.x,0,dir.z);
		dir.normalize();

		float3 fin = oldp + dir*(speed*speed_mult*speed_trap*fElapsed());
		fin.y( BASE_Y );
		this.setPos(fin);
		
		float3 r = this.getRot();
		if (minangle(CUR_HEADING,r.h) > 5)
		{
			float tar_h = turnangle(r.h,CUR_HEADING,abs(minangle(r.h,CUR_HEADING))*0.25*fElapsed());
			this.setRot(hpr(normalizeAngle(tar_h),0.0,0.0));
		}
		else
		{
			this.setRot(hpr(CUR_HEADING,0.0,0.0));
		}			
	}
}
public void setBaseY(float _y) { BASE_Y=max(1.9,_y); }
public void setSpeedTrap(float _s) { speed_trap=_s; }
public void setTimeScale(float _s) { actor.setTimeScale(_s); }

public string getForceNext() { return force_next; }
public string resetForceNext() { 
	force_next_count=max(0,force_next_count-1);
	if (force_next_count<=0) force_next=""; 
}
public void setForceNext(string _dir) { force_next=_dir;force_next_count++; }

