 /****************************************************************************
 * This file is part of Bunny Maze Application                                *
 *                                                                           *
 * Copyright (c) 2011, Eyelead Software. All rights reserved.                *
 * PROPRIETARY AND CONFIDENTIAL                                              *
 *                                                                           *
 * This file is subject to the terms and conditions of the                   *
 * Commercial Software License Agreement by and between                      *
 * Qualcomm Incorporated and Eyelead Software S.A                            *
 * as made and entered on September 30th, 2011                               *
 *****************************************************************************/
#using stdlib game
#hive 1

String APPNAME = "bunnymaze";

//if script is used for preloading
input int PRELOADER=0;

input float speed = 0.07;
input float speed_mult=1.0;

vfile actor,fx_hitstars,snowball;
Resource res;
int state,blendstate;
float3 GOTO_POS,PREV_GOTO_POS; 
float BASE_Y;

int CONST_STATE_IDLE=0; 
int CONST_STATE_WALK=1;

int CONST_BLENDSTATE_NONE=0;
int CONST_BLENDSTATE_ATTACK=1;
int CONST_BLENDSTATE_HIT=2;
int CONST_BLENDSTATE_ROTATE=3;

int bLoaded=0;

string force_next;
int force_next_count;

float CONST_ATTACK_TIMEOUT=60.0;
float attack_timeout;

void defaults() {
	force_next="";
	force_next_count=0;

	state=0;
	blendstate=0;
	BASE_Y=2.33;
	attack_timeout=CONST_ATTACK_TIMEOUT;

	this.setChannel("col.height",3.0);
	this.setChannel("col.radius",0.5);
	this.setChannel("prox.radius",0.7);
	fx_hitstars.hide();
	
	//assuming that the vfile is an actor object, set its initial animation
	actor.setMaster("idle",1);
	//make vfile visible
	actor.show();
	actor.setScale(float3(1.6,1.6,1.6));
	actor.setTimeScale(1.0);
	
	snowball.hide();

	GOTO_POS=this.getPos();
	PREV_GOTO_POS=GOTO_POS;
	
	clearListeners(ON_TICK);
	clearListeners(ON_EVENT);	
	clearListeners(ON_TIMEOUT_FRAMES);
	clearListeners(ON_ENTITYENTER);
	clearListeners(ON_ENTITYLEAVE);
}

void init()
{
	if (PRELOADER) {
		debugLog("Preloading penguin"); 
		setTag("dummy");
		res=Resource("levelsets/traps/penguin.hvfs","");	
		return;
	}

	bLoaded=0;

	//set entity tag
	debugLog("Loading enemy: "+getTag());
	
	clearScene(this.path()+"/fx");
	system("mkdir "+this.path()+"/fx");
	loadScene("levelsets/generic/hitstars.hvfs",this.path()+"/fx","cbHitstarsLoaded");	
	
	//check for existing
	snowball=0;
	if (vfexists(this.path()+"/../snowball."+getTag()))	snowball=vfopen(this.path()+"/../snowball."+getTag(),"r");
	if (int(snowball)==0) {	
		snowball=vfopen(spawnTrigger(this.path()+"/../snowball."+getTag(),"scripts/snowball.hs","snowball."+getTag(),PROXSHAPE_SPHERE3D,0.0),"r");
	}
	snowball.hide();
}
void cbHitstarsLoaded() {
	fx_hitstars=vfopen(this.path()+"/fx/hitstars.actor","r");
	res=Resource("levelsets/traps/penguin.hvfs","cbResLoaded");	
}
void cbResLoaded() {
	actor = res.use("penguin.actor","reset");
}
public void regEvents() {
	addListener(ON_ENTITYENTER,"e_enter");
	addListener(ON_ENTITYLEAVE,"e_leave");
	addListener(ON_EVENT,"ROTATE","e_rotate");
	addListener(ON_EVENT,"GOTO","e_goto");
	addListener(ON_EVENT,"STOP","e_stop");
	addListener(ON_EVENT,"HIT","e_hit");
	addListener(ON_TICK,"tick");
}
public void reset() {
	if (bLoaded) {
		debugLog("Reset "+getTag());
	}
	else {
		bLoaded=1;
		debugLog(getTag()+" loaded");
	}
	defaults();
}
public string isLoaded() {
	return string(bLoaded);
}
public void savestate() {
	appPropertySet(APPNAME,"ENEMY_POS:"+getTag(),string(this.getPos()));
	appPropertySet(APPNAME,"ENEMY_ROT:"+getTag(),string(this.getRot()));
	appPropertySet(APPNAME,"ENEMY_GOTO:"+getTag(),string(GOTO_POS));
	appPropertySet(APPNAME,"ENEMY_STATE:"+getTag(),string(state));
	appPropertySet(APPNAME,"ENEMY_FORCENEXT:"+getTag(),force_next);
	appPropertySet(APPNAME,"ENEMY_FORCENEXTCOUNT:"+getTag(),string(force_next_count));
}
public void loadstate() {
	this.setPos(float3(appPropertyGet(APPNAME,"ENEMY_POS:"+getTag(),string(this.getPos()))));
	this.setRot(float3(appPropertyGet(APPNAME,"ENEMY_ROT:"+getTag(),string(this.getRot()))));
	GOTO_POS=float3(appPropertyGet(APPNAME,"ENEMY_GOTO:"+getTag(),string(GOTO_POS)));
	state=int(appPropertyGet(APPNAME,"ENEMY_STATE:"+getTag(),string(state)));
	force_next=appPropertyGet(APPNAME,"ENEMY_FORCENEXT:"+getTag(),force_next);
	force_next_count=int(appPropertyGet(APPNAME,"ENEMY_FORCENEXTCOUNT:"+getTag(),string(force_next_count)));
	fCall("cbFinishAttack");
}
void cbFinishAttack() {
	blendstate=CONST_BLENDSTATE_NONE;
	if (state==CONST_STATE_IDLE) { actor.setMaster("idle",1); }
	else if (state==CONST_STATE_WALK) { actor.setMaster("walk",1); }
}
void e_rotate() {
	if (blendstate!=CONST_BLENDSTATE_HIT) {
		blendstate=CONST_BLENDSTATE_ROTATE;
		attack_timeout=CONST_ATTACK_TIMEOUT+rand(int(CONST_ATTACK_TIMEOUT));		
		delListener(ON_TIMEOUT_FRAMES,"cbFinishHit");
		delListener(ON_TIMEOUT_FRAMES,"cbFinishAttack");
		delListener(ON_TIMEOUT_FRAMES,"cbLaunchAttack");
		addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("rotate",0)),"cbFinishHit");	
		["hero"->playPenguin(this.getPos().xz(),"turn")];
	}
}
void e_hit() {
	if (blendstate!=CONST_BLENDSTATE_HIT) {
		//inform gameplay script that clunny is down
		blendstate=CONST_BLENDSTATE_HIT;
		attack_timeout=CONST_ATTACK_TIMEOUT+rand(int(CONST_ATTACK_TIMEOUT));
		delListener(ON_TIMEOUT_FRAMES,"cbFinishHit");
		delListener(ON_TIMEOUT_FRAMES,"cbFinishAttack");
		delListener(ON_TIMEOUT_FRAMES,"cbLaunchAttack");
		addListener(ON_TIMEOUT_FRAMES,string(actor.setMaster("fall",0)),"cbFinishHit");	
		fx_hitstars.show();
		fx_hitstars.setMaster("hitclunny",0);
	}
}
void cbFinishHit() {
	cbFinishAttack();
}
void e_enter(string _tag) {	
	if (_tag.startsWithNoCase("enemy")) {
		if (blendstate==CONST_BLENDSTATE_HIT) return;		
		Event e=Event("ATTACK",_tag);
		e.addArg(this.getPos());
		e.sendEvent();	
		e_hit();
		["hero"->playAttackSound(this.getPos().xz())];
	}
}
void e_leave(string _tag) { 
}
void e_stop() {
	GOTO_POS=this.getPos();
	if (state!=CONST_STATE_IDLE) {
		if (blendstate==CONST_BLENDSTATE_NONE) {
			actor.setMaster("idle",1);
		}
		state=CONST_STATE_IDLE;
	}
}
void e_goto(float3 _newpos) {
	PREV_GOTO_POS=GOTO_POS;
	GOTO_POS=_newpos;
	if (state!=CONST_STATE_WALK) {
		if (blendstate==CONST_BLENDSTATE_NONE) actor.setMaster("walk",1);
		state=CONST_STATE_WALK;
	}
}

//AI
public string getStepWidth() { return string(speed*speed_mult*fElapsed()*1.2)); }
public float3 getGOTO() {
	return GOTO_POS;
}
public float3 getPREVGOTO() {
	return PREV_GOTO_POS;
}
public string getDir() {
	int heading=int(normalizeangle(this.getRot().h()));
	if (heading>45 && heading<135) return "W";
	if (heading>=135 && heading<225) return "S";
	if (heading>=225 && heading<315) return "E";
	return "N";
}
public int isOnTarget() {
	return int((this.getPos().xz()-GOTO_POS.xz()).length()<(speed*speed_mult*fElapsed()*1.1));
}
public int isHit() {
	return int(blendstate==CONST_BLENDSTATE_HIT);
}
public int isHitable() {
	return !isHit();
}
void tick() {
	if (blendstate!=CONST_BLENDSTATE_NONE) return;

	float3 oldp = this.getPos();
	if (length( (oldp-GOTO_POS).xz() ) <= speed*speed_mult*fElapsed()) {
		this.setPos(float3(GOTO_POS.x(),BASE_Y,GOTO_POS.z()));
	}
	else {
		float CUR_HEADING = normalizeAngle(oldp.lookat(GOTO_POS).h);
		float3 dir = GOTO_POS - oldp;
		dir = float3(dir.x,0,dir.z);
		dir.normalize();

		float3 fin = oldp + dir*(speed*speed_mult*fElapsed());
		fin.y( BASE_Y );
		this.setPos(fin);
		
		float3 r = this.getRot();
		if (minangle(CUR_HEADING,r.h) > 5)
		{
			float tar_h = turnangle(r.h,CUR_HEADING,abs(minangle(r.h,CUR_HEADING))*0.25*fElapsed());
			this.setRot(hpr(normalizeAngle(tar_h),0.0,0.0));
		}
		else
		{
			this.setRot(hpr(CUR_HEADING,0.0,0.0));
			attack_timeout-=fElapsed();
			if (attack_timeout<=0) {
				attack_timeout=CONST_ATTACK_TIMEOUT+rand(int(CONST_ATTACK_TIMEOUT));
				blendstate=CONST_BLENDSTATE_ATTACK;
				float frames=actor.setMaster("snowball",0);
				addListener(ON_TIMEOUT_FRAMES,string(frames),"cbFinishAttack");
				addListener(ON_TIMEOUT_FRAMES,string(frames*0.75),"cbLaunchAttack");
			}
		}			
	}
}
void cbLaunchAttack() {
	Event e=Event("GOTO",snowball.getTag());
	e.addArg(this.getPos());
	e.addArg(["ui.play"->findCorridorEnd(this.getPos(),getDir()) as float3]);
	e.sendEvent();
	["hero"->playPenguin(this.getPos().xz(),"ball")];
}

public void setBaseY(float _y) { BASE_Y=_y; }
public void setTimeScale(float _s) { actor.setTimeScale(_s); }
public void setSpeedTrap(float _s) { }

public string getForceNext() { return force_next; }
public string resetForceNext() { 
	force_next_count=max(0,force_next_count-1); 
	if (force_next_count<=0) force_next=""; 
}
public void setForceNext(string _dir) { force_next=_dir;force_next_count++; }



