 /****************************************************************************
 * This file is part of Bunny Maze Application                                *
 *                                                                           *
 * Copyright (c) 2011, Eyelead Software. All rights reserved.                *
 * PROPRIETARY AND CONFIDENTIAL                                              *
 *                                                                           *
 * This file is subject to the terms and conditions of the                   *
 * Commercial Software License Agreement by and between                      *
 * Qualcomm Incorporated and Eyelead Software S.A                            *
 * as made and entered on September 30th, 2011                               *
 *****************************************************************************/

#using stdlib gui sound analytics gamepad
#hive 1
String APPNAME = "bunnymaze";

Sound musiclobby,musicgame;
int level_to_load,level_restart=0;
string theme_to_load;
Timer statsTimer;
string debrief_resume_str;
Gamepad gpad;

int controller_home=0;
int controller_tutorial=0;
int controller_play=0;

void init() {
  setTag("sc.control");
  
  //load music
  musicgame=Sound("sounds/ingame1.audio","e_volume");
  musiclobby=Sound("sounds/lobby.audio","e_volume");
  
  //set fps
  setMultiPassTicking();
  setTickRange(0,360);
  setFPS(30);  
  setBaseFPS(30);  
  
  //ui.init->control
  addListener(ON_EVENT,"HOME","e_home");
  addListener(ON_EVENT,"RESUME","e_resume");
  
  //ui.home->control
  addListener(ON_EVENT,"PLAY","e_play");

  //ui.loading->control
  addListener(ON_EVENT,"UI_LOADING_VISIBLE","e_ui_loading_visible");
  
  //sc.loader->control
  addListener(ON_EVENT,"SC_CONTROL_LEVEL_LOADED","e_sc_control_level_loaded");
  addListener(ON_EVENT,"SC_CONTROL_LEVEL_LOADED_RESUME","e_sc_control_level_loaded_resume");
  addListener(ON_EVENT,"SC_CONTROL_LEVEL_LOADED_DEBRIEF_RESUME","e_sc_control_level_loaded_debrief_resume");
  
  
  //ui.tutorial->control
  addListener(ON_EVENT,"UI_TUTORIAL_COMPLETED","e_ui_tutorial_completed");
  
  //ui.brief->control
  addListener(ON_EVENT,"UI_BRIEF_COMPLETED","e_ui_brief_completed");
  
  //ui.debrief->control
  addListener(ON_EVENT,"UI_DEBRIEF_MENU","e_ui_debrief_menu");
  addListener(ON_EVENT,"UI_DEBRIEF_TRYAGAIN","e_ui_debrief_tryagain");
  addListener(ON_EVENT,"UI_DEBRIEF_NEXT","e_ui_debrief_next");
  
  //ui.options->control
  addListener(ON_EVENT,"RESETLEVELS","e_resetlevels");
  addListener(ON_EVENT,"RESTARTLEVEL","e_restartlevel");
  addListener(ON_EVENT,"QUITLEVEL","e_quitlevel");
  addListener(ON_EVENT,"VOLUME","e_volume");
  
  //ui.play->control
  addListener(ON_EVENT,"DEBRIEF","e_debrief");
  
  //ui.notepad->control
  addListener(ON_EVENT,"UI_NOTEPAD_DEBRIEF","e_ui_notepad_debrief");
  addListener(ON_EVENT,"UI_NOTEPAD_BRIEF","e_ui_notepad_brief");
  addListener(ON_EVENT,"UI_NOTEPAD_PLAY","e_ui_notepad_play");
  addListener(ON_EVENT,"UI_NOTEPAD_DEBRIEF_NEXT","e_ui_notepad_debrief_next");
  addListener(ON_EVENT,"UI_NOTEPAD_DEBRIEF_MENU","e_ui_notepad_debrief_menu");
  
  //ui.story->control
  addListener(ON_EVENT,"UI_START_LOADING","e_ui_start_loading");

  controller_home=0;
  controller_tutorial=0;
  controller_play=0;
  gpad=Gamepad();
  gpad.onButtonDown("cbPadButtonDown");
  
  //use it to capture total play time
  statsTimer=Timer();
  addListener(ON_EVENT,"APPPAUSE","cbAppPause");
  
  //debugger
  if (!["ui.init"->exist() as int]) addListener(ON_SCENELOADED,"e_home");
}

///////// CONTROLLER FLAG ///////////

void cbPadButtonDown(int _controller,int _action) {
	if (controller_home*controller_tutorial*controller_play) return;
	if (gpad.isDPAD(_action) || _action==GAMEPAD_A) {
		if (!controller_home) controller_home=["ui.home"->showController() as int];
		if (!controller_tutorial) controller_home=["ui.tutorial"->showController() as int];
		if (!controller_play) controller_home=["ui.play"->showController() as int];
	}
}

///////// STATISTICS /////////

void cbAppPause() {
	analytics_trackevent("time","session","",statsTimer.getMSec()/1000);
}

///////// MUSIC ////////

void stopMusic() {
	musiclobby.stop();
	musicgame.stop();
	delListener(ON_TIMEOUT_MSEC,"cbReplayGameMusic");
}
void cbReplayGameMusic() {
	musicgame.play();
	//55-65 delay
	addListener(ON_TIMEOUT_MSEC,string((55+rand(10)*1000)),"cbReplayGameMusic");
}
void playLobbyMusic() {
	stopMusic();
	musiclobby.loop("sounds/lobby.audio",1);
	musiclobby.play();
}
void playGameMusic() {
	stopMusic();
	cbReplayGameMusic();
}
void e_volume() {
	musicgame.setVolume(float(appPropertyGet(APPNAME,"musicvol","1.0")));
	musiclobby.setVolume(float(appPropertyGet(APPNAME,"musicvol","1.0")));
}

///////// RESUME FROM PAUSE ///////////

void e_resume() {
	system("chrnd -v /scripts/ui.notepad");
	string laststate=appPropertyGet(APPNAME,"state","");
	if (laststate.startsWithNoCase("LEVELSELECT:")) {
		laststate.trimBegin(12);
		["ui.home"->e_setCurrentTheme(laststate)];
		fCall("e_quitlevel");
		analytics_trackview("/resume/lobby");
	}
	else if (laststate.startsWithNoCase("PLAY:")) {
		laststate.trimbegin(5);
		lists tokens=lists();
		laststate.split(":",tokens);
		theme_to_load=tokens.get(0);
		level_to_load=int(tokens.get(1));
		if (!["ui.home"->validate(theme_to_load,level_to_load) as int]) {
			fCall("e_quitlevel");
			sendEvent("HIDE","ui.init");
			return;
		}
		analytics_trackview("/resume/play");
		//reset to level menu
		["ui.home"->e_setCurrentTheme(theme_to_load)];
		sendEvent("LEVELS","ui.home");		
		//hide home ui
		system("chrnd -v /scripts/ui.home");
		//our loading hud is on, start loading the actual level
		Event e=Event("LOADLEVEL","sc.loader");
		e.addArg(theme_to_load);
		e.addArg(level_to_load);
		e.addArg(getTag());
		e.addArg("SC_CONTROL_LEVEL_LOADED_RESUME");
		sendEvent(e);
		return;
	}
	else if (laststate.startsWithNoCase("LOAD:")) {
		laststate.trimbegin(5);
		lists tokens=lists();
		laststate.split(":",tokens);
		theme_to_load=tokens.get(0);
		level_to_load=int(tokens.get(1));
		if (!["ui.home"->validate(theme_to_load,level_to_load) as int]) {
			fCall("e_quitlevel");
			sendEvent("HIDE","ui.init");
			return;
		}
		analytics_trackview("/resume/play");
		//reset to level menu
		["ui.home"->e_setCurrentTheme(theme_to_load)];
		fCall("e_play("+theme_to_load+","+level_to_load+")");
	}
	else if (laststate.startsWithNoCase("DEBRIEF:")) {
		debrief_resume_str=laststate;
		laststate.trimbegin(8);
		lists tokens=lists();
		laststate.split(":",tokens);
		if (tokens.num()==9) {
			theme_to_load=tokens.get(0);
			level_to_load=int(tokens.get(1));
			if (["ui.home"->validate(theme_to_load,level_to_load) as int]) {
				analytics_trackview("/resume/play");
				//reset to level menu
				["ui.home"->e_setCurrentTheme(theme_to_load)];
				sendEvent("LEVELS","ui.home");		
				//hide home ui
				system("chrnd -v /scripts/ui.home");

				//resave state
				appPropertySet(APPNAME,"state",debrief_resume_str);
				appPropertyFlush();	
				
				//our loading hud is on, start loading the actual level
				Event e=Event("LOADLEVEL","sc.loader");
				e.addArg(theme_to_load);
				e.addArg(level_to_load);
				e.addArg(getTag());
				e.addArg("SC_CONTROL_LEVEL_LOADED_DEBRIEF_RESUME");
				sendEvent(e);
				return;
			}
		}
		fCall("e_quitlevel");
		sendEvent("HIDE","ui.init");
		return;
	}
	else {
		analytics_trackview("/resume/lobby");
		fCall("e_home");
	}
	//fadeout splash
	sendEvent("HIDE","ui.init");
}

///////// NORMAL GAME START ///////////

void e_home() {
	playLobbyMusic();
	system("chrnd -v /scripts/ui.brief");
	system("chrnd -v /scripts/ui.debrief");
	system("chrnd +v /scripts/ui.home");
	system("chrnd -v /scripts/ui.loading");
	system("chrnd -v /scripts/ui.options");
	system("chrnd -v /scripts/ui.play");
	system("chrnd -v /scripts/ui.tutorials");
	system("chrnd -v /scripts/ui.story");
	//fadeout splash
	sendEvent("HIDE","ui.init");
	sendEvent("HOME","ui.home");
}

///////// SELECT LEVEL AND PLAY ///////////

void e_play(string _theme,int _level) {
	stopMusic();
	level_to_load=_level;
	theme_to_load=_theme;
	analytics_trackview("/play/"+theme_to_load+"/"+level_to_load+"/start");
	Event e=Event("SHOW","ui.loading");
	e.addArg(getTag());
	e.addArg("UI_LOADING_VISIBLE");//reply event
	sendEvent(e);
	analytics_dispatch();		
	appPropertySet(APPNAME,"state","LOAD:"+theme_to_load+":"+level_to_load);
	appPropertyFlush();
}

void e_ui_start_loading() {
	//our loading hud is on, start loading the actual level
	Event e=Event("LOADLEVEL","sc.loader");
	e.addArg(theme_to_load);
	e.addArg(level_to_load);
	e.addArg(getTag());
	e.addArg("SC_CONTROL_LEVEL_LOADED");
	sendEvent(e);
}

//loading ui is on, proceed to load level
void e_ui_loading_visible() {
	//hide home ui
	system("chrnd -v /scripts/ui.home");
	if ((level_restart==0) && (theme_to_load=="forest") && (level_to_load==1)) {
		Event e=Event("SHOW","ui.story");
		e.addArg(getTag());
		e.addArg("UI_START_LOADING");//reply event
		sendEvent(e);
	}
	else e_ui_start_loading();
}
//level loaded, hide loading ui
void e_sc_control_level_loaded() {
	//play game music
	playGameMusic();

	//prepare gameplay
	sendEvent("LOAD","ui.play");

	//hide loading ui
	sendEvent("HIDE","ui.loading");
	
	//show notepad
	if (level_restart) {
		["ui.notepad"->forcebrief()];
		fCall("e_ui_notepad_brief");
	}
	else ["ui.notepad"->brief(getTag(),"UI_NOTEPAD_BRIEF")];
}
void e_ui_notepad_brief() {
	if (level_restart) {
		fCall("e_ui_tutorial_completed");
	}
	else {
		//show tutorial
		Event e=Event("SHOW","ui.tutorial");
		e.addArg(theme_to_load);
		e.addArg(level_to_load);
		e.addArg(getTag());
		e.addArg("UI_TUTORIAL_COMPLETED");
		sendEvent(e);
	}
}
void e_sc_control_level_loaded_resume() {
	playGameMusic();
	sendEvent("HIDE","ui.init");
	sendEvent("RESUME","sc.loader");
	sendEvent("LOAD","ui.play");
	if (["sc.loader"->getCurrentScore() as int]>0) {
		//normal gameplay
		sendEvent("ENABLE_ENTITIES","sc.loader");
		sendEvent("SHOW","ui.play");		
	}
	else {
		//tutorial
		//show tutorial
		["ui.notepad"->forcebrief()];
		Event e=Event("SHOW","ui.tutorial");
		e.addArg(theme_to_load);
		e.addArg(level_to_load);
		e.addArg(getTag());
		e.addArg("UI_TUTORIAL_COMPLETED");
		sendEvent(e);
	}
}
void e_sc_control_level_loaded_debrief_resume() {
	debrief_resume_str.trimbegin(8);
	lists tokens=lists();
	debrief_resume_str.split(":",tokens);
	if (tokens.num()==9) {
		playGameMusic();
		sendEvent("HIDE","ui.init");
		sendEvent("RESUME","sc.loader");
		["ui.notepad"->forcebrief()];
		Event e=Event("SHOW","ui.debrief");
		e.addArg(int(tokens.get(2)));
		e.addArg(int(tokens.get(3)));
		e.addArg(int(tokens.get(4)));
		e.addArg(int(tokens.get(5)));
		e.addArg(int(tokens.get(6)));
		e.addArg(int(tokens.get(7)));
		e.addArg(int(tokens.get(8)));				
		sendEvent(e);			
	}	
	else e_sc_control_level_loaded_resume();
}
//tutorial completed
void e_ui_tutorial_completed() {
	//hide tutorial
	sendEvent("HIDE","ui.tutorial");
	//show brief
	Event e=Event("SHOW","ui.brief");
	e.addArg(level_to_load);
	e.addArg(["sc.loader"->getRequiredCarrots() as int]);//carrots
	e.addArg(["sc.loader"->getRequiredTime() as int]);//time
	e.addArg(getTag());
	e.addArg("UI_BRIEF_COMPLETED");
	e.addArg(level_restart);
	sendEvent(e);
}
//brief completed
void e_ui_brief_completed() {
	level_restart=0;
	//hidepad
	["ui.notepad"->play(getTag(),"UI_NOTEPAD_PLAY")];
	//hide brief
	sendEvent("HIDE","ui.brief");
	//play begin animation of bunny
	["hero"->beginLevel()];
}
void e_ui_notepad_play() {
	//show gameplay
	sendEvent("ENABLE_ENTITIES","sc.loader");
	sendEvent("SHOW","ui.play");
}

/////// OPTIONS PANEL ////////

void e_resetlevels() {
	sendEvent("RESETLEVELS","ui.home");
}
void e_restartlevel() {
	enableTick(0);

	system("chrnd -v /scripts/ui.brief");
	system("chrnd -v /scripts/ui.debrief");
	system("chrnd -v /scripts/ui.home");
	system("chrnd -v /scripts/ui.loading");
	system("chrnd -v /scripts/ui.options");
	system("chrnd -v /scripts/ui.play");
	system("chrnd -v /scripts/ui.tutorials");
	system("chrnd -v /scripts/ui.story");
	
	sendEvent("HIDE","ui.home");
	sendEvent("HIDE","ui.play");

	level_restart=1;
	e_play(theme_to_load,level_to_load);
}
void e_quitlevel() {
	enableTick(0);

	system("chrnd -v /scripts/ui.brief");
	system("chrnd -v /scripts/ui.debrief");
	system("chrnd +v /scripts/ui.home");
	system("chrnd -v /scripts/ui.loading");
	system("chrnd -v /scripts/ui.options");
	system("chrnd -v /scripts/ui.play");
	system("chrnd -v /scripts/ui.tutorials");
	system("chrnd -v /scripts/ui.story");
	
	//fadeout splash
	sendEvent("HIDE","ui.play");
	sendEvent("LEVELS","ui.home");
	
	playLobbyMusic();
}

///////// DEBRIEFING ////////

void e_debrief() {
	["ui.notepad"->debrief(getTag(),"UI_NOTEPAD_DEBRIEF")];
}
void e_ui_notepad_debrief() {
	int totalstars=["ui.home"->getTotalStars() as int];
	["ui.play"->debriefSave()];
	int newtotalstars=["ui.home"->getTotalStars() as int];
	//show debrief
	string state_string="DEBRIEF:"+theme_to_load+":"+level_to_load+":";
	int val;
	
	Event e=Event("SHOW","ui.debrief");
	val=["sc.loader"->getTotalCarrots() as int];
	e.addArg(val); state_string+=string(val)+":";
	
	val=["sc.loader"->getCurrentScore() as int];
	e.addArg(val); state_string+=string(val)+":";
	
	val=["sc.loader"->getBonusPUP() as int];
	e.addArg(val); state_string+=string(val)+":";
	
	val=["sc.loader"->getBonusJump() as int];
	e.addArg(val); state_string+=string(val)+":";
	
	val=["sc.loader"->getCurrentTime() as int];
	e.addArg(val); state_string+=string(val)+":";
	
	e.addArg(totalstars); state_string+=string(totalstars)+":";	
	e.addArg(newtotalstars); state_string+=string(newtotalstars);
		
	appPropertySet(APPNAME,"state",state_string);
	appPropertyFlush();	
		
	sendEvent(e);	
}

void e_ui_debrief_menu() {
	analytics_dispatch();	
	sendEvent("HIDE","ui.debrief");
	["ui.notepad"->hide(getTag(),"UI_NOTEPAD_DEBRIEF_MENU")];
}
void e_ui_notepad_debrief_menu() {
	e_quitlevel();
}

void e_ui_debrief_tryagain() {
	analytics_dispatch();	
	sendEvent("HIDE","ui.debrief");
	e_restartlevel();
}

void e_ui_debrief_next() {
	analytics_dispatch();	
	sendEvent("HIDE","ui.debrief");
	["ui.notepad"->hide(getTag(),"UI_NOTEPAD_DEBRIEF_NEXT")];
}
void e_ui_notepad_debrief_next() {
	Event e=Event("NEXTLEVEL","ui.home");
	e.addArg(level_to_load+1);
	sendEvent(e);
}



