 /****************************************************************************
 * This file is part of Bunny Maze Application                                *
 *                                                                           *
 * Copyright (c) 2011, Eyelead Software. All rights reserved.                *
 * PROPRIETARY AND CONFIDENTIAL                                              *
 *                                                                           *
 * This file is subject to the terms and conditions of the                   *
 * Commercial Software License Agreement by and between                      *
 * Qualcomm Incorporated and Eyelead Software S.A                            *
 * as made and entered on September 30th, 2011                               *
 *****************************************************************************/

#using stdlib gui game sound analytics gamepad
#hive 1

/////// TUTORIAL FLAGS /////////

int TUT_CAN_JUMP,TUT_CAN_GOBACK,TUT_CAN_DRAGON,show_controller;

/////// CONSTANTS ///////
String APPNAME = "bunnymaze";
String hero="hero";
float CONST_CARROT_UI_TIMEOUT=10.0;
float CONST_LNOTIFICATION_TIMEOUT=50.0;
float CONST_PUOFF_OPACITY = 0.25;
int CONST_GRID_NULL = 0;
int CONST_GRID_NODE = 1;
int CONST_GRID_CARROT = 2;
int CONST_GRID_PUP = 3;
int CONST_GRID_DEADEND = 4;
int CONST_GRID_TRAP = 5;

int CONST_NORTH=0;
int CONST_WEST=1;
int CONST_SOUTH=2;
int CONST_EAST=3;

float CONST_PERC_STAR1 = 0.5;
float CONST_PERC_STAR2 = 1.0;
float CONST_PERC_STAR3 = 1.5;

float CONST_TIMEOUT_PU1=240.0;
float CONST_TIMEOUT_PU2=240.0;
float CONST_TIMEOUT_PU3=240.0;

int CONST_AI_OFFENSIVE=0;
int CONST_AI_DEFENSIVE=1;
int CONST_AI_DONTCARE=2;

/////// UI ///////

Layout HUD;
Widget TIMEBG,TIMEBAR,TIMECOUNT,CARROTCOUNT,TIMEFG,STAR1,STAR2,STAR3,STARICON,STARFX,STARFG,STARFLASH1,STARFLASH2;
Widget COMPASSBG,COMPASSICON,LABELFPS;
Widget BONUSICON_CLOCK;
Widget PUBUTTON1,PUBUTTON2,PUBUTTON3,PUSLOT1,PUSLOT2,PUSLOT3,PULABEL1,PULABEL2,PULABEL3,LNOTIFICATION,PURING1,PURING2,PURING3,LNOTIFICATION_STREAKS;
Widget PUCONTROLLER1,PUCONTROLLER2,PUCONTROLLER3;
Widget EGGUI_BG,EGGUI_FIRE,EGGUI_WINGS,EGGUI_TAIL;

Curve LNOTIFICATION_CURVE,LNOTIFICATION_STREAKS_CURVE;
float LNOTIFICATION_TIMEOUT;
float2 LNOTIFICATION_TARGET_POS;
int LNOTIFICATION_WAIT;

Curve CARROT_UI_CURVE;
float CARROT_UI_TIMER;

/////// GLOBALS ////////

int PAUSE_AFTER_RESUME=0;
int BPAUSED=1,BSHOWFPS=0;
vfile hero_file,carrots,statue,dragon_coins;
listvf l_enemies,l_hedgehog,l_gopher,l_penguin;
Grid posgrid,itemgrid,pivotindexgrid,trapgrid;
float2 gridmin,gridmax;
float gridinterval;
listvf l_deadends,l_pups,l_dragonslots,l_clocks;
list l_wclunny,l_wclunnycompass;
Timer clickTimer,playTimer,jumpClickTimer, comboTimer;
float MAX_TIMER,CUR_TIMER;
int SCORE_COUNT,PREV_SCORE_COUNT;
int TOTAL_CARROTS,CARROTS_LEFT;
int BONUS_JUMP,BONUS_PUP,BONUS_TIME;
int PU1_STACK,PU2_STACK,PU3_STACK;
int SCORE_SAFE_CAP,COUNTER_LOSE;
float TIMEOUT_PU1,TIMEOUT_PU2,TIMEOUT_PU3;
int EGG_JUMPS_INROW,EGG_CARROTS_INROW,EGG_SQUASH_INROW;
int CARROT_AUDIO_COUNTER=1;
int CLUNNY_AUDIO_CANPLAY=1;
int CARROT_AUDIO_COUNTER_MAX=8;
Sound snd;
Gamepad gpad;

int IS_DOUBLESWIPE;

//gopher locking
string locked_dir;
string roadblock_dir;
int force_stop=0,force_bunny_back=0;

/////// IMPLEMENTATION ////////
float normSizef(float _size) {
	return (_size*float(HUD.getW()))/800.0;
}
int scaleUI(int _size) {
	return (_size*0.9);
}
float scaleUIf(float _size) {
	return (_size*0.9);
}
void setupFont(Widget _label,int _size,string _align) {
	_label.setParam(PARAM_FONTIMAGE,"LINEAR:textures/ui/font.image");
	_label.setparam(PARAM_FONTSIZE,string(_size)+","+_size);
	_label.setparam(PARAM_FONTSPACING,"-0.4");
	_label.setparam(PARAM_TEXTALIGN,_align);
	_label.setSize(_size,_size);
	_label.enableGhost(1);
}
int toGridX(float _x) {
	return int(((_x+gridinterval*0.25)-gridmin.x())/gridinterval);
}
int toGridY(float _y) {
	return int(((_y+gridinterval*0.25)-gridmin.y())/gridinterval);
}
float2 getTile(float3 epos) {
	int TEMPTILEX=toGridX(epos.x());
	int TEMPTILEY=toGridY(epos.z());
	int NEWTILEX=TEMPTILEX;
	int NEWTILEY=TEMPTILEY;
	//for precision errors check if we are closer to another grid tile
	float mindist=10000.0,dist;
	int x,y;
	for (x=TEMPTILEX-1;x<=TEMPTILEX+1;x++) {
		for (y=TEMPTILEY-1;y<=TEMPTILEY+1;y++) {
			if (posgrid.exists(x,y)) {
				if (posgrid.geti(x,y)>0) {
					dist=(epos-itemgrid.getvf(x,y).getPos()).xz().length();
					if (dist<mindist) {
						mindist=dist;
						NEWTILEX=x;
						NEWTILEY=y;
					}
				}
			} 
		}
	}
	return float2(NEWTILEX,NEWTILEY);
}
void init() {
	PAUSE_AFTER_RESUME = 0;
	setTag("ui.play");

	show_controller=0;
	
	carrots=vfopen("/sector/carrots.mlink","w");
	dragon_coins=vfopen("/sector/dragon_coins.mlink","w");
	statue=vfopen("/sector/statue","r");
	
	gpad=Gamepad();
	comboTimer=Timer();
	clickTimer=Timer();
	jumpClickTimer=Timer();
	l_enemies=listvf();
	l_hedgehog=listvf();
	l_penguin=listvf();
	l_gopher=listvf();
	l_wclunny=list();
	l_wclunnycompass=list();
	
	IS_DOUBLESWIPE = 0;
	
	HUD=Layout();
	HUD.setBaseSize(800,480,"autofit");
	HUD.enableGhost(1);
	HUD.show();

	TIMEBG=Widget(HUD,UI_ICON);
	TIMEBG.setSize(scaleUI(512),scaleUI(128));
	TIMEBG.setparam(PARAM_IMAGE,"LINEAR:textures/ui/time_bg.image");
	TIMEBG.setPos(scaleUI(-18),scaleUI(-14));
	TIMEBG.enableGhost(1);
	
	TIMECOUNT=Widget(TIMEBG,UI_TEXFONTLABEL);
	setupFont(TIMECOUNT,24,"left");
	TIMECOUNT.setSize(scaleUI(100),scaleUI(28));
	TIMECOUNT.setparam(PARAM_COLOR,"255,241,145,255");
	TIMECOUNT.setPos(scaleUI(114),scaleUI(32));
	TIMECOUNT.setParam(PARAM_TEXT,"0");
	
	CARROTCOUNT=Widget(TIMEBG,UI_TEXFONTLABEL);
	setupFont(CARROTCOUNT,24,"right");
	CARROTCOUNT.setSize(scaleUI(100),scaleUI(28));
	CARROTCOUNT.setparam(PARAM_COLOR,"244,120,38,255");
	CARROTCOUNT.setPos(scaleUI(254),scaleUI(32));
	CARROTCOUNT.setParam(PARAM_TEXT,"0");
	
	TIMEFG=Widget(TIMEBG,UI_SCISSOR);
	TIMEFG.setSize(scaleUI(256),scaleUI(32));
	TIMEFG.setPos(scaleUI(125),scaleUI(72));
	TIMEFG.enableGhost(1);
	
	TIMEBAR=Widget(TIMEFG,UI_ICON);
	TIMEBAR.setSize(TIMEFG.getW(),TIMEFG.getH());
	TIMEBAR.setparam(PARAM_IMAGE,"LINEAR:textures/ui/time_fg.image");
	TIMEBAR.enableGhost(1);	
	
	EGGUI_BG=Widget(HUD,UI_ICON);
	EGGUI_BG.enableGhost(1);
	EGGUI_BG.setPivot(PIVOTX_RIGHT,PIVOTY_TOP);
	EGGUI_BG.setSize(scaleUI(100),scaleUI(100));
	EGGUI_BG.setPos(670,6);
	EGGUI_BG.setparam(PARAM_IMAGE,"LINEAR:textures/ui/ui_dragon_bg.image");
	
	EGGUI_FIRE=Widget(EGGUI_BG,UI_ICON);
	EGGUI_FIRE.setSize("100%","100%");
	EGGUI_FIRE.setparam(PARAM_IMAGE,"LINEAR:textures/ui/ui_dragon_fire.image");
	 
	EGGUI_WINGS=Widget(EGGUI_BG,UI_ICON);
	EGGUI_WINGS.setSize("100%","100%");
	EGGUI_WINGS.setparam(PARAM_IMAGE,"LINEAR:textures/ui/ui_dragon_wings.image");
	
	EGGUI_TAIL=Widget(EGGUI_BG,UI_ICON);	
	EGGUI_TAIL.setSize("100%","100%");
	EGGUI_TAIL.setparam(PARAM_IMAGE,"LINEAR:textures/ui/ui_dragon_tail.image");
	
	Widget board;
	board=Widget(HUD,UI_ICON);
	board.setPivot(PIVOTX_RIGHT,PIVOTY_TOP);
	board.setSize(scaleUI(128),scaleUI(32));
	
	board.setPos(790,scaleUI(82));
	board.setparam(PARAM_IMAGE,"LINEAR:textures/ui/board.image");
	board.enableGhost(1);

	STAR1=Widget(board,UI_ICON);
	STAR1.setSize(scaleUI(128),scaleUI(128));
	STAR1.setPos(scaleUI(-16),scaleUI(-100));
	STAR1.setparam(PARAM_IMAGE,"LINEAR:textures/ui/star_bg.image");
	STAR1.enableGhost(1);

	STARICON=Widget(STAR1,UI_ICON);
	STARICON.setSize(STAR1.getW(),STAR1.getH());
	STARICON.setparam(PARAM_IMAGE,"LINEAR:textures/ui/star_fg.image");
	STARICON.enableGhost(1);
	
	STAR2=Widget(board,UI_ICON);
	STAR2.setSize(scaleUI(128),scaleUI(128));
	STAR2.setPos(scaleUI(-2),scaleUI(-98));
	STAR2.setparam(PARAM_IMAGE,"LINEAR:textures/ui/star_bg.image");
	STAR2.enableGhost(1);

	STARICON=Widget(STAR2,UI_ICON);
	STARICON.setSize(STAR2.getW(),STAR2.getH());
	STARICON.setparam(PARAM_IMAGE,"LINEAR:textures/ui/star_fg.image");
	STARICON.enableGhost(1);

	Widget starcont;
	starcont=Widget(board,UI_WIDGET);
	starcont.enableGhost(1);
	starcont.setSize(scaleUI(128),scaleUI(128));
	starcont.setPos(scaleUI(12),scaleUI(-96));
	
	STARFX=Widget(starcont,UI_LOADRING);
	STARFX.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
	STARFX.setParam(PARAM_IMAGE,"LINEAR:textures/ui/homestreaks.image");
	STARFX.setParam(PARAM_ROTATION_SPEED,"-3");
	STARFX.setSize(0,0);
	STARFX.setPos(starcont.getW()/2,starcont.getH()/2);
	STARFX.enableGhost(1);
	
	STAR3=Widget(board,UI_ICON);
	STAR3.setSize(scaleUI(128),scaleUI(128));
	STAR3.setPos(scaleUI(12),scaleUI(-96));
	STAR3.setparam(PARAM_IMAGE,"LINEAR:textures/ui/star_bg.image");
	STAR3.enableGhost(1);
	
	STARFG=Widget(STAR3,UI_SCISSOR);
	STARFG.setSize(scaleUI(128),scaleUI(128));
	STARFG.enableGhost(1);
	
	STARICON=Widget(STARFG,UI_ICON);
	STARICON.setSize(STARFG.getW(),STARFG.getH());
	STARICON.setparam(PARAM_IMAGE,"LINEAR:textures/ui/star_fg.image");
	STARICON.enableGhost(1);

	STARFLASH1=Widget(STAR3,UI_ICON);
	STARFLASH1.setSize(STAR3.getW(),STAR3.getH());
	STARFLASH1.setparam(PARAM_IMAGE,"LINEAR:textures/ui/star_flash1.image");
	STARFLASH1.setOpacity(0.0);
	STARFLASH1.enableGhost(1);

	STARFLASH2=Widget(STAR3,UI_ICON);
	STARFLASH2.setSize(STAR3.getW(),STAR3.getH());
	STARFLASH2.setparam(PARAM_IMAGE,"LINEAR:textures/ui/star_flash2.image");
	STARFLASH2.setOpacity(0.0);
	STARFLASH2.enableGhost(1);
	
	//COMPASS
	COMPASSBG=Widget(HUD,UI_ICON);
	COMPASSBG.setPivot(PIVOTX_LEFT,PIVOTY_BOTTOM);
	COMPASSBG.setSize(130,130);
	COMPASSBG.setPos(0,480);
	COMPASSBG.setparam(PARAM_IMAGE,"linear:textures/ui/cuparrowbg.image");
	COMPASSBG.enableGhost(1);
	COMPASSBG.hide();

	COMPASSICON=Widget(COMPASSBG,UI_ICON);
	COMPASSICON.setparam(PARAM_IMAGE,"linear:textures/ui/cuparrow.image");
	COMPASSICON.setSize(COMPASSBG.getW(),COMPASSBG.getH());
	COMPASSICON.enableGhost(1);
	 
	BONUSICON_CLOCK=Widget(COMPASSBG,UI_ICON);
	BONUSICON_CLOCK.setSize(60,60);
	BONUSICON_CLOCK.setPos((COMPASSBG.getW()-BONUSICON_CLOCK.getW())/2,(COMPASSBG.getH()-BONUSICON_CLOCK.getH())/2);
	BONUSICON_CLOCK.enableGhost(1);	
	BONUSICON_CLOCK.setparam(PARAM_IMAGE,"linear:textures/ui/pu.clock.image");
	
	//power ups
	PUBUTTON1=Widget(HUD,UI_PUSHBUTTON);
	PUBUTTON1.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
	PUBUTTON1.setSize(120,120);
	PUBUTTON1.setparam(PARAM_THEME,"LINEAR:textures/ui/powerup_normal.image,textures/ui/powerup_normal.image,textures/ui/powerup_pressed.image");
	PUBUTTON1.setPos(60,295);
	PUBUTTON1.setOpacity(0.0);
	PUBUTTON1.setParam(PARAM_ONCLICK,"cbPUButton(1)");
	PUBUTTON1.enableGhost(1);
	
	PUBUTTON2=Widget(HUD,UI_PUSHBUTTON);
	PUBUTTON2.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
	PUBUTTON2.setSize(120,120);
	PUBUTTON2.setparam(PARAM_THEME,"LINEAR:textures/ui/powerup_normal.image,textures/ui/powerup_normal.image,textures/ui/powerup_pressed.image");
	PUBUTTON2.setPos(165,315);
	PUBUTTON2.setOpacity(0.0);
	PUBUTTON2.setParam(PARAM_ONCLICK,"cbPUButton(2)");
	PUBUTTON2.enableGhost(1);

	PUBUTTON3=Widget(HUD,UI_PUSHBUTTON);
	PUBUTTON3.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
	PUBUTTON3.setSize(120,120);
	PUBUTTON3.setparam(PARAM_THEME,"LINEAR:textures/ui/powerup_normal.image,textures/ui/powerup_normal.image,textures/ui/powerup_pressed.image");
	PUBUTTON3.setPos(195,420);
	PUBUTTON3.setOpacity(0.0);
	PUBUTTON3.setParam(PARAM_ONCLICK,"cbPUButton(3)");
	PUBUTTON3.enableGhost(1);
	 
	PUSLOT1=Widget(HUD,UI_ICON);
	PUSLOT1.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
	PUSLOT1.setSize(PUBUTTON1.getW()/2,PUBUTTON1.getH()/2);
	PUSLOT1.setparam(PARAM_IMAGE,"LINEAR:textures/ui/pu.shoe.image");
	PUSLOT1.setPos(PUBUTTON1.getX(),PUBUTTON1.getY());
	PUSLOT1.enableGhost(1);
	PUSLOT1.setOpacity(CONST_PUOFF_OPACITY);

	PUSLOT2=Widget(HUD,UI_ICON);
	PUSLOT2.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
	PUSLOT2.setSize(PUBUTTON2.getW()/2,PUBUTTON2.getH()/2);
	PUSLOT2.setparam(PARAM_IMAGE,"LINEAR:textures/ui/pu.rock.image");
	PUSLOT2.setPos(PUBUTTON2.getX(),PUBUTTON2.getY());
	PUSLOT2.enableGhost(1);
	PUSLOT2.setOpacity(CONST_PUOFF_OPACITY);

	PUSLOT3=Widget(HUD,UI_ICON);
	PUSLOT3.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
	PUSLOT3.setSize(PUBUTTON3.getW()/2,PUBUTTON3.getH()/2);
	PUSLOT3.setparam(PARAM_IMAGE,"LINEAR:textures/ui/pu.potion.image");
	PUSLOT3.setPos(PUBUTTON3.getX(),PUBUTTON3.getY());
	PUSLOT3.enableGhost(1);
	PUSLOT3.setOpacity(CONST_PUOFF_OPACITY);	
	
	int label_offset_x=-39;
	int label_offset_y=7;
	PULABEL1=Widget(HUD,UI_TEXFONTLABEL);
	setupFont(PULABEL1,20,"center");
	PULABEL1.setSize(32,32);	
	PULABEL1.setPos(PUBUTTON1.getX()+label_offset_x,PUBUTTON1.getY()+label_offset_y);
	PULABEL1.enableGhost(1);

	PULABEL2=Widget(HUD,UI_TEXFONTLABEL);
	setupFont(PULABEL2,20,"center");
	PULABEL2.setSize(32,32);	
	PULABEL2.setPos(PUBUTTON2.getX()+label_offset_x,PUBUTTON2.getY()+label_offset_y);
	PULABEL2.enableGhost(1);

	PULABEL3=Widget(HUD,UI_TEXFONTLABEL);
	setupFont(PULABEL3,20,"center");
	PULABEL3.setSize(32,32);	
	PULABEL3.setPos(PUBUTTON3.getX()+label_offset_x,PUBUTTON3.getY()+label_offset_y);
	PULABEL3.enableGhost(1);

	PURING1=Widget(HUD,UI_LOADRING);
	PURING1.setParam(PARAM_IMAGE,"LINEAR:textures/ui/powerup_ring.image");
	PURING1.setParam(PARAM_ROTATION_SPEED,"10.0");
	PURING1.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);	
	PURING1.enableGhost(1);
	PURING1.setSize(105,105);
	PURING1.setPos(PUBUTTON1.getX(),PUBUTTON1.getY()-3);
	PURING1.setOpacity(0.0);

	PURING2=Widget(HUD,UI_LOADRING);
	PURING2.setParam(PARAM_IMAGE,"LINEAR:textures/ui/powerup_ring.image");
	PURING2.setParam(PARAM_ROTATION_SPEED,"10.0");
	PURING2.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);	
	PURING2.enableGhost(1);
	PURING2.setSize(105,105);
	PURING2.setPos(PUBUTTON2.getX(),PUBUTTON2.getY()-3);
	PURING2.setOpacity(0.0);

	PURING3=Widget(HUD,UI_LOADRING);
	PURING3.setParam(PARAM_IMAGE,"LINEAR:textures/ui/powerup_ring.image");
	PURING3.setParam(PARAM_ROTATION_SPEED,"10.0");
	PURING3.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);	
	PURING3.enableGhost(1);
	PURING3.setSize(105,105);
	PURING3.setPos(PUBUTTON3.getX(),PUBUTTON3.getY()-3);
	PURING3.setOpacity(0.0);
	
	PUCONTROLLER1=Widget(HUD,UI_ICON);
	PUCONTROLLER1.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
	PUCONTROLLER1.enableGhost(1);
	PUCONTROLLER1.setSize(64,64);
	PUCONTROLLER1.setPos(PUBUTTON1.getX()+34,PUBUTTON1.getY()+34);
	PUCONTROLLER1.setOpacity(0.0);
		
	PUCONTROLLER2=Widget(HUD,UI_ICON);
	PUCONTROLLER2.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
	PUCONTROLLER2.enableGhost(1);
	PUCONTROLLER2.setSize(64,64);
	PUCONTROLLER2.setPos(PUBUTTON2.getX()+34,PUBUTTON2.getY()+34);
	PUCONTROLLER2.setOpacity(0.0);

	PUCONTROLLER3=Widget(HUD,UI_ICON);
	PUCONTROLLER3.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
	PUCONTROLLER3.enableGhost(1);
	PUCONTROLLER3.setSize(64,64);
	PUCONTROLLER3.setPos(PUBUTTON3.getX()+34,PUBUTTON3.getY()+34);
	PUCONTROLLER3.setOpacity(0.0);
	
	LNOTIFICATION_STREAKS=Widget(HUD,UI_LOADRING);
	LNOTIFICATION_STREAKS.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
	LNOTIFICATION_STREAKS.setParam(PARAM_IMAGE,"LINEAR:textures/ui/homestreaks.image");
	LNOTIFICATION_STREAKS.setParam(PARAM_ROTATION_SPEED,"-3");
	LNOTIFICATION_STREAKS.setOpacity(0.0);
	LNOTIFICATION_STREAKS.enableGhost(1);
	
	LNOTIFICATION=Widget(HUD,UI_TEXFONTLABEL);
	LNOTIFICATION.setParam(PARAM_FONTIMAGE,"LINEAR:textures/ui/font.image");
	LNOTIFICATION.setparam(PARAM_FONTSPACING,"-0.45");
	LNOTIFICATION.setparam(PARAM_TEXTALIGN,"center");
	LNOTIFICATION.enableGhost(1);
	LNOTIFICATION.hide();
	LNOTIFICATION.setSize(1,1);

	LNOTIFICATION_CURVE=Curve();
	LNOTIFICATION_CURVE.addKey("L20:0;L40:4;L43:40;L0:50");
	
	LNOTIFICATION_STREAKS_CURVE=Curve();
	LNOTIFICATION_STREAKS_CURVE.addKey("L0:0;L1:10;L1:40;L0:45");
	
	CARROT_UI_CURVE=Curve();
	CARROT_UI_CURVE.addKey("L36:0;L26:"+string(CONST_CARROT_UI_TIMEOUT));
	
	//pause menu
	Widget BPAUSE=Widget(HUD,UI_PUSHBUTTON);
	BPAUSE.setPivot(PIVOTX_RIGHT,PIVOTY_BOTTOM);
	BPAUSE.setSize(90,90);
	BPAUSE.setPos(795,480);
	BPAUSE.setparam(PARAM_THEME,"LINEAR:textures/ui/ui_pause_normal.image,textures/ui/ui_pause_normal.image,textures/ui/ui_pause_pressed.image");
	BPAUSE.setParam(PARAM_ONCLICK,"cbPause");	
	
	LABELFPS=Widget(HUD,UI_TEXFONTLABEL);
	setupFont(LABELFPS,64,"right");
	LABELFPS.setPivot(PIVOTX_RIGHT,PIVOTY_BOTTOM);
	LABELFPS.setPos(800-90,480-15);
	LABELFPS.setParam(PARAM_TEXT,"60");
	LABELFPS.enableGhost(1);	
	LABELFPS.hide();
	
	BSHOWFPS=0;
	if (debug()) {
		BSHOWFPS=int(appPropertyGet(APPNAME,"showfps","1"));
	}
	if (BSHOWFPS) {
		LABELFPS.show();
		addListener(ON_TIMER,"1000","printFPS");
	}
	
	addListener(ON_EVENT,"SHOW","e_show");
	addListener(ON_EVENT,"HIDE","e_hide");
	addListener(ON_EVENT,"LOAD","e_load");	
	addListener(ON_EVENT,"ENEMY_ISROCKED","e_enemy_isrocked");
	addListener(ON_EVENT,"GOPHER_ISHIT","e_gopher_ishit");
	addListener(ON_EVENT,"CLUNNY_ISHIT","e_clunny_ishit");
	//addListener(ON_EVENT,"CLUNNY_ISJUMPED","e_clunny_isjumped");
	addListener(ON_EVENT,"HEDGE_ISHIT","e_hedge_ishit");
	addListener(ON_EVENT,"HEDGE_ISJUMPED","e_hedge_isjumped");
	addListener(ON_EVENT,"BUNNY_ISHIT","e_bunny_ishit");
	addListener(ON_EVENT,"GOPHER_LOCK","e_gopherlock");
	addListener(ON_EVENT,"GOPHER_UNLOCK","e_gopherunlock");
	addListener(ON_EVENT,"DO_TAILHIT","e_do_tailhit");
	addListener(ON_EVENT,"DO_FIREHIT","e_do_firehit");
	addListener(ON_EVENT,"VOLUME","e_volume");
	addListener(ON_EVENT,"ROCK_OFF","e_rock_off");
	addListener(ON_EVENT,"FORCESTOP","e_forcestop");
	addListener(ON_EVENT,"UNFORCESTOP","e_unforcestop");
	addListener(ON_EVENT,"ROADBLOCK","e_roadblock");
	addListener(ON_EVENT,"ROADBLOCK_OFF","e_roadblock_off");
	
	//Widget HIDDENPOWERUP=Widget(HUD,UI_PUSHBUTTON);
	//HIDDENPOWERUP.setSize(64,64);
	//HIDDENPOWERUP.setParam(PARAM_ONCLICK,"cbForceEggs");

	//fCall("showController");
	
}
//void cbForceEggs() {
	//fCall("enableFire(1)");
	//fCall("enableTail(1)");
	//fCall("enableWings(1)");
//}
void addNotification(string _text,int _positive,float2 _pos) {
	if (LNOTIFICATION_WAIT && LNOTIFICATION_TIMEOUT>0.1) return;
	LNOTIFICATION_TIMEOUT=0.0;
	LNOTIFICATION.setParam(PARAM_TEXT,_text);
	if (_positive) {
		LNOTIFICATION.setParam(PARAM_COLOR,"176,234,0,255");
	}
	else {
		LNOTIFICATION.setParam(PARAM_COLOR,"255,0,0,255");
	}
	LNOTIFICATION.setPos(400,scaleUI(150));
	LNOTIFICATION_TARGET_POS=_pos-float2(400,scaleUI(150));
}

void addNotification(string _text,int _positive) {
	addNotification(_text,_positive,float2(400,150));
}

void addNotificationDragon(string _text) {
	if (LNOTIFICATION_WAIT && LNOTIFICATION_TIMEOUT>0.1) return;
	addNotification(_text,1);
	LNOTIFICATION_WAIT=1;
	LNOTIFICATION_STREAKS.setPos(400,scaleUI(150));	
	LNOTIFICATION_STREAKS.setOpacity(0.0);
	LNOTIFICATION_STREAKS.setSize(0,0);
	LNOTIFICATION.setParam(PARAM_COLOR,"255,200,55,255");
}

void updateNotification() { 
	if (LNOTIFICATION_TIMEOUT<CONST_LNOTIFICATION_TIMEOUT) {
		LNOTIFICATION_TIMEOUT+=fElapsed();
		float eval=LNOTIFICATION_CURVE.evaluate(LNOTIFICATION_TIMEOUT);
		LNOTIFICATION.setparam(PARAM_FONTSIZE,string(eval)+","+eval);
		float2 offset=float2(0,0);
		if (LNOTIFICATION_TIMEOUT>40) {
			offset=lerp(float2(0,0),LNOTIFICATION_TARGET_POS,(LNOTIFICATION_TIMEOUT-40.0)/(CONST_LNOTIFICATION_TIMEOUT-40.0));
		}
		LNOTIFICATION.setPos(400+int(offset.x()),scaleUI(150)+int(offset.y()));		
		LNOTIFICATION.show();
		
		if (LNOTIFICATION_WAIT) {
			eval=LNOTIFICATION_STREAKS_CURVE.evaluate(LNOTIFICATION_TIMEOUT);
			LNOTIFICATION_STREAKS.setOpacity(eval);
			LNOTIFICATION_STREAKS.setSize(300*eval,300*eval);
		}
	}
	else {
		LNOTIFICATION_WAIT=0;
		LNOTIFICATION.hide();
	}
}
void cbPause() {
	if (BPAUSED || (!this.isVisible()) || (!HUD.isVisible())) return;	
	snd.fplay("sounds/button.audio");
	BPAUSED=1;
	enableTick(0);
	debugLog("ui.play -> launching pause menu");
	addListener(ON_EVENT,"RESUME","e_resume");	
	sendEvent("PAUSE","ui.options");
	HUD.fadeOut(200,"");
}
void e_resume() {
	if (BPAUSED==0) return;
	BPAUSED=0;
	debugLog("ui.play -> resuming from pause menu");
	delListener(ON_EVENT,"RESUME","e_resume");
	enableTick(1);
	HUD.fadeIn(200,"");
} 
void e_show() {
	//open sound node
	vfile vfsnd;
	vfsnd=vfopen("/sector/ui.sound","r");
	snd=Sound(vfsnd);
	fCall("e_volume");
		
	int i;
	if (PU1_STACK) {
		PUBUTTON1.enableGhost(0);
		PUBUTTON1.fadeIn(200,"");PUCONTROLLER1.fadeIn(200,"");
		PUSLOT1.fadeIn(200,"");
		if (PU1_STACK>1) PULABEL1.setParam(PARAM_TEXT,string(PU1_STACK));
		else PULABEL1.setParam(PARAM_TEXT,string());
	}
	if (PU2_STACK) {
		PUBUTTON2.enableGhost(0);
		PUBUTTON2.fadeIn(200,"");PUCONTROLLER2.fadeIn(200,"");
		PUSLOT2.fadeIn(200,"");
		if (PU2_STACK>1) PULABEL2.setParam(PARAM_TEXT,string(PU2_STACK));
		else PULABEL2.setParam(PARAM_TEXT,string());
	}
	if (PU3_STACK) {
		PUBUTTON3.enableGhost(0);
		PUBUTTON3.fadeIn(200,"");PUCONTROLLER3.fadeIn(200,"");
		PUSLOT3.fadeIn(200,"");
		if (PU3_STACK>1) PULABEL3.setParam(PARAM_TEXT,string(PU3_STACK));
		else PULABEL3.setParam(PARAM_TEXT,string());
	}
	
	if (TIMEOUT_PU1>0.0) {
		float tmp=TIMEOUT_PU1;
		PU1_STACK++;
		fCall("cbPUButton(1)");
		TIMEOUT_PU1=tmp;
	}
	if (TIMEOUT_PU2>0.0) {
		float tmp=TIMEOUT_PU2;
		PU2_STACK++;
		fCall("cbPUButton(2)");
		TIMEOUT_PU2=tmp;
	}
	if (TIMEOUT_PU3>0.0) {
		float tmp=TIMEOUT_PU3;
		PU3_STACK++;
		fCall("cbPUButton(3)");
		TIMEOUT_PU3=tmp;
	}
	

	Event e=Event("GOTO",hero);
	e.addArg( hero_file.getPos() );
	e.addArg( hero_file.getRot().h() );
	sendEvent(e);
	
	int heading=int(normalizeAngle(hero_file.getRot().h()));
	//N
	if (heading<45) {
		[hero->setNextDir("N")];
		[hero->setDir("N")];
	}
	//W
	else if (heading<(90+45)) {
		[hero->setDir("W")];
		[hero->setNextDir("W")];
	}
	//S
	else if (heading<(180+45)) {
		[hero->setDir("S")];
		[hero->setNextDir("S")];
	}
	//E
	else if (heading<(270+45)) {
		[hero->setDir("E")];
		[hero->setNextDir("E")];
	}

	["statue"->reset()];
	if (TUT_CAN_DRAGON) ["statue"->start()];
	
	playTimer=Timer();
	playTimer.start();
	
	HUD.enableGhost(0);
	
	addListener(ON_TICK,"tickEnemies");
	addListener(ON_TICK,"tickBunny");
	addListener(ON_TICK,"tickUI");	
	addListener(ON_MOUSEDOWN,"cbMouseDown");
	addListener(ON_MOUSEUP,"cbMouseUp");
	addListener(ON_MOUSEMOTION,"cbMouseMotion");
	addListener(ON_APPFOCUSLOST,"cbAppPause");
	addListener(ON_KEYDOWN,"cbKeyDown");
	
	addListener(ON_TIMEOUT_MSEC,"200","cbOpenGamePad");
	
	if (PAUSE_AFTER_RESUME>0) {
		PAUSE_AFTER_RESUME = 0;
		BPAUSED=0;
		cbPause();
	}
}
void cbOpenGamePad() {
	gpad.onButtonDown("cbPadButtonDown");
}
void cbKeyDown(int _key) {
	if (_key==BK_S) {
		errLog("Keyboard pause");	
		fCall("cbAppPause");
	} 
	else if (_key==BK_A){
		CUR_TIMER = CUR_TIMER+500;
	}
}
void e_volume() {
	if (snd) snd.setVolume(float(appPropertyGet(APPNAME,"sfxvol","1.0")));
}
void e_hide() {
	clearListeners(ON_TIMEOUT_FRAMES);	
	clearListeners(ON_APPFOCUSLOST);
	enableTick(0);
	HUD.enableGhost(1);
	addListener(ON_TIMEOUT_MSEC,"1500","cbHide");
	BPAUSED=1; 
	analytics_dispatch();
}
void cbHide() {
	this.hide();
	["ui.init"->clearRender()];
}
void cbAppPause() {
	if (!this.isVisible()) { return; }	
	fCall("saveState");
	["sc.loader"->e_AppPause()];
}
public void enableWings(int _asegg) {
	if (!TUT_CAN_DRAGON) return;
	if ([hero->hasWings() as int]) return;
	EGGUI_BG.show();	
	EGGUI_WINGS.show();
	if (EGGUI_WINGS.isVisible() && EGGUI_TAIL.isVisible() && EGGUI_FIRE.isVisible()) {
		CARROT_AUDIO_COUNTER_MAX=4;	
		addNotificationDragon("Dragon set complete");
	}
	else {
		if (_asegg) addNotificationDragon("You are almost flying");	
		else addNotificationDragon("Dragon wings");
	}
	SCORE_COUNT+=5;
	BONUS_JUMP+=5;
	LNOTIFICATION_WAIT=1;
	addListener(ON_TIMEOUT_FRAMES,"1","cbGrowWings");
}
public void enableFire(int _asegg) {
	if (!TUT_CAN_DRAGON) return;
	if ([hero->hasFire() as int]) return;
	EGGUI_BG.show();
	EGGUI_FIRE.show();
	if (EGGUI_WINGS.isVisible() && EGGUI_TAIL.isVisible() && EGGUI_FIRE.isVisible()) {
		CARROT_AUDIO_COUNTER_MAX=4;		
		addNotificationDragon("Dragon set complete");
	}
	else {
		if (_asegg) addNotificationDragon("You are on fire");	
		else addNotificationDragon("Dragon fire");
	}
	SCORE_COUNT+=5;
	LNOTIFICATION_WAIT=1;
	sendEvent("FIRE",hero);	
}
public void enableTail(int _asegg) {
	if (!TUT_CAN_DRAGON) return;
	if ([hero->hasTail() as int]) return;
	EGGUI_BG.show();	
	EGGUI_TAIL.show();
	SCORE_COUNT+=5;
	BONUS_JUMP+=5;
	if (EGGUI_WINGS.isVisible() && EGGUI_TAIL.isVisible() && EGGUI_FIRE.isVisible()) {
		CARROT_AUDIO_COUNTER_MAX=4;		
		addNotificationDragon("Dragon set complete");		
	}
	else {
		if (_asegg) addNotificationDragon("Deadly jump");	
		else addNotificationDragon("Dragon tail");
	}
	LNOTIFICATION_WAIT=1;
	addListener(ON_TIMEOUT_FRAMES,"1","cbGrowTail");
}
//void e_clunny_isjumped() {
//	snd.play("sounds/bonus1.audio");
//	EGG_JUMPS_INROW++;
//	if (EGG_JUMPS_INROW==5) {
		//enableWings(1);
	//}
	//else {
		//SCORE_COUNT+=2;
		//BONUS_JUMP+=2;
		//addNotification("+2 Carrots",1,float2(scaleUI(310),scaleUI(20)));	
	//}
//}
void cbGrowWings() {
	sendEvent("WINGS",hero);
}
void e_clunny_ishit(int _hits,int _jumps) {
	int bonus=0;
	if (_hits>0) {
		EGG_SQUASH_INROW++;
		if (EGG_SQUASH_INROW==3) {
			enableTail(1);
		}
		else {
			bonus+=(8*_hits);
			SCORE_COUNT+=(8*_hits);
			BONUS_JUMP+=(8*_hits);
		}
	}
	if (_jumps>0) {
		snd.play("sounds/bonus1.audio");
		EGG_JUMPS_INROW++;
		if (EGG_JUMPS_INROW==5) {
			enableWings(1);
		}
		else {
			SCORE_COUNT+=(2*_jumps);
			BONUS_JUMP+=(2*_jumps);
			bonus+=(2*_jumps);
		}
	}
	if (bonus>0) {
		addNotification("+"+string(bonus)+" Carrots",1,float2(scaleUI(310),scaleUI(20)));
	}
}
void cbGrowTail() {
	sendEvent("TAIL",hero);
}
void e_enemy_isrocked() {
	addNotification("+3 Carrots",1,float2(scaleUI(310),scaleUI(20)));		
	snd.play("sounds/bonus1.audio");
	SCORE_COUNT+=3;
	BONUS_JUMP+=3;
}
void e_gopher_ishit() {
	addNotification("+5 Carrots",1,float2(scaleUI(310),scaleUI(20)));		
	SCORE_COUNT+=5;
	BONUS_JUMP+=5;
}
void e_hedge_isjumped() {
	addNotification("+1 Carrot",1,float2(scaleUI(310),scaleUI(20)));
	snd.play("sounds/bonus1.audio");
	SCORE_COUNT+=1;
	BONUS_JUMP+=1;
}
void e_hedge_ishit() {
	addNotification("+4 Carrots",1,float2(scaleUI(310),scaleUI(20)));		
	SCORE_COUNT+=4;
	BONUS_JUMP+=4;
}
void e_bunny_ishit() {
	int prev=SCORE_COUNT;
	COUNTER_LOSE++;
	if (COUNTER_LOSE<=5) {
		SCORE_COUNT=max(SCORE_SAFE_CAP,SCORE_COUNT-(COUNTER_LOSE+1));
	}
	else {
		SCORE_COUNT=max(SCORE_SAFE_CAP,SCORE_COUNT-6);
	}
	if (SCORE_COUNT<prev) {
		addNotification("-"+(prev-SCORE_COUNT)+" Carrots",0,float2(scaleUI(310),scaleUI(20)));		
	}
	EGG_JUMPS_INROW=0;
	EGG_CARROTS_INROW=0;
	EGG_SQUASH_INROW=0;
}
void e_rock_off() {
	TIMEOUT_PU2=MIN(TIMEOUT_PU2,30.0);
}
int checkTail(String dir,float3 _heropos,vfile _enemy,float _dist) {
	float3 norm=_enemy.getPos()-_heropos;
	if (length(norm)<=_dist) {
		return 1;
	}	
	return 0;
}
void doEggHit(float _radius) {
	if ([hero->isDragon() as int]) _radius+=2.0;
	float3 heropos=hero_file.getPos();
	string tag,dir=[hero->getDir() as string];
	int i,num;
	num=l_enemies.num();
	for (i=0;i<num;i++) {
		if (checkTail(dir,heropos,l_enemies.get(i),_radius)) {
			tag=l_enemies.get(i).getTag();
			if (![tag->isHit() as int]) {
				sendEvent("HIT",tag);
				e_clunny_ishit(1,0);
			}
		}
	}
	num=l_hedgehog.num();
	for (i=0;i<num;i++) {
		if (checkTail(dir,heropos,l_hedgehog.get(i),_radius)) {
			tag=l_hedgehog.get(i).getTag();
			if (![tag->isHit() as int]) {
				sendEvent("FORCE_HIT",tag);
				e_hedge_ishit();
			}
		}
	}
	num=l_gopher.num();
	for (i=0;i<num;i++) {
		if (checkTail(dir,heropos,l_gopher.get(i),_radius)) {
			sendEvent("HIT",l_gopher.get(i).getTag());
		}
	}
}
void e_do_tailhit() {
	doEggHit(8.0);
}
void e_do_firehit() {
	doEggHit(6.0);
}
void e_load() {
	["ui.init"->renderSector()];

	enableTick(1);

	gpad.onButtonDown("");

	carrots.show();
	dragon_coins.hide();
	
	roadblock_dir="";
	locked_dir="";
	force_stop=0;
	force_bunny_back=0;
	BPAUSED=0;

	CLUNNY_AUDIO_CANPLAY=1;
	CARROT_AUDIO_COUNTER=1;
	CARROT_AUDIO_COUNTER_MAX=8;
	
	TUT_CAN_JUMP=1;
	TUT_CAN_GOBACK=1;
	TUT_CAN_DRAGON=1;
	
	if (["sc.loader"->getTheme() as string]=="forest") {
		int lvl=["sc.loader"->getLevel() as int];
		//if (lvl<2) TUT_CAN_GOBACK=0;
		//if (lvl<4) TUT_CAN_JUMP=0;
		if (lvl<14) TUT_CAN_DRAGON=0;
	}
	
	
	int mazedim=["sc.loader"->getMazeDim() as int];
	int dimx=mazedim*3;
	int dimz=mazedim*3;
	
	//read grid
	float2 f1=float2( ["sc.loader"->getLeftPos() as float], ["sc.loader"->getTopPos() as float] );
	float2 f2=float2( ["sc.loader"->getRightPos() as float], ["sc.loader"->getBottomPos() as float] );
	gridmin=float2( min( f1.x(),f2.x() ),min( f1.y(),f2.y() );
	gridmax=float2( max( f1.x(),f2.x() ),max( f1.y(),f2.y() );
	gridinterval=["sc.loader"->getInterval() as float];
	
	posgrid=Grid(dimx,dimz);
	itemgrid=Grid(dimx,dimz);
	pivotindexgrid=Grid(dimx,dimz);
	trapgrid=Grid(dimx,dimz);
	l_deadends=listvf();
	l_pups=listvf();
	l_clocks=listvf();
	l_dragonslots=listvf();
	//initialize grid
	int i,x,y,num;	
	vfile tmp;
	for (x=0;x<dimx;x++) {
		for (y=0;y<dimz;y++) {
			posgrid.seti(x,y,CONST_GRID_NULL);
			itemgrid.setvf(x,y,tmp);
			trapgrid.setvf(x,y,tmp);
			pivotindexgrid.seti(x,y,-1);
		} 
	} 

	//read carrots
	debugLog("ui.play: Reading carrots");
	l_gopher.clear();	
	carrots.clearPivots();
	dragon_coins.clearPivots();
	CARROTS_LEFT=0;
	lists ll=vfreadfolder("/sector/carrots");
	int index=0;
	int hiddencarrots=0;
	string tmpname;
	num=ll.num();
	for (i=0;i<num;i++) {
		tmp=vfopen(ll.get(i),"r");
		x=toGridX(tmp.getPos().x());
		y=toGridY(tmp.getPos().z());
		itemgrid.setvf(x,y,tmp);
		tmpname=tmp.name();
		if (tmpname.startsWithNoCase("carrot")) {
			posgrid.seti(x,y,CONST_GRID_CARROT);
			carrots.addPivot(tmp.getWorld());
			dragon_coins.addPivot(tmp.getWorld());
			pivotindexgrid.seti(x,y,index);			
			if (tmp.isVisible()) { CARROTS_LEFT++; }
			else { carrots.hidePivot(index); dragon_coins.hidePivot(index); hiddencarrots++; }
			index++;
		}	
		else if (tmpname.startsWithNoCase("pu")) {			
			posgrid.seti(x,y,CONST_GRID_PUP);
			if (tmp.isVisible()) l_pups.push(tmp);
			else l_dragonslots.push(tmp);
			//clocks
			if (tmpname.startsWithNoCase("pu_clock")) {
				if (tmp.isVisible()) l_clocks.push(tmp);
			}
		}
		else if (tmpname.startsWithNoCase("trap")) {			
			posgrid.seti(x,y,CONST_GRID_TRAP);
			if (tmp.getTag().startsWithNoCase("gopher")) {
				l_gopher.push(tmp);
			}
		}
	} 
	debugLog("ui.play: Hidden carrots "+string(hiddencarrots));
	//read deadends
	ll=vfreadfolder("/sector/corridors");
	num=ll.num();
	for (i=0;i<num;i++) {
		tmp=vfopen(ll.get(i),"r");
		tmpname=tmp.name();
		if (tmpname.startsWithNoCase("deadend")) {
			x=toGridX(tmp.getPos().x());
			y=toGridY(tmp.getPos().z());
			itemgrid.setvf(x,y,tmp);
			posgrid.seti(x,y,CONST_GRID_DEADEND);
			l_deadends.push(tmp);
		}
		else if (tmpname.startsWithNoCase("log")) {
			x=toGridX(tmp.getPos().x());
			y=toGridY(tmp.getPos().z());
			tmp=vfopen(ll.get(i)+"/log.script","r");
			trapgrid.setvf(x,y,tmp);
		}
		else if (tmpname.startsWithNoCase("pier")) {
			x=toGridX(tmp.getPos().x());
			y=toGridY(tmp.getPos().z());
			tmp=vfopen(ll.get(i)+"/pier.script","r");
			trapgrid.setvf(x,y,tmp);
		}		
		else if (tmpname.startsWithNoCase("pond")) {
			x=toGridX(tmp.getPos().x());
			y=toGridY(tmp.getPos().z());
			tmp=vfopen(ll.get(i)+"/pond.script","r");
			trapgrid.setvf(x,y,tmp);
		}
		else if (tmpname.startsWithNoCase("quicksand")) {
			x=toGridX(tmp.getPos().x());
			y=toGridY(tmp.getPos().z());
			tmp=vfopen(ll.get(i)+"/quicksand.script","r");
			trapgrid.setvf(x,y,tmp);
		}
		else if (tmpname.startsWithNoCase("crack")) {
			x=toGridX(tmp.getPos().x());
			y=toGridY(tmp.getPos().z());
			tmp=vfopen(ll.get(i)+"/crack.script","r");
			trapgrid.setvf(x,y,tmp);
		}
		else if (tmpname.startsWithNoCase("bridge")) {
			x=toGridX(tmp.getPos().x());
			y=toGridY(tmp.getPos().z());
			tmp=vfopen(ll.get(i)+"/bridge.script","r");
			trapgrid.setvf(x,y,tmp);
		}
		else if (tmpname.startsWithNoCase("duck")) {
			x=toGridX(tmp.getPos().x());
			y=toGridY(tmp.getPos().z());
			tmp=vfopen(ll.get(i)+"/duck.script","r");
			trapgrid.setvf(x,y,tmp);
		}
		else if (tmpname.startsWithNoCase("faucet")) {
			x=toGridX(tmp.getPos().x());
			y=toGridY(tmp.getPos().z());
			tmp=vfopen(ll.get(i)+"/faucet.script","r");
			trapgrid.setvf(x,y,tmp);
		}
		else tmp.close();
	}

	hero_file=vfopen("/sector/hero","r");
	l_enemies.clear();
	l_hedgehog.clear();
	l_penguin.clear();
	ll=vfreadfolder("/sector/enemies");
	num=ll.num();
	for (i=0;i<num;i++) {
		tmp=vfopen(ll.get(i),"r");
		tmpname=tmp.name();
		if (tmpname.startsWithNoCase("enemy")) {
			l_enemies.push(tmp);
		}
		else if (tmpname.startsWithNoCase("hedgehog")) {
			l_hedgehog.push(tmp);
		}
		else if (tmpname.startsWithNoCase("penguin")) {
			l_penguin.push(tmp);
		}
		else tmp.close();
	}
	
	//prepare widgets
	while(l_wclunny.num()<l_enemies.num()) {
		Widget w=Widget(HUD,UI_ICON);
		w.setSize(64,64);
		w.enableGhost(1);
		w.setPivot(PIVOTX_CENTER,PIVOTY_MIDDLE);
		w.setparam(PARAM_IMAGE,"LINEAR:textures/ui/clunny_compass_bg.image");
		l_wclunny.push(w);		
		
		Widget c=Widget(w,UI_ICON);
		c.setSize(64,64);
		c.enableGhost(1);
		c.setparam(PARAM_IMAGE,"LINEAR:textures/ui/clunny_compass_fg.image");
		l_wclunnycompass.push(c);
	} 
	num=l_wclunny.num();
	for (i=0;i<num;i++) {
		Widget(l_wclunny.get(i)).setOpacity(0.0);
	}
	
	//initialize UI
	EGGUI_BG.hide();
	EGGUI_FIRE.hide();
	EGGUI_WINGS.hide();
	EGGUI_TAIL.hide();
		
	STAR1.hide();
	STAR2.hide();
	STAR1.setOpacity(0.0);
	STAR2.setOpacity(0.0);
	STARFLASH1.setOpacity(0.0);
	STARFLASH2.setOpacity(0.0);
	STARFG.setSize(scaleUI(128),0);
	STARFG.setPos(0,scaleUI(128)-STARFG.getH());
	STARICON.setPos(0,-STARFG.getY());			
	
	LNOTIFICATION_STREAKS.setOpacity(0.0);
	
	PUBUTTON1.enableGhost(1);
	PUBUTTON1.hide();PUCONTROLLER1.hide();

	PUBUTTON2.enableGhost(1);
	PUBUTTON2.hide();PUCONTROLLER2.hide();

	PUBUTTON3.enableGhost(1);
	PUBUTTON3.hide();PUCONTROLLER3.hide();
	
	PULABEL1.setParam(PARAM_TEXT,string());
	PULABEL2.setParam(PARAM_TEXT,string());
	PULABEL3.setParam(PARAM_TEXT,string());
	
	PUSLOT1.setOpacity(CONST_PUOFF_OPACITY);
	PUSLOT2.setOpacity(CONST_PUOFF_OPACITY);
	PUSLOT3.setOpacity(CONST_PUOFF_OPACITY);
	
	PURING1.setOpacity(0.0);	
	PURING2.setOpacity(0.0);	
	PURING3.setOpacity(0.0);	

	TIMEBAR.clearAnimate("");
	
	HUD.show();
	HUD.setOpacity(1.0);
	this.show();
	
	//initialize UI
	MAX_TIMER=getFps()* ["sc.loader"->getRequiredTime() as float];
	CUR_TIMER=getFps()* ["sc.loader"->getCurrentTime() as float];
	SCORE_COUNT=["sc.loader"->getCurrentScore() as int];
	TOTAL_CARROTS=["sc.loader"->getTotalCarrots() as int];
	BONUS_JUMP=["sc.loader"->getBonusJump() as int];
	BONUS_TIME=["sc.loader"->getBonusTime() as int];
	BONUS_PUP=["sc.loader"->getBonusPUP() as int];
	PU1_STACK=["sc.loader"->getStackPU1() as int];
	PU2_STACK=["sc.loader"->getStackPU2() as int];
	PU3_STACK=["sc.loader"->getStackPU3() as int];
	COUNTER_LOSE=["sc.loader"->getCounterLose() as int];
	TIMEOUT_PU1=["sc.loader"->getTimeoutPU1() as float];
	TIMEOUT_PU2=["sc.loader"->getTimeoutPU2() as float];
	TIMEOUT_PU3=["sc.loader"->getTimeoutPU3() as float];
	EGG_CARROTS_INROW=["sc.loader"->setEggCarrotsInRow() as int];
	EGG_JUMPS_INROW=["sc.loader"->setEggJumpsInRow() as int];
	EGG_SQUASH_INROW=["sc.loader"->setEggSquashInRow() as int];
	
	PREV_SCORE_COUNT=0;
	SCORE_SAFE_CAP=0;
	BPAUSED=0;
	LNOTIFICATION_WAIT=0;	
	
	if (SCORE_COUNT<=0) {
		//rotate bunny to not face deadend
		float2 bunnypos=getTile(hero_file.getPos());
		//do action
		vfile bunnytilevfile=itemgrid.getvf(int(bunnypos.x()),int(bunnypos.y()));
		if (posgrid.geti(int(bunnypos.x()),int(bunnypos.y()))==CONST_GRID_DEADEND) {
			hero_file.setRot( hpr(normalizeAngle(bunnytilevfile.getRot().h()+270.0),0.0,0.0) );
			int angle=int(normalizeAngle(hero_file.getRot().h()));
			if (angle==0) [hero->setDir("N")];
			else if (angle==90) [hero->setDir("W")];
			else if (angle==180) [hero->setDir("S")];
			else [hero->setDir("E")];
		}
	}
	else {
	 PAUSE_AFTER_RESUME = 1;
	}
	
	
	clearListeners(ON_TICK);
	clearListeners(ON_MOUSEDOWN);
	clearListeners(ON_MOUSEUP);
	clearListeners(ON_MOUSEMOTION);
	clearListeners(ON_APPFOCUSLOST);
	clearListeners(ON_TIMEOUT_MSEC);
	addListener(ON_TICK,"tickhud");
	
	fCall("tickui");
}

///////////// CONTROLS ////////////////

int mouse_down=0;
float2 touch_vec;
int tap_count=0;
int skip_tap=0;
int flag_moveback=0;
int CONST_MOVE_LEFT = 1;
int CONST_MOVE_RIGHT = 2;
int CONST_MOVE_BACK = 3;
int CONST_MOVE_JUMP = 4;
int prevmove =0 ;// 0:undefined, 1: left, 2: right, 3: back, 4: jump

void procTouchVec(float _safearea) {
	if (touch_vec.length()>_safearea) {
		skip_tap=1;
		if (abs(touch_vec.x()) >= abs(touch_vec.y())) {
			if (touch_vec.x() < 0) { 
				force_stop=0;				
				if ((IS_DOUBLESWIPE==1) && (prevmove == CONST_MOVE_LEFT)){
					flag_moveback = 1;					
					if (TUT_CAN_GOBACK) [hero->setNextBack()];
					prevmove = CONST_MOVE_BACK;
				} else {
					flag_moveback=0;
					[hero->setNextLeft()];
					prevmove = CONST_MOVE_LEFT; // de
				}
				touch_vec=float2(0,0);
			} //left
			else if (touch_vec.x() > 0) { 
				force_stop=0;
				if ((IS_DOUBLESWIPE == 1) && (prevmove == CONST_MOVE_RIGHT)){
					flag_moveback = 1;					
					if (TUT_CAN_GOBACK) [hero->setNextBack()];
					prevmove = CONST_MOVE_BACK;
				} else {
					flag_moveback=0;
					[hero->setNextRight()];
					prevmove = CONST_MOVE_RIGHT;
				}
				touch_vec=float2(0,0);
			} //right
		}
		else {
			//down
			if (touch_vec.y() > 4) { 
				prevmove = CONST_MOVE_BACK; 
				if (!flag_moveback) {
					force_stop=0;
					flag_moveback=1;					
					// if (TUT_CAN_GOBACK) 
					[hero->setNextBack()];
					prevmove = CONST_MOVE_BACK;
					touch_vec=float2(0,0);
				}
			} //up
			else if (touch_vec.y()<20) {
				force_stop=0;
				flag_moveback=0;
				touch_vec=float2(0,0);
				prevmove = CONST_MOVE_JUMP;
				if (TUT_CAN_JUMP) {
					Event e=Event("JUMP",hero);
					e.addArg(jumpClickTimer.getMSec());
					e.sendEvent();
					prevmove = CONST_MOVE_JUMP;
					jumpClickTimer.start();					
				}
			}
		}
	}
}

void cbMouseMotion(float2 offset,float2 pos) {
	if (mouse_down == 0) { return; }
	if (offset.length()>=20.0) {
		skip_tap=1;
	}
	touch_vec+=offset;	
	procTouchVec(normSizef(150.0));
}
void cbMouseDown(int button,float3 mod,float2 pos) {
	int combotime = 0;
	
	if (BPAUSED || HUD.isMouseOnTop(pos)) {
		mouse_down=0;
		return;
	}
	skip_tap=0;
	mouse_down=1;
	flag_moveback=0;
	touch_vec=float2(0,0);
	clickTimer.start();
	combotime = comboTimer.getMSec();
	if (combotime > 10 && combotime < 130 ){
		IS_DOUBLESWIPE = 1;
	} else {
		IS_DOUBLESWIPE = 0;
	}
	comboTimer.start();
	jumpClickTimer.start();
}
void cbMouseUp(int _button, float3 _mod) {
	if (mouse_down == 0) { return; }
	mouse_down=0;
	if ((!skip_tap) && (touch_vec.length()<=normSizef(40.0)) && (clickTimer.getMSec()<300)) {
		force_stop=0;
		if (TUT_CAN_JUMP) {
			Event e=Event("JUMP",hero);
			e.addArg(jumpClickTimer.getMSec());
			e.sendEvent();
			jumpClickTimer.start();					
		}
	}	
	else {
		tap_count=0;
		procTouchVec(normSizef(40.0));
	}
}

////// GAMEPAD //////

void cbPadButtonDown(int _controller,int _button,int _value) {
	if (BPAUSED) { return; }
	if (_button==GAMEPAD_RIGHT || _button==GAMEPAD_RBUTTON || _button==GAMEPAD_RTRIGGER) {
		force_stop=0;	
		[hero->setNextRight()];
	}
	else if (_button==GAMEPAD_LEFT || _button==GAMEPAD_LBUTTON || _button==GAMEPAD_LTRIGGER) {
		force_stop=0;	
		[hero->setNextLeft()];
	}
	else if (_button==GAMEPAD_A) {
		force_stop=0;
		if (TUT_CAN_JUMP) {
			Event e=Event("JUMP",hero);
			e.addArg(jumpClickTimer.getMSec());
			e.sendEvent();
			jumpClickTimer.start();					
		}
	}
	else if (_button==GAMEPAD_DOWN) {
		force_stop=0;
		if (TUT_CAN_GOBACK) [hero->setNextBack()];
	}	
	else if (_button==GAMEPAD_START || _button==GAMEPAD_MENU) {
		fCall("cbPause");
	}
	else if (_button==GAMEPAD_X) {
		if (PUBUTTON1.isVisible() && PUBUTTON1.isEnabled()) {
			PUBUTTON1.doAction();
		}
	}
	else if (_button==GAMEPAD_Y) {
		if (PUBUTTON2.isVisible() && PUBUTTON2.isEnabled()) {
			PUBUTTON2.doAction();
		}
	}
	else if (_button==GAMEPAD_B) {
		if (PUBUTTON3.isVisible() && PUBUTTON3.isEnabled()) {
			PUBUTTON3.doAction();			
		}
	}
}

////// AI //////
void debugInfo() {
	if (BSHOWFPS) {
		LABELFPS.setParam(PARAM_TEXT,string(sysGetRenderFPS()));
	}
} 
void printFPS() {
	warnLog("FPS: "+string(sysGetRenderFPS()));
}
void enablePU(int _pu) {
	if (_pu==1) {
		PU1_STACK++;
		addNotification("Running Shoes!",1,float2(PUBUTTON1.getX()+(PUBUTTON1.getW()/2),PUBUTTON1.getY()+(PUBUTTON1.getH()/2)));
		snd.fplay("sounds/collect_shoes.audio"); 
		PUBUTTON1.enableGhost(0);
		PUBUTTON1.fadeIn(200,"");PUCONTROLLER1.fadeIn(200,"");
		PUSLOT1.fadeIn(200,"");
		if (PU1_STACK>1) PULABEL1.setParam(PARAM_TEXT,string(PU1_STACK));
		else PULABEL1.setParam(PARAM_TEXT,string());
		
		if (TIMEOUT_PU1<=0.0) {
			PURING1.show();
			PURING1.animate("A","L0:0;S1:1000;S0:2000","");
		}
	}
	else if (_pu==2) {
		PU2_STACK+=3;
		addNotification("Rock n' Roll!",1,float2(PUBUTTON2.getX()+(PUBUTTON2.getW()/2),PUBUTTON2.getY()+(PUBUTTON2.getH()/2)));
		snd.fplay("sounds/collect_stone.audio"); 
		PUBUTTON2.enableGhost(0);
		PUBUTTON2.fadeIn(200,"");PUCONTROLLER2.fadeIn(200,"");
		PUSLOT2.fadeIn(200,"");
		if (PU2_STACK>1) PULABEL2.setParam(PARAM_TEXT,string(PU2_STACK));
		else PULABEL2.setParam(PARAM_TEXT,string());
		
		if (TIMEOUT_PU2<=0.0) {
			PURING2.show();
			PURING2.animate("A","L0:0;S1:1000;S0:2000","");
		}
	}
	else if (_pu==3) {
		PU3_STACK++;
		addNotification("Invisibility Potion!",1,float2(PUBUTTON3.getX()+(PUBUTTON3.getW()/2),PUBUTTON3.getY()+(PUBUTTON3.getH()/2)));
		snd.fplay("sounds/collect_potion.audio"); 
		PUBUTTON3.enableGhost(0);
		PUBUTTON3.fadeIn(200,"");PUCONTROLLER3.fadeIn(200,"");
		PUSLOT3.fadeIn(200,"");
		if (PU3_STACK>1) PULABEL3.setParam(PARAM_TEXT,string(PU3_STACK));
		else PULABEL3.setParam(PARAM_TEXT,string());
		
		if (TIMEOUT_PU3<=0.0) {
			PURING3.show();
			PURING3.animate("A","L0:0;S1:1000;S0:2000","");
		}
	} 
}
void cbPUButton(int _num) {	
	if (_num==1) {
		["sc.loader"->incUse_SHOES()];
		snd.fplay("sounds/collect_shoes.audio"); 
		sendEvent("SHOES_ON",hero);
		TIMEOUT_PU1=CONST_TIMEOUT_PU1;
		PUBUTTON1.enableGhost(1);
		PUBUTTON1.setOpacity(0.5);
		PUCONTROLLER1.setOpacity(0.5);
		PUSLOT1.fadeIn(200,"");		
		PURING1.clearanimate("A");		
		PURING1.fadeIn(200,"");
		PU1_STACK--;
		if (PU1_STACK>1) PULABEL1.setParam(PARAM_TEXT,string(PU1_STACK));
		else PULABEL1.setParam(PARAM_TEXT,string());
	}
	else if (_num==2) {
		["sc.loader"->incUse_ROCK()];
		snd.fplay("sounds/collect_stone.audio"); 
		TIMEOUT_PU2=CONST_TIMEOUT_PU2;
		PUBUTTON2.enableGhost(1);
		PUBUTTON2.setOpacity(0.5);
		PUCONTROLLER2.setOpacity(0.5);
		PUSLOT2.fadeIn(200,"");		
		PURING2.clearanimate("A");
		PURING2.fadeIn(200,"");
		PU2_STACK--;
		if (PU2_STACK>1) PULABEL2.setParam(PARAM_TEXT,string(PU2_STACK));
		else PULABEL2.setParam(PARAM_TEXT,string());
		
		float2 bunny_tile=getTile(hero_file.getPos());
		int tx=int(bunny_tile.x());
		int ty=int(bunny_tile.y());
		int nx=tx,ny=ty;
		//find next tile
		string dir=[hero->getDir() as string];
		if (dir=="N") {
			while(posgrid.geti(nx,ny-1)) {
				ny--;
			}
		}
		else if (dir=="S") {
			while(posgrid.geti(nx,ny+1)) {
				ny++;
			}
		}
		else if (dir=="W") {
			while(posgrid.geti(nx-1,ny)) {
				nx--;
			}
		}
		else if (dir=="E") {
			while(posgrid.geti(nx+1,ny)) {
				nx++;
			}
		}
		Event e=Event("GOTO","rock");
		e.addArg(hero_file.getPos());
		e.addArg(itemgrid.getvf(nx,ny).getPos());
		sendEvent(e);
	}
	else if (_num==3) {
		["sc.loader"->incUse_POTION()];
		snd.fplay("sounds/collect_potion.audio"); 
		TIMEOUT_PU3=CONST_TIMEOUT_PU3;
		PUBUTTON3.enableGhost(1);
		PUBUTTON3.setOpacity(0.5);
		PUCONTROLLER3.setOpacity(0.5);
		PUSLOT3.fadeIn(200,"");		
		PURING3.clearanimate("A");
		PURING3.fadeIn(200,"");
		sendEvent("GHOST_ON",hero);
		PU3_STACK--;
		if (PU3_STACK>1) PULABEL3.setParam(PARAM_TEXT,string(PU3_STACK));
		else PULABEL3.setParam(PARAM_TEXT,string());
	}
}
void cbTimeoutPU1() {
	snd.fplay("sounds/pu-end.audio"); 
	sendEvent("SHOES_OFF",hero);
	PURING1.fadeOut(200,"");
	if (PU1_STACK>0) {
		PUBUTTON1.enableGhost(0);
		PUBUTTON1.fadeIn(200,"");PUCONTROLLER1.fadeIn(200,"");
		PUSLOT1.fadeIn(200,"");
	}
	else {
		PUBUTTON1.fadeOut(200,"");PUCONTROLLER1.fadeOut(200,"");
		PUSLOT1.setOpacity(CONST_PUOFF_OPACITY);	
	}
}
void cbTimeoutPU2() {
	PURING2.fadeOut(200,"");
	snd.fplay("sounds/pu-end.audio"); 
	if (PU2_STACK>0) {
		PUBUTTON2.enableGhost(0);
		PUBUTTON2.fadeIn(200,"");PUCONTROLLER2.fadeIn(200,"");
		PUSLOT2.fadeIn(200,"");
	}
	else {
		PUBUTTON2.fadeOut(200,"");PUCONTROLLER2.fadeOut(200,"");
		PUSLOT2.setOpacity(CONST_PUOFF_OPACITY);	
	}
}
void cbTimeoutPU3() {
	PURING3.fadeOut(200,"");
	sendEvent("GHOST_OFF",hero);
	snd.fplay("sounds/pu-end.audio"); 
	if (PU3_STACK>0) {
		PUBUTTON3.enableGhost(0);
		PUBUTTON3.fadeIn(200,"");PUCONTROLLER3.fadeIn(200,"");
		PUSLOT3.fadeIn(200,"");
	}
	else {
		PUBUTTON3.fadeOut(200,"");PUCONTROLLER3.fadeOut(200,"");
		PUSLOT3.setOpacity(CONST_PUOFF_OPACITY);	
	}
}
void tickhud() {
	updateNotification();
	debugInfo();
}
void heroGotoGrid(int _x,int _y,string _dir) {
	roadblock_dir = "";
	[hero->setDir(_dir)];
	float3 pos=itemgrid.getvf(_x,_y).getPos();
	pos.y(hero_file.getPos().y());
	Event e=Event("GOTO",hero);
	e.addArg( pos );
	if (_dir=="N") { e.addArg(0.0); }
	else if (_dir=="S") { e.addArg(180.0); }
	else if (_dir=="W") { e.addArg(90.0); }
	else if (_dir=="E") { e.addArg(270.0); }
	sendEvent(e);
}
void heroStop() {
	sendEvent("STOP",hero);
}
public void heroJumpedOut() {
	force_stop=0;
	addListener(ON_TICK,"tickBunny");
}
int returnNewDeadend(int _curde) {
	int ret = -1;
	float max_dist = -1.0;

	int i,j,num,enemynum;
	num=l_deadends.num();
	for (i=0; i<num; i++)
	{
		if (i != _curde)
		{
			float min_dist = -1.0;
			vfile de = l_deadends.get(i);
			float3 dpos = de.getPos();
			float dist = 0.0;
			enemynum=l_enemies.num();
			for (j=0;j<enemynum;j++) {
				dist = (dpos-l_enemies.get(j).getPos()).length(); if ((min_dist < 0) || (dist < min_dist)) min_dist = dist;
			}
			if ((max_dist < 0) || (min_dist > max_dist)) { max_dist = min_dist; ret = i; }
		}
	}

	if (max_dist < 0)
	{
		num=l_deadends.num();
		ret = rand(num);
		if (num>1) {
			while (ret == _curde) { ret = rand(num); }
		}
	}
	return ret;
}
int isFacingDeadEnd(vfile _deadend) {
	int rot=int(normalizeAngle(_deadend.getRot().h()));
	string dir=[hero->getDir() as string];
	if (dir=="N" && rot==270) {
		return 1;
	}
	if (dir=="S" && rot==90) {
		return 1;
	}
	if (dir=="E" && rot==180) {
		return 1;
	}
	if (dir=="W" && rot==0) {
		return 1;
	}
	return 0;
	//return int(minangle( hero_file.getRot().h(), _deadend.getRot().h()+90 )==0);
}
string getNextDeadEndDir(vfile _deadend) {
	int angle=int(normalizeAngle(_deadend.getRot().h()));
	if (angle==0) return "E";
	else if (angle==90) return "N";
	else if (angle==180) return "W";
	return "S"; 
}
void e_roadblock_off() {
	roadblock_dir = "";
}
void e_roadblock() {
	Event e=Event("GOTO",hero);
	e.addArg( hero_file.getPos() );
	roadblock_dir=[hero->getDir() as string];
	if (roadblock_dir=="N") { e.addArg(0.0); }
	else if (roadblock_dir=="S") { e.addArg(180.0); }
	else if (roadblock_dir=="W") { e.addArg(90.0); }
	else if (roadblock_dir=="E") { e.addArg(270.0); }
	sendEvent(e);
	sendEvent("STOP",hero);
}

void e_gopherlock() {
	Event e=Event("GOTO",hero);
	e.addArg( hero_file.getPos() );
	locked_dir=[hero->getDir() as string];
	//print(" GOPHERLOCK:: locked_dir = "+locked_dir );
	if (locked_dir=="N") { e.addArg(0.0); }
	else if (locked_dir=="S") { e.addArg(180.0); }
	else if (locked_dir=="W") { e.addArg(90.0); }
	else if (locked_dir=="E") { e.addArg(270.0); }
	sendEvent(e);
	sendEvent("STOP",hero);
}
void e_gopherunlock() {
	locked_dir="";
}
void e_forcestop() { 
	force_stop=1;
}
void e_unforcestop() {
	force_stop=0;
}
int canCollect(vfile _collectible) {
	if (!_collectible.isVisible()) return 0;
	return int(abs((hero_file.getPos()-_collectible.getPos()).y())<2.0);
}
int canGotoTile(vfile _entity,int _tx,int _tz,float _heading) {
	if (posgrid.exists(_tx,_tz)) {
		if (posgrid.geti(_tx,_tz)>0) {
			vfile f=trapgrid.getvf(_tx,_tz);
			if (f) {
				string name=f.name();
				if (name.startsWithNoCase("log") || name.startsWithNoCase("bridge") || name.startsWithNoCase("duck")) {
					if (![f.getTag()->validate(_heading) as int]) {
						if (_entity.name()==hero) {
							if ([hero->isMoving() as int]) {
								[hero->playTrapSound()];
							}
						}
						return 0;
					}
				}
				else if (name.startsWithNoCase("crack")) {
					if (_entity.name()!=hero) { return 0; }
				}
				else if (name.startsWithNoCase("pier")) {
					if (_entity.name().startsWithNoCase("hedgehog")) { return 0; }
					else {	
						float3 pos=_entity.getPos();
						if (![f.getTag()->validate(_entity.name(),_heading) as int]) {
							if (_entity.name()==hero) {
								if ([hero->isMoving() as int]) {
									[hero->playTrapSound()];
								}
							}
							return 0;
						}
					}
				}
			}
			return 1;
		}
	}
	return 0;
}
int canTurn(int _tx,int _tz) {
	if (posgrid.exists(_tx,_tz)) {
		if (posgrid.geti(_tx,_tz)>0) {
			vfile f=trapgrid.getvf(_tx,_tz);
			if (f) {
				string name=f.name();
				if (name.startsWithNoCase("log")) {
					return 0;
				}
				if (name.startsWithNoCase("bridge")) {
					return 0;
				}
			}
		}
	}
	return 1;
}
//used by hedgehog
public void forceBunnyBack() {
	force_bunny_back=1;
}
void tickBunny() {	
	float3 hero_pos=hero_file.getPos();	

	int NEWTILEX,NEWTILEY,i,num;
	float2 p=getTile(hero_file.getPos());
	NEWTILEX=int(p.x());
	NEWTILEY=int(p.y());
	
	//do action
	vfile tilevfile=itemgrid.getvf(NEWTILEX,NEWTILEY);
	int tiletype=posgrid.geti(NEWTILEX,NEWTILEY);
	
	//collect elements only if we are not in jumping state
	if ([hero->isOnTarget() as int]) {	
		if (tiletype==CONST_GRID_CARROT) {
			if (canCollect(tilevfile)) {
				SCORE_COUNT++;
				CARROTS_LEFT--;
				tilevfile.hide();
				carrots.hidePivot( pivotindexgrid.geti(NEWTILEX,NEWTILEY) );
				dragon_coins.hidePivot( pivotindexgrid.geti(NEWTILEX,NEWTILEY) );
				if (CARROT_AUDIO_COUNTER<=3) {
					snd.play("sounds/carrot1.audio");
				}
				else {
					if (CARROT_AUDIO_COUNTER==CARROT_AUDIO_COUNTER_MAX) {
						snd.play("sounds/carrot6.audio");
					}
					else {
						snd.play("sounds/carrot"+string(CARROT_AUDIO_COUNTER-2)+".audio");
					}
				}
				if (CARROT_AUDIO_COUNTER==CARROT_AUDIO_COUNTER_MAX) {
					CARROT_AUDIO_COUNTER=0;
					SCORE_COUNT+=2; addNotification("+2 Carrots",1,float2(scaleUI(310),scaleUI(20)));						
				}
				CARROT_AUDIO_COUNTER=min(CARROT_AUDIO_COUNTER_MAX,CARROT_AUDIO_COUNTER+1);
				EGG_CARROTS_INROW++;
				if (EGG_CARROTS_INROW==15) {
					enableFire(1);
				}
			}
			else {
				EGG_CARROTS_INROW=0;
				CARROT_AUDIO_COUNTER=1;
			}
		}
		else if (tiletype==CONST_GRID_PUP) {
			CARROT_AUDIO_COUNTER=1;
			EGG_CARROTS_INROW=0;
			if (canCollect(tilevfile)) {
				SCORE_COUNT++;
				BONUS_PUP++;
				tilevfile.hide();
				l_dragonslots.push(tilevfile);				
				l_pups.delIndex(l_pups.findIndex(tilevfile));
				if (tilevfile.name().startsWithNoCase("pu_shoe")) {
					["sc.loader"->incPick_SHOES()];
					enablePU(1);
				}
				else if (tilevfile.name().startsWithNoCase("pu_rock")) {
					["sc.loader"->incPick_ROCK()];
					enablePU(2);
				}
				else if (tilevfile.name().startsWithNoCase("pu_potion")) {
					["sc.loader"->incPick_POTION()];
					enablePU(3);
				}
				else if (tilevfile.name().startsWithNoCase("pu_clock")) {
					l_clocks.delIndex(l_clocks.findIndex(tilevfile));
					["sc.loader"->incPick_CLOCK()];
					addNotification("+15 Sec",1,float2(scaleUI(120),scaleUI(20)));
					CUR_TIMER=min(CUR_TIMER+getFPS()*15,MAX_TIMER);
					snd.fplay("sounds/collect_clock.audio"); 
				}
				else if (tilevfile.name().startsWithNoCase("pu_chest")) {
					["sc.loader"->incPick_CHEST()];
					int rnd=1+rand(4);
					if (rnd==1) {
						Event e=Event("CHEST",hero);
						e.addArg("shoe");
						sendEvent(e);
						enablePU(1);
					}
					else if (rnd==2) {
						Event e=Event("CHEST",hero);
						e.addArg("rock");
						sendEvent(e);
						enablePU(2);
					}
					else if (rnd==3) {
						Event e=Event("CHEST",hero);
						e.addArg("potion");
						sendEvent(e);
						enablePU(3);
					}
					else if (rnd==4) {
						Event e=Event("CHEST",hero);
						e.addArg("life");
						sendEvent(e);
						snd.play("sounds/bonus1.audio");
						BONUS_PUP+=9; SCORE_COUNT+=9; addNotification("+10 Carrots",1,float2(scaleUI(310),scaleUI(20)));						
					}
				}
				sendEvent("HAPPY",hero);
			}
		}
		else { CARROT_AUDIO_COUNTER=1;EGG_CARROTS_INROW=0; }
	}
		
	//check if next to node is a deadend node
	vfile deadendvfile,tmpvf;
	deadendvfile=0;
	
	num=l_deadends.num();
	for (i=0;i<num;i++) {
		tmpvf=l_deadends.get(i);
		if ((hero_pos.xz()-tmpvf.getPos().xz()).length()<=3.2) {
			if (isFacingDeadEnd(tmpvf)){
				deadendvfile=tmpvf;
				break;
			}
		}
	}
	if (deadendvfile) {
		//fix bunny pos
		float3 fixpos=float3(3.0,2.33,0.0);
		fixpos.roth(deadendvfile.getRot().h());
		fixpos=deadendvfile.getPos()+fixpos;
	
		if ([hero->isJumping() as int]) {
			Event e=Event("GOTO",hero);
			e.addArg( fixpos );
			string dir=[hero->getDir() as string];
			if (dir=="N") { e.addArg(0.0); }
			else if (dir=="S") { e.addArg(180.0); }
			else if (dir=="W") { e.addArg(90.0); }
			else if (dir=="E") { e.addArg(270.0); }
			sendEvent(e);
			return;
		}

		int index=returnNewDeadend( l_deadends.findIndex(deadendvfile) );
		deadendvfile.setMaster("jump_in",0);
		heroGotoGrid(
			toGridX(l_deadends.get(index).getPos().x()),
			toGridY(l_deadends.get(index).getPos().z()),
			getNextDeadEndDir(l_deadends.get(index)));
		
		//fix bunny pos
		hero_file.setPos(fixpos);
		hero_file.setRot( deadendvfile.getRot()+hpr(90.0,0.0,0.0) ); 
		
		Event e=Event("DEADEND",hero);
		float3 newpos=l_deadends.get(index).getPos();
		newpos.y( hero_pos.y() );
		e.addArg(newpos);
		sendEvent(e);
		delListener(ON_TICK,"tickBunny");
		return;
	}
	
	if (force_stop) {
		heroStop();
		return;
	}
	
	if (([hero->isOnTarget() as int]==0) && (force_bunny_back==0)) return;
	
	int reset_next=0;
	string next=[hero->getDir() as string];
	string nextdir=[hero->getNextDir() as string];
	string forcenext=[hero->getForceNext() as string];
	if (forcenext.length()) nextdir=forcenext;
	
	if (next!=nextdir) touch_vec=float2(0,0);
	
	//check for back
	int force_back=force_bunny_back;
	if ( (next=="N" && nextdir=="S") ||
		 (next=="S" && nextdir=="N") ||
		 (next=="W" && nextdir=="E") ||
		 (next=="E" && nextdir=="W") ) 
	{
		force_back=1;
	}
	if (((NEWTILEX%3)==1 && (NEWTILEY%3)==1) || force_back) {
		if (canTurn(NEWTILEX,NEWTILEY)) {
			next=nextdir;
		}
		reset_next=1;
	}
	int is_stopped=0;
	if (next==locked_dir || next==roadblock_dir) {
		heroStop();
		is_stopped=1;
	}
	else if (next=="N" ) {
		if (force_bunny_back && canGotoTile(hero_file,NEWTILEX,NEWTILEY-2,hero_file.getRot().h())) {
			heroGotoGrid(NEWTILEX,NEWTILEY-2,"N");
		}
		else if (canGotoTile(hero_file,NEWTILEX,NEWTILEY-1,hero_file.getRot().h())) {
			heroGotoGrid(NEWTILEX,NEWTILEY-1,"N");
		}
		else {
			heroStop();
			is_stopped=1;
		}
	}
	else if (next=="S") {
		if (force_bunny_back && canGotoTile(hero_file,NEWTILEX,NEWTILEY+2,hero_file.getRot().h())) {
			heroGotoGrid(NEWTILEX,NEWTILEY+2,"S");
		}
		else if (canGotoTile(hero_file,NEWTILEX,NEWTILEY+1,hero_file.getRot().h())) {
			heroGotoGrid(NEWTILEX,NEWTILEY+1,"S");
		}
		else {
			heroStop();
			is_stopped=1;
		}
	}
	else if (next=="W") {
		if (force_bunny_back && canGotoTile(hero_file,NEWTILEX-2,NEWTILEY,hero_file.getRot().h())) {
			heroGotoGrid(NEWTILEX-2,NEWTILEY,"W");
		}
		else if (canGotoTile(hero_file,NEWTILEX-1,NEWTILEY,hero_file.getRot().h())) {
			heroGotoGrid(NEWTILEX-1,NEWTILEY,"W");
		}
		else {
			heroStop();
			is_stopped=1;
		}
	}
	else if (next=="E") {
		if (force_bunny_back && canGotoTile(hero_file,NEWTILEX+2,NEWTILEY,hero_file.getRot().h())) {
			heroGotoGrid(NEWTILEX-2,NEWTILEY,"E");
		}
		else if (canGotoTile(hero_file,NEWTILEX+1,NEWTILEY,hero_file.getRot().h())) {
			heroGotoGrid(NEWTILEX+1,NEWTILEY,"E");
		}
		else {
			heroStop();
			is_stopped=1;			
		}
	}
	force_bunny_back=0;
	if (is_stopped && forcenext.length()) [hero->resetForceNext()];
	if (reset_next) {
		string dir=[hero->getDir() as string];
		[hero->setNextDir(dir)];
	}
	
}
void driveClunny(vfile _clunny,int _ai) {
	int NEWTILEX,NEWTILEY; 
	float2 p=getTile(_clunny.getPos());
	NEWTILEX=int(p.x());
	NEWTILEY=int(p.y());
	//check for trap
	float2 gotopos=getTile([_clunny.getTag()->getGOTO() as float3]);
	int GOTOTILEX=int(gotopos.x());
	int GOTOTILEY=int(gotopos.y());
	float testangle=_clunny.getRot().h();
	if (GOTOTILEX>NEWTILEX) testangle=270.0;
	else if (GOTOTILEX<NEWTILEX) testangle=90.0;
	else if (GOTOTILEY>NEWTILEY) testangle=180.0;
	
	if (canGotoTile(_clunny,GOTOTILEX,GOTOTILEY,testangle)) {
		if ([_clunny.getTag()->isOnTarget() as int]==0) return;		
	}
	

	if ((NEWTILEX%3)==1 && (NEWTILEY%3)==1) {
		if ( ((_clunny.getPos()).xz()-(itemgrid.getvf(NEWTILEX,NEWTILEy).getPos()).xz()).length()<max(1.0,[_clunny.getTag()->getStepWidth() as float])) {
			string forcedir=[_clunny.getTag()->getForceNext() as string];
			string dir=[_clunny.getTag()->getDir() as string];
			//must turn, find available tiles
			int bturn=canTurn(NEWTILEX,NEWTILEY);
			listvf tiles=listvf();
			if (forcedir.length()) {
				if (forcedir=="N") {
					tiles.push(itemgrid.getvf(NEWTILEX,NEWTILEY-3));
				}
				else if (forcedir=="S") {
					tiles.push(itemgrid.getvf(NEWTILEX,NEWTILEY+3));
				}
				else if (forcedir=="W") {
					tiles.push(itemgrid.getvf(NEWTILEX-3,NEWTILEY));
				}
				else if (forcedir=="E") {
					tiles.push(itemgrid.getvf(NEWTILEX+3,NEWTILEY));
				}
			}
			else {
				if (canGotoTile(_clunny,NEWTILEX+3,NEWTILEY,270.0)) {
					if (posgrid.geti(NEWTILEX+1,NEWTILEY)>0) {
						if (bturn || ((!bturn) && (dir=="E"))) {
							tiles.push(itemgrid.getvf(NEWTILEX+3,NEWTILEY));
						}
					}
				}
				if (canGotoTile(_clunny,NEWTILEX-3,NEWTILEY,90.0)) {
					if (posgrid.geti(NEWTILEX-1,NEWTILEY)>0) {
						if (bturn || ((!bturn) && (dir=="W"))) {
							tiles.push(itemgrid.getvf(NEWTILEX-3,NEWTILEY));
						}
					}
				}
				if (canGotoTile(_clunny,NEWTILEX,NEWTILEY-3,180.0)) {
					if (posgrid.geti(NEWTILEX,NEWTILEY-1)>0) {
						if (bturn || ((!bturn) && (dir=="N"))) {
							tiles.push(itemgrid.getvf(NEWTILEX,NEWTILEY-3));
						}
					}
				}
				if (canGotoTile(_clunny,NEWTILEX,NEWTILEY+3,0.0)) {
					if (posgrid.geti(NEWTILEX,NEWTILEY+1)>0) {
						if (bturn || ((!bturn) && (dir=="S"))) {
							tiles.push(itemgrid.getvf(NEWTILEX,NEWTILEY+3));
						}
					}
				} 
			}
			if (tiles.num()>0) {
				float3 hero_pos=hero_file.getPos();
				int index=0,i,num;
				float dist2,dist=(hero_pos.xz()-tiles.get(0).getPos().xz()).length();
				float herodist=(hero_pos.xz()-_clunny.getPos().xz()).length();
				
				int rndnum=clamp(int(herodist),10,60);
				if (((_ai!=CONST_AI_DONTCARE) && (rand(100)>=rndnum)) || (_ai==CONST_AI_DEFENSIVE)) {
					if (_ai==CONST_AI_DEFENSIVE) {
						//find farthest from bunny
						num=tiles.num();
						for (i=1;i<num;i++) {
							dist2=(hero_pos.xz()-tiles.get(i).getPos().xz()).length();
							if (dist2>dist) {
								dist=dist2;
								index=i;
							}
						}
					}
					else {
						//find closest to bunny
						num=tiles.num();
						for (i=1;i<num;i++) {
							dist2=(hero_pos.xz()-tiles.get(i).getPos().xz()).length();
							if (dist2<dist) {
								dist=dist2;
								index=i;
							}
						}
					}
				}
				else {
					//goto random position
					index=rand(tiles.num());
				}


				//check if previous position is the same
				if (tiles.num()>1) {
					float3 prevpos=[_clunny.getTag()->getPREVGOTO() as float3];
					float3 nextpos=tiles.get(index).getPos();
					if ( int(prevpos.x())==int(nextpos.x()) && int(prevpos.z())==int(nextpos.z()) ) {
						index=rand(tiles.num());
					}
				}
				
				
				//goto to new pos
				float3 newpos=tiles.get(index).getPos();
				newpos.y( _clunny.getPos().y() );
				Event e=Event("GOTO",_clunny.getTag());
				e.addArg(newpos);
				sendEvent(e);
			}
		}
	}
}
void tickEnemies() {
	int bunny_god=[hero->isGod() as int];
	int i,num;
	float dist=1000000.0;
	float3 bunny_pos=hero_file.getPos();
	vfile vf;
	num=l_enemies.num();
	for (i=0;i<num;i++) {
		vf=l_enemies.get(i);
		if (![vf.getTag()->isHit() as int]) {
			dist=min(dist, length( (bunny_pos - vf.getPos() ).xz() ));
		}
		driveClunny(vf,bunny_god);
	}
	if (CLUNNY_AUDIO_CANPLAY && dist<20.0) {
		if (dist<10.0) snd.play("sounds/clunny-near"+(1+rand(2))+".audio");
		else snd.play("sounds/clunny-far"+(1+rand(2))+".audio");
		CLUNNY_AUDIO_CANPLAY=0;
		addListener(ON_TIMEOUT_FRAMES,string((1+rand(3))*getFps()),"cbResetClunnyAudio");
	}
	num=l_hedgehog.num();
	for (i=0;i<num;i++) {
		driveClunny(l_hedgehog.get(i),CONST_AI_DONTCARE);
	}
	num=l_penguin.num();
	for (i=0;i<num;i++) {
		driveClunny(l_penguin.get(i),CONST_AI_DONTCARE);
	}
}
void cbResetClunnyAudio() { CLUNNY_AUDIO_CANPLAY=1; }
void tickUI() {
	float hero_h=hero_file.getRot().h();
	float3 hero_pos=hero_file.getPos();

	//closest power up
	//if (l_pups.num() || statue.isVisible()) {
	if (l_clocks.num()) {
		COMPASSBG.show();
		
		//find closest distance
		float dist=1000000.0;
		float3 pos2 = float3(0,0,0);
		vfile active;
		int i,num;
		num=l_clocks.num();
		for (i=0;i<num;i++) {
			vfile f=l_clocks.get(i);
			float d=length(f.getPos()-hero_pos);
			if (d<dist) {
				active=f;
				dist=d;
				pos2=f.getPos();
			}
		}
		if (statue.isVisible()) {
			if (length(statue.getPos()-hero_pos)<dist) {
				active=statue;
				pos2=statue.getPos();
			}
		}

		float3 dif = pos2-hero_pos;
		dif.y(0.0);		
		dif.normalize();
		float3 vec = float3(0,0,-1);
		vec.roth(hero_h);

		float3 cr = cross(vec,dif);
		float dt = dot(vec,dif);
		float ac = degrees(acos(dt));

		float iconh = ac;
		if (cr.y < 0) iconh = ac*(-1.0);
		COMPASSICON.setparam(PARAM_ROTATION_ANGLE,string(iconh));
	}
	else {
		COMPASSBG.hide();
	}	

	//lesser timer
	int prev_sec=max(0,int(CUR_TIMER/getFPS()));	
	CUR_TIMER=max(0,CUR_TIMER-fElapsed());
	int next_sec=max(0,int(CUR_TIMER/getFPS()));	
	if ((prev_sec!=next_sec) && (next_sec<=5) && (next_sec>0)) {
		 if (next_sec==1) {
			addNotification(string(next_sec)+" second left",1);
		 }
		 else {
			addNotification(string(next_sec)+" seconds left",1);
		 }
		snd.fplay("sounds/timeout.audio"); 		 
	}
	if (prev_sec!=next_sec && next_sec<=10) {
		TIMEBAR.clearAnimate("GB");
		TIMEBAR.animate("GB","0.4:0;1:1000","");
	}
	TIMEFG.setSize(scaleUI(256)*(CUR_TIMER/MAX_TIMER),TIMEFG.getH());
	TIMECOUNT.setParam(PARAM_TEXT,string( max(0,int(CUR_TIMER/getFPS())) ));
	CARROTCOUNT.setparam(PARAM_TEXT,string(SCORE_COUNT));
	
	//update star UI
	if (SCORE_COUNT!=PREV_SCORE_COUNT) {
		if (next_sec>5 && CARROTS_LEFT<=5) {
			if (CARROTS_LEFT==1) {
				addNotification("1 carrot left",1);
			}
			else if (CARROTS_LEFT>1) {
				addNotification(string(CARROTS_LEFT)+" carrots left",1);
			}
		}
	
		PREV_SCORE_COUNT=SCORE_COUNT;
		CARROT_UI_TIMER=0;
		float revealperc=0.0;
		if (SCORE_COUNT>(TOTAL_CARROTS*CONST_PERC_STAR2)) {
			SCORE_SAFE_CAP=(TOTAL_CARROTS*CONST_PERC_STAR2)+1;
			if (!STAR1.isVisible()) {
				snd.fplay("sounds/win2.audio"); 
				STAR2.setOpacity(1.0);
				STAR2.show();
				STAR1.fadeIn(200,"");
				STARFX.animate("W","L0:0;L"+string(scaleUI(400))+":250;L"+string(scaleUI(400))+":1000;L0:1500","");
				STARFX.animate("H","L0:0;L"+string(scaleUI(400))+":250;L"+string(scaleUI(400))+":1000;L0:1500","");
				STARFX.animate("A","L0:0;L1:250;L1:1000;L0:1500","");
				STARFLASH1.animate("A","S0:0;S1:500;S0:1000","");
				STARFLASH2.animate("A","S0:250;S1:750;S0:1250","");
			}
			revealperc=float(SCORE_COUNT-(TOTAL_CARROTS*CONST_PERC_STAR2))/float(TOTAL_CARROTS*(CONST_PERC_STAR3-CONST_PERC_STAR2));
		}
		else if (SCORE_COUNT>(TOTAL_CARROTS*CONST_PERC_STAR1)) {
			SCORE_SAFE_CAP=(TOTAL_CARROTS*CONST_PERC_STAR1)+1;
			if (!STAR2.isVisible()) {
				snd.fplay("sounds/win2.audio"); 
				STAR2.fadeIn(200,"");
				STARFX.animate("W","L0:0;L"+string(scaleUI(400))+":250;L"+string(scaleUI(400))+":1000;L0:1500","");
				STARFX.animate("H","L0:0;L"+string(scaleUI(400))+":250;L"+string(scaleUI(400))+":1000;L0:1500","");
				STARFX.animate("A","L0:0;L1:250;L1:1000;L0:1500","");
				STARFLASH1.animate("A","S0:0;S1:500;S0:1000","");
				STARFLASH2.animate("A","S0:250;S1:750;S0:1250","");
			}
			revealperc=float(SCORE_COUNT-(TOTAL_CARROTS*CONST_PERC_STAR1))/float(TOTAL_CARROTS*(CONST_PERC_STAR2-CONST_PERC_STAR1));
		}
		else {
			SCORE_SAFE_CAP=0;
			revealperc=float(SCORE_COUNT)/float(TOTAL_CARROTS*CONST_PERC_STAR1);
		}
		revealperc=clamp(revealperc,0.0,1.0);
		STARFG.setSize(scaleUI(128),scaleUI(26+70*revealperc));
		STARFG.setPos(0,scaleUI(128)-STARFG.getH());
		STARICON.setPos(0,-STARFG.getY());		
	}

	CARROT_UI_TIMER=min(CARROT_UI_TIMER+fElapsed(),CONST_CARROT_UI_TIMEOUT);
	float eval=CARROT_UI_CURVE.evaluate(CARROT_UI_TIMER);
	CARROTCOUNT.setparam(PARAM_FONTSIZE,string(scaleUIf(eval))+","+scaleUIf(eval));
	
	//power up ui
	if (TIMEOUT_PU1>0.0) {
		TIMEOUT_PU1-=fElapsed();
		if (TIMEOUT_PU1<=0.0) cbTimeoutPU1();
		else {
			if (TIMEOUT_PU1<=60.0) {
				PUSLOT1.setOpacity(float(int((60.0-TIMEOUT_PU1)/10.0)%2)); 				
				PURING1.setOpacity(PUSLOT1.getOpacity());
			}
		}
	}
	if (TIMEOUT_PU2>0.0) {
		TIMEOUT_PU2-=fElapsed();
		if (TIMEOUT_PU2<=0.0) cbTimeoutPU2();
		else {
			if (TIMEOUT_PU2<=60.0) {
				PUSLOT2.setOpacity(float(int((60.0-TIMEOUT_PU2)/10.0)%2)); 				
				PURING2.setOpacity(PUSLOT2.getOpacity());
			}
		}
	}
	if (TIMEOUT_PU3>0.0) {
		TIMEOUT_PU3-=fElapsed();
		if (TIMEOUT_PU3<=0.0) cbTimeoutPU3();
		else {
			if (TIMEOUT_PU3<=60.0) {
				PUSLOT3.setOpacity(float(int((60.0-TIMEOUT_PU3)/10.0)%2)); 				
				PURING3.setOpacity(PUSLOT3.getOpacity());
			}
		}
	}

	//ENEMY COMPASS
	int j,num;
	num=l_enemies.num();
	for (j=0;j<num;j++) {
		float3 epos=l_enemies.get(j).getPos();
		float2 pos=HUD.unproject(epos+float3(0,4,0));
		Widget w=l_wclunny.get(j);
		float dist=length(hero_pos.xz()-epos.xz());
		if ((pos.x>32) && (pos.x<768) && (pos.y>32) && (pos.y<448)) {
			w.setPos(int(pos.x),int(pos.y));
			w.setOpacity(1.0);
			Widget c=l_wclunnycompass.get(j);			
			c.setparam(PARAM_ROTATION_ANGLE,string(45+180));
		}		
		else {
			w.setOpacity(1.0-(clamp( dist-10.0,0.0,60.0)/60.0));
			w.setPos(clamp(int(pos.x),32,768),clamp(int(pos.y),32,448));
			Widget c=l_wclunnycompass.get(j);
			
			float3 dif = epos-hero_pos;
			dif.y(0.0);		
			dif.normalize();
			float3 vec = float3(0,0,-1);
			vec.roth(hero_h);

			float3 cr = cross(vec,dif);
			float dt = dot(vec,dif);
			float ac = degrees(acos(dt));

			float iconh = ac;
			if (cr.y < 0) iconh = ac*(-1.0);
			iconh=normalizeAngle(iconh);
			c.setparam(PARAM_ROTATION_ANGLE,string(iconh+45.0));

			if ((iconh>90) && (iconh<270) && (pos.y<240)) {
				w.setPos(clamp(int(pos.x),32,768),448);
			}
			else if (((iconh<90) || (iconh>270)) && (pos.y>240)) {
				w.setPos(clamp(int(pos.x),32,768),32);
			}
			
			if (w.getY()>384 && w.getX()>704) {
				w.setPos(768,384);
			}
		}
	} 
	
	//end condition
	if (next_sec<=0 || CARROTS_LEFT<=0) {
		debugLog("Level finished");
		
		if (next_sec<=0) {
			TIMEFG.setSize(0,TIMEFG.getH());
		}
		int fastfinishbonus = 0;
		if (CUR_TIMER > 0 ) fastfinishbonus = 2;
		BONUS_TIME= fastfinishbonus + max(0,int( 2* (CUR_TIMER/getFPS())*0.7 ));	
		SCORE_COUNT+=BONUS_TIME;	
		
		//bonus for unused power ups
		int PUSTACK_MULT=["sc.loader"->getPUStackBonus() as int];
		SCORE_COUNT+= ((PU1_STACK+(PU2_STACK/3)+PU3_STACK)*PUSTACK_MULT);
		BONUS_PUP+= ((PU1_STACK+(PU2_STACK/3)+PU3_STACK)*PUSTACK_MULT);
		
		gpad.onButtonDown("");
		clearListeners(ON_TICK);
		addListener(ON_TICK,"tickhud");
		clearListeners(ON_MOUSEDOWN);
		clearListeners(ON_MOUSEUP);
		clearListeners(ON_MOUSEMOTION);
		//stop ai
		int i;
		num=l_enemies.num();
		for (i=0;i<num;i++) {
			l_enemies.get(i).sendEvent("STOP");
		}
		num=l_hedgehog.num();
		for (i=0;i<num;i++) {
			l_hedgehog.get(i).sendEvent("STOP");
		}
		num=l_penguin.num();
		for (i=0;i<num;i++) {
			l_penguin.get(i).sendEvent("STOP");
		}
		//check for success
		if (SCORE_COUNT>=(TOTAL_CARROTS*CONST_PERC_STAR1)) {
			sendEvent("VICTORY",hero);
			addNotification("Level complete!",1);
		}
		else {
			sendEvent("FAIL",hero);
			addNotification("Out of time!",0);
		}		
		addListener(ON_TIMEOUT_FRAMES,"60","cbVictoryOff");	
	}
}
void cbVictoryOff() {
	sendEvent("DEBRIEF","sc.control");
	
	HUD.enableGhost(1);
	[hero->endLevel()];
	addListener(ON_TIMEOUT_MSEC,"1500","cbHide");
}
public void saveState() {
	debugLog("ui.play: Saving state");
	["sc.loader"->setCurrentScore(SCORE_COUNT)];
	["sc.loader"->setCurrentTime(int(CUR_TIMER/getFps()))];
	["sc.loader"->setBonusJump(BONUS_JUMP)];
	["sc.loader"->setBonusTime(BONUS_TIME)];
	["sc.loader"->setBonusPUP(BONUS_PUP)];
	["sc.loader"->setCounterLose(COUNTER_LOSE)];
	["sc.loader"->setStackPU1(PU1_STACK)];
	["sc.loader"->setStackPU2(PU2_STACK)];
	["sc.loader"->setStackPU3(PU3_STACK)];
	["sc.loader"->setTimeoutPU1(TIMEOUT_PU1)];
	["sc.loader"->setTimeoutPU2(TIMEOUT_PU2)];
	["sc.loader"->setTimeoutPU3(TIMEOUT_PU3)];
	["sc.loader"->setEggCarrotsInRow(EGG_CARROTS_INROW)];
	["sc.loader"->setEggJumpsInRow(EGG_JUMPS_INROW)];
	["sc.loader"->setEggSquashInRow(EGG_SQUASH_INROW)];

	appPropertySet(APPNAME,"state","");
	appPropertyFlush();	
}
public void debriefSave() {
	enableTick(0);
	debugLog("ui.play: Preparing for debrief");
	//inform loader about current score
	saveState();
	
	analytics_trackevent("time",["sc.loader"->getTheme() as string],["sc.loader"->getLevel() as string],playTimer.getMSec()/1000);
	if (["sc.loader"->getPick_SHOES() as int]) analytics_trackevent("powerup","pick","shoes",["sc.loader"->getPick_SHOES() as int]);
	if (["sc.loader"->getPick_POTION() as int]) analytics_trackevent("powerup","pick","invisibility",["sc.loader"->getPick_POTION() as int]);
	if (["sc.loader"->getPick_ROCK() as int]) analytics_trackevent("powerup","pick","rocknroll",["sc.loader"->getPick_ROCK() as int]);
	if (["sc.loader"->getPick_CLOCK() as int]) analytics_trackevent("powerup","pick","clock",["sc.loader"->getPick_CLOCK() as int]);
	if (["sc.loader"->getPick_CHEST() as int]) analytics_trackevent("powerup","pick","chest",["sc.loader"->getPick_CHEST() as int]);
	if (["sc.loader"->getUse_SHOES() as int]) analytics_trackevent("powerup","use","shoes",["sc.loader"->getUse_SHOES() as int]);
	if (["sc.loader"->getUse_POTION() as int]) analytics_trackevent("powerup","use","invisibility",["sc.loader"->getUse_POTION() as int]);
	if (["sc.loader"->getUse_ROCK() as int]) analytics_trackevent("powerup","use","rocknroll",["sc.loader"->getUse_ROCK() as int]);
	
	//check for success
	if (SCORE_COUNT>=(TOTAL_CARROTS*CONST_PERC_STAR1)) {
		if (SCORE_COUNT>=(TOTAL_CARROTS*CONST_PERC_STAR3)) {
			analytics_trackview("/play/"+["sc.loader"->getTheme() as string]+"/"+["sc.loader"->getLevel() as string]+"/end_3");
		}
		else if (SCORE_COUNT>=(TOTAL_CARROTS*CONST_PERC_STAR2)) {
			analytics_trackview("/play/"+["sc.loader"->getTheme() as string]+"/"+["sc.loader"->getLevel() as string]+"/end_2");
		}
		else {
			analytics_trackview("/play/"+["sc.loader"->getTheme() as string]+"/"+["sc.loader"->getLevel() as string]+"/end_1");
		}
		["sc.loader"->e_SaveLevelScore()];
	}
	else {
		analytics_trackview("/play/"+["sc.loader"->getTheme() as string]+"/"+["sc.loader"->getLevel() as string]+"/end_0");
	}
}

//////// TOOLS ///////

public float3 findCorridorEnd(float3 _startpos,string _dir) {
	float2 start_tile=getTile(_startpos);
	int tx=int(start_tile.x());
	int ty=int(start_tile.y());
	int nx=tx,ny=ty;
	//find next tile
	if (_dir=="N") {
		while(posgrid.geti(nx,ny-1)) {
			ny--;
		}
	}
	else if (_dir=="S") {
		while(posgrid.geti(nx,ny+1)) {
			ny++;
		}
	}
	else if (_dir=="W") {
		while(posgrid.geti(nx-1,ny)) {
			nx--;
		}
	}
	else if (_dir=="E") {
		while(posgrid.geti(nx+1,ny)) {
			nx++;
		}
	}
	return itemgrid.getvf(nx,ny).getPos();
}
/////// STATS ///////

public float3 getAvailableDragonSlot() {
	l_dragonslots.shuffle();
	if (l_dragonslots.num()) {
		return l_dragonslots.get(0).getPos();
	}
	return float3(-1,-1,-1);
}

public void switchToSnapdragon() {
	carrots.hide();
	dragon_coins.show();
}
public int showController() {
	if (!show_controller) {
		show_controller=1;
		PUCONTROLLER1.setParam(PARAM_IMAGE,"LINEAR:textures/ui/controller_buttonx.image");	
		PUCONTROLLER2.setParam(PARAM_IMAGE,"LINEAR:textures/ui/controller_buttony.image");
		PUCONTROLLER3.setParam(PARAM_IMAGE,"LINEAR:textures/ui/controller_buttonb.image");			
	}
	return 1;
}
